/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package actions;

// Modifi verssion 4.6.3 pour associer une icne  l'outil
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.beans.PropertyVetoException;
import java.io.IOException;
import dialogs.NouvelleFigureAvecRepereDlg;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import mathgraph32.MtgMenuFileItem;
import objets.CCalcul;
import objets.CConstante;
import objets.CElementGenerique;
import objets.CImplementationProto;
import objets.CListeObjets;
import objets.CMathGrafDoc;
import objets.CPrototype;
import types.UniteAngle;

@SuppressWarnings("serial")
public class ActionFichierNouvelleFigureAvecRepere extends MtgAction {
	public ActionFichierNouvelleFigureAvecRepere(MtgFrame frameInit) {
		super(frameInit, null, getString("menuNouvelleFigureAvecRepere"),
				getString("tipNouvelleFigureAvecRepere"), iconeAssociee(), null, true);
	}
	
  @Override
  public int idMenu() {
    return 33013;
  }	
	
  // Ajout version 4.6.3
  public static String iconeAssociee() {
    return "images/RepQuadrille";
  }

	@Override
	public void actionPerformed(ActionEvent e) {
		// MtgInternalFrame cadreActif = frame.cadreActif();
		// if (cadreActif != null) cadreActif.cacheDialogueAffichageVariables();
		CMathGrafDoc doc = new CMathGrafDoc(frame);
		doc.listePrincipale.ajouteConstantePi();
		doc.listePrincipale.uniteAngle = frame.pref_angleUnit;
		NouvelleFigureAvecRepereDlg dlg = new NouvelleFigureAvecRepereDlg(frame, doc.listePrincipale);
		dlg.setLocationRelativeTo(frame);
		dlg.setVisible(true);
		if (dlg.retourOK()) {
			// Modification version 3.5 : Si la prcdente fentre a t maximise, on maximise la nouvelle
			MtgInternalFrame cadreActif = frame.cadreActif();
			boolean maximise = cadreActif == null ? false : cadreActif.isMaximum();
			MtgInternalFrame cadre = new MtgInternalFrame(frame, "MathGraph" + Integer.toString(frame.instanceNouvelleFenetre()), doc,
					MtgInternalFrame.largeurNouvelleFigure, MtgInternalFrame.hauteurNouvelleFigure);
			Dimension sizefen = new Dimension(cadre.getWidth(), cadre.getHeight());
			try {
				cadre.gestionnaire.initialise();
			} catch (IOException e1) {}
			// frame.metAJourIconesEtMenus(cadre.document);  // Version 4.1.2 : Inutile car fait dans MtgInternalFrame.internalFrameActivated
			frame.mtgDesktopPane.add(cadre);
			/*mtgDesktopPane.setSelectedFrame(cadre);
	    try {
	      cadre.setSelected(true);
	    } catch (java.beans.PropertyVetoException err) {};	          
			*/
			// Modification version 3.5.1
			frame.mtgDesktopPane.getDesktopManager().activateFrame(cadre);
			try {
				cadre.setSelected(true);
			} catch (PropertyVetoException e1) {
			}
			if (maximise) {
				try {
					cadre.setMaximum(true);
				} catch (PropertyVetoException e1) {
					frame.mtgDesktopPane.getDesktopManager().maximizeFrame(cadre);
				}
			}
			else {
				frame.preparePositionPourFenenetre(sizefen);
				cadre.setLocation(frame.xfenetre, frame.yfenetre);
			}
			frame.activeOutilCapture();
			if (dlg.rbGraduationSimple.isSelected()) {
				CListeObjets liste = cadre.listePr;
	  		String nom1 = "nbgradx";
	  		CCalcul calcul1 = new CCalcul(liste, null, false, nom1, "20", new CConstante(liste, 20));
	  		frame.ajouteElement(calcul1);
	  		String nom2 = "nbgrady";
	  		CCalcul calcul2 = new CCalcul(liste, null, false, nom2, "20", new CConstante(liste, 20));
	  		frame.ajouteElement(calcul2);
	    	CPrototype proto = MtgFrame.screenIsHighResolution ? frame.chargeConstructionDansDossierMtgFrame("GraduationAxesRepereHD.mgc") :
	    		frame.chargeConstructionDansDossierMtgFrame("GraduationAxesRepere.mgc");
	    	//On implmente maintenant ce prototype dans la figure
	  		((CElementGenerique)proto.get(0)).elementAssocie = liste.premierRepere();
	  		((CElementGenerique)proto.get(1)).elementAssocie = calcul1;
	  		((CElementGenerique)proto.get(2)).elementAssocie = calcul2;
	  		CImplementationProto impProto = new CImplementationProto(liste, proto);
	  		impProto.implemente(liste, proto, cadre, 0); // Modifi version 4.8
	  		frame.cadreActif().redessineAvecCalcul();
			}
			else {
				if (dlg.rbGraduationTrigo.isSelected()) {
					CListeObjets liste = cadre.listePr;
					liste.uniteAngle = UniteAngle.Radian;
		  		String nom1 = "nbgradx";
		  		CCalcul calcul1 = new CCalcul(liste, null, false, nom1, "20", new CConstante(liste, 20));
		  		frame.ajouteElement(calcul1);
		  		String nom2 = "nbgrady";
		  		CCalcul calcul2 = new CCalcul(liste, null, false, nom2, "20", new CConstante(liste, 20));
		  		frame.ajouteElement(calcul2);
		    	CPrototype proto = MtgFrame.screenIsHighResolution ? frame.chargeConstructionDansDossierMtgFrame("GraduationReperePourTrigoHD.mgc")
		    			: frame.chargeConstructionDansDossierMtgFrame("GraduationReperePourTrigo.mgc");
		    	//On implmente maintenant ce prototype dans la figure
		  		((CElementGenerique)proto.get(0)).elementAssocie = liste.premierRepere();
		  		((CElementGenerique)proto.get(1)).elementAssocie = calcul1;
		  		((CElementGenerique)proto.get(2)).elementAssocie = calcul2;
		  		CImplementationProto impProto = new CImplementationProto(liste, proto);
		  		impProto.implemente(liste, proto, cadre, 0); // Modifi version 4.8
		  		frame.cadreActif().redessineAvecCalcul();
					
				}
			}
			try {
				cadre.gestionnaire.initialise();
			} catch (IOException e1) {}
			// On cre si ncessaire un item de menu dans le menu Fenetres pour activer cete fentre
			int nfen = frame.nombreCadresOuverts();
			if (nfen <= MtgFrame.nombreFenetresDansMenu) {
				MtgMenuFileItem item = new MtgMenuFileItem(cadre);
				frame.menuFenetre.add(item);
			}
			else if (nfen == MtgFrame.nombreFenetresDansMenu + 1) frame.menuFenetre.add(frame.itemChoixFenetre);
		}
		else {
			doc.listePrincipale = null;
			doc = null;
			System.gc();
		}
	}
}
