/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package actions;

import glob.Kernel;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.beans.PropertyVetoException;
import java.io.ByteArrayInputStream;
import objets.CMathGrafDoc;
import types.NatureObjet;
import dialogs.FigureParCodeDlg;
import entreesSorties.MathGraphDataInputStream;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import mathgraph32.MtgMenuFileItem;

@SuppressWarnings("serial")
public class ActionFichierNouvelleFigureParCode extends MtgAction {
	public ActionFichierNouvelleFigureParCode(MtgFrame frameInit) {
		super(frameInit, null, getString("menuNouvelleFigureParCode"), getString("menuNouvelleFigureParCode"), null, null, false);
	}
	public void actionPerformed(ActionEvent e) {
		MtgInternalFrame cadre;
		MathGraphDataInputStream inps = null;
		FigureParCodeDlg dlg= new FigureParCodeDlg(frame);
		dlg.setLocationRelativeTo(frame);
		dlg.setVisible(true);
		if(dlg.retourOK()) {
			String code = dlg.code;
			try {
				ByteArrayInputStream ba = new ByteArrayInputStream(Kernel.base64Decode(code));
				inps = new MathGraphDataInputStream(ba);
				CMathGrafDoc doc = new CMathGrafDoc(frame);
				doc.chargeDepuisFlux(inps);
				cadre = new MtgInternalFrame(frame, "MathGraph" + Integer.toString(frame.instanceNouvelleFenetre()), doc,
						MtgInternalFrame.largeurNouvelleFigure, MtgInternalFrame.hauteurNouvelleFigure);
				// cadre.setTitle(titre);
				// cadre.path = file.getAbsolutePath();
				cadre.gestionnaire.initialise();
				// cadre.setVisible(true);
				frame.mtgDesktopPane.add(cadre);
				inps.close();
				frame.mtgDesktopPane.getDesktopManager().activateFrame(cadre);
				try {
					cadre.setSelected(true);
				} catch (PropertyVetoException e1) {
				}
				Dimension sizefen = new Dimension(doc.sizeWindowx, doc.sizeWindowy);
				cadre.paneCenter.setPreferredSize(sizefen); // Modifi version 3.5
				cadre.pack();
				doc.sizeWindowx = cadre.paneFigure.getWidth();
				doc.sizeWindowy = cadre.paneFigure.getHeight();
				int width = cadre.getWidth();
				int height = cadre.getHeight();
				boolean b = frame.preparePositionPourFenenetre(new Dimension(width, height));
				if (b) {
					cadre.reshape(frame.xfenetre, frame.yfenetre, width, height);
				}
				else {
					try {
						cadre.setMaximum(true);
					} catch (PropertyVetoException e1) {
						frame.mtgDesktopPane.getDesktopManager().maximizeFrame(cadre);
					}
				}
				if (frame.outilActif != null) frame.outilActif.select(); // Ncessaire dans le cas o des objets sont fichs
				// On cre si ncessaire un item de menu dans le menu Fenetres pour activer cete fentre
				int nfen = frame.nombreCadresOuverts();
				if (nfen <= MtgFrame.nombreFenetresDansMenu) {
					MtgMenuFileItem item = new MtgMenuFileItem(cadre);
					frame.menuFenetre.add(item);
				}
				else if (nfen == MtgFrame.nombreFenetresDansMenu + 1) frame.menuFenetre.add(frame.itemChoixFenetre);
				// Les deux lignes suivantes rajoutes version 4.4
				if (cadre.listePr.nombreObjetsParNature(NatureObjet.NMacro) != 0) frame.activeOutilExecutionMacro();
				else frame.activeOutilCapture();
			} catch (Exception err) {
				frame.dialogueAvertissement(getString("ch35"));
				return;
			}
		}
	}

}
