/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.Vector;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import objets.CCommentaire;
import objets.CListeObjets;
import types.NatureCarSpe;
import mathgraph32.MtgDesktopPane;
import mathgraph32.MtgFrame;

// Modifi version 5.0 : Plis de boutons pour LaTeX mais des boutons pour le changement de style
@SuppressWarnings("serial")
public class CommentaireDlg extends MtgDlg {
	private boolean modification; // True si on modifie les coordonnes d'un commentaire dj cr
	private CCommentaire commentaire;
	private CCommentaire comClign;
	private String texteInitial;
	private int indiceMax;
	private MtgLabel label1;
	private MtgButton btnInsertionAffichageDynamique;
  private MtgButton btnCancel;
  private MtgButton btnOK;
  private PaneTaillePolice paneTaillePolice;
  private PaneModeAffichage paneModeAffichage;
  private TextAreaAvecCarSpeciaux editeurTexteCommentaire;
  private MtgFocusTraversalPolicy mtgFocusTraversalPolicy;

	public CommentaireDlg(MtgFrame frameInit, CCommentaire commentaireInit, boolean modificationInit,
			CCommentaire comClignInit)	{
		super(getString("dlgCommentaire1"), true, frameInit);
		commentaire = commentaireInit;
		modification = modificationInit;
		comClign = comClignInit;
    CListeObjets liste = frame.cadreActif().listePr;
    // Dans le cas ou on cre un nouvel objet, celui-ci n'a pas encore t ajout
    //  la liste des objets de l'applet.
    if (modification) {
    	indiceMax = liste.indexOf(commentaireInit) - 1;
    	texteInitial = commentaire.chaineCommentaire;
    }
    else indiceMax = liste.size() - 1;
 		editeurTexteCommentaire = new TextAreaAvecCarSpeciaux(this, frame, NatureCarSpe.Tout);
 		editeurTexteCommentaire.setRows(8);
 		editeurTexteCommentaire.setColumns(45);
    JScrollPane scrollEditeurTexte = new JScrollPane
  		(editeurTexteCommentaire, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    if (modification) editeurTexteCommentaire.setText(commentaire.chaineCommentaire);

    // On prpare les boutons spciaux latex et les deux barre qui les contiendront
		JToolBar bar1 = new JToolBar(JToolBar.HORIZONTAL);
		bar1.setBorder(BorderFactory.createEmptyBorder());
		/* Abandonn version 5.0 : On dcourage l'emploi du LaTeX dans les affichages de textes car pas support par l'export html
		MtgLatexButtonPourCommentaire buttonQuotient = new MtgLatexButtonPourCommentaire("LatexQuotient", "\\frac{ }{ }", 6, editeurTexteCommentaire);
		bar1.add(buttonQuotient);
		MtgLatexButtonPourCommentaire buttonRacine = new MtgLatexButtonPourCommentaire("LatexRacine", "\\sqrt{}", 6, editeurTexteCommentaire);
		bar1.add(buttonRacine);
		MtgLatexButtonPourCommentaire buttonRacineN = new MtgLatexButtonPourCommentaire("LatexRacineN", "\\sqrt [ ] { }", 7, editeurTexteCommentaire);
		bar1.add(buttonRacineN);
		MtgLatexButtonPourCommentaire buttonVecteur = new MtgLatexButtonPourCommentaire("LatexVecteur", "\\overrightarrow { }", 17, editeurTexteCommentaire);
		bar1.add(buttonVecteur);
		MtgLatexButtonPourCommentaire buttonMesureAlgebrique = new MtgLatexButtonPourCommentaire("LatexMesureAlgebrique", "\\overline { }", 11, editeurTexteCommentaire);
		bar1.add(buttonMesureAlgebrique);
		MtgLatexButtonPourCommentaire buttonSousligne = new MtgLatexButtonPourCommentaire("LatexSouligne", "\\underline { }", 12, editeurTexteCommentaire);
		bar1.add(buttonSousligne);
		MtgLatexButtonPourCommentaire buttonIntegrale = new MtgLatexButtonPourCommentaire("LatexIntegrale", "\\int_{ }^{ } ", 6, editeurTexteCommentaire);
		bar1.add(buttonIntegrale);
		MtgLatexButtonPourCommentaire buttonSigma = new MtgLatexButtonPourCommentaire("LatexSigma", "\\sum_{ }^{ }", 6, editeurTexteCommentaire);
		bar1.add(buttonSigma);
		JToolBar bar2 = new JToolBar(JToolBar.HORIZONTAL);		
		bar2.setBorder(BorderFactory.createEmptyBorder());	
		MtgLatexButtonPourCommentaire buttonParouv = new MtgLatexButtonPourCommentaire("LatexPar", "\\left( { } \\right)", 8, editeurTexteCommentaire);
		bar2.add(buttonParouv);
		MtgLatexButtonPourCommentaire buttonCrochet = new MtgLatexButtonPourCommentaire("LatexCrochet", "\\left[ { } \\right]", 8, editeurTexteCommentaire);
		bar2.add(buttonCrochet);
		MtgLatexButtonPourCommentaire buttonAccolade = new MtgLatexButtonPourCommentaire("LatexAccolade", "\\left\\{ { } \\right.", 9, editeurTexteCommentaire);
		bar2.add(buttonAccolade);
		// MtgLatexButtonPourCommentaire buttonAccoladeDouble = new MtgLatexButtonPourCommentaire("LatexAccoladeDouble", "\\left\\{ { } \\right}", 9, editeurTexteCommentaire); // Rectifi version 4.9.3
		MtgLatexButtonPourCommentaire buttonAccoladeDouble = new MtgLatexButtonPourCommentaire("LatexAccoladeDouble", "\\left\\{  \\right\\}", 8, editeurTexteCommentaire);
		
		bar2.add(buttonAccoladeDouble);
		MtgLatexButtonPourCommentaire buttonNorme = new MtgLatexButtonPourCommentaire("LatexNorme", "\\left\\| { } \\right\\|", 9, editeurTexteCommentaire);
		bar2.add(buttonNorme);
		// On met un array align  gauche pour uen matrice colonne
		MtgLatexButtonPourCommentaire buttonMatrice21 = new MtgLatexButtonPourCommentaire("LatexMatrice21", "\\begin{array}{l} { } \\\\ { } \\end{array}", 18, editeurTexteCommentaire);
		bar2.add(buttonMatrice21);
		MtgLatexButtonPourCommentaire buttonMatrice12 = new MtgLatexButtonPourCommentaire("LatexMatrice12", "\\begin{matrix} { } & { } \\end{matrix}", 16, editeurTexteCommentaire);
		bar2.add(buttonMatrice12);
		MtgLatexButtonPourCommentaire buttonMatrice22 = new MtgLatexButtonPourCommentaire("LatexMatrice22", "\\begin{matrix} { } & { } \\\\ { } & { } \\end{matrix}", 16, editeurTexteCommentaire);
		bar2.add(buttonMatrice22);
		JToolBar bar3 = new JToolBar(JToolBar.HORIZONTAL);		
		bar3.setBorder(BorderFactory.createEmptyBorder());	
		MtgLatexButtonPourCommentaire buttonAngle = new MtgLatexButtonPourCommentaire("LatexAngle", "\\widehat { }", 10, editeurTexteCommentaire);
		bar3.add(buttonAngle);
		MtgLatexButtonPourCommentaire buttonLimite = new MtgLatexButtonPourCommentaire("LatexLimite", "\\lim_{x \\to { }}", 13, editeurTexteCommentaire);
		bar3.add(buttonLimite);
		MtgLatexButtonPourCommentaire buttonSin = new MtgLatexButtonPourCommentaire("LatexSin", "\\sin { }", 6, editeurTexteCommentaire);
		bar3.add(buttonSin);
		MtgLatexButtonPourCommentaire buttonCos = new MtgLatexButtonPourCommentaire("LatexCos", "\\cos { }", 6, editeurTexteCommentaire);
		bar3.add(buttonCos);
		MtgLatexButtonPourCommentaire buttonTan = new MtgLatexButtonPourCommentaire("LatexTan", "\\tan { }", 6, editeurTexteCommentaire);
		bar3.add(buttonTan);
		MtgLatexButtonPourCommentaire buttonExp = new MtgLatexButtonPourCommentaire("LatexExp", "\\mathrm{e}^{ }", 12, editeurTexteCommentaire);
		bar3.add(buttonExp);
		MtgLatexButtonPourCommentaire buttonLn = new MtgLatexButtonPourCommentaire("LatexLn", "\\ln{ }", 4, editeurTexteCommentaire);
		bar3.add(buttonLn);
		MtgLatexButtonPourCommentaire buttonTexte = new MtgLatexButtonPourCommentaire("LatexTexte", "\\text{}", 6, editeurTexteCommentaire);
		bar3.add(buttonTexte);
		*/
		MtgLatexButtonPourCommentaire buttonExposant = new MtgLatexButtonPourCommentaire("StyleTexteExposant", "#H()", 3, editeurTexteCommentaire);
		buttonExposant.setToolTipText(getString("exposant"));
		bar1.add(buttonExposant);
		MtgLatexButtonPourCommentaire buttonIndice = new MtgLatexButtonPourCommentaire("StyleTexteIndice", "#L()", 3, editeurTexteCommentaire);
		buttonIndice.setToolTipText(getString("indice"));
		bar1.add(buttonIndice);
		MtgLatexButtonPourCommentaire buttonNormal = new MtgLatexButtonPourCommentaire("StyleTexteNormal", "#N", 2, editeurTexteCommentaire);
		buttonNormal.setToolTipText(getString("normal"));
		bar1.add(buttonNormal);
		MtgLatexButtonPourCommentaire buttonGras = new MtgLatexButtonPourCommentaire("StyleTexteGras", "#G", 2, editeurTexteCommentaire);
		buttonGras.setToolTipText(getString("gras"));
		bar1.add(buttonGras);
		MtgLatexButtonPourCommentaire buttonItalique = new MtgLatexButtonPourCommentaire("StyleTexteItalique", "#I", 2, editeurTexteCommentaire);
		buttonItalique.setToolTipText(getString("italique"));
		bar1.add(buttonItalique);
		MtgLatexButtonPourCommentaire buttonSouligne = new MtgLatexButtonPourCommentaire("StyleTexteSouligne", "#U", 2, editeurTexteCommentaire);
		buttonSouligne.setToolTipText(getString("souligne"));
		bar1.add(buttonSouligne);

 		label1 = new MtgLabel(frame, getString("dlgCommentaire2"));
    btnOK = new MtgButton(frame, getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        btnOKActionPerformed();
      }
    });
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());
    paneTaillePolice = new PaneTaillePolice(commentaireInit.indiceTaillePolice);
    paneTaillePolice.listeTailles.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				metAJourCommentaireClignotant();
			}
		});
    paneModeAffichage = new PaneModeAffichage(frame, commentaireInit, true); // Modifi version 4.5.2
    paneModeAffichage.checkEffacementFond.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				metAJourCommentaireClignotant();
			}
		});
    paneModeAffichage.radioEncadrement3D.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				metAJourCommentaireClignotant();
			}
		});
    paneModeAffichage.radioEncadrementSimple.addActionListener(new ActionListener() {	
			public void actionPerformed(ActionEvent e) {
				metAJourCommentaireClignotant();	
			}
		});
    paneModeAffichage.radioSansEncadrement.addActionListener(new ActionListener() {			
			public void actionPerformed(ActionEvent e) {
				metAJourCommentaireClignotant();
			}
		});
    // Ajout version 4.7.1.1
    paneModeAffichage.rbAligHorCentre.addActionListener(new ActionListener() {			
			public void actionPerformed(ActionEvent e) {
				// Ajout version 4.7.7
				comClign.dec_x = 0;
				comClign.dec_y = 0;
				//
				metAJourCommentaireClignotant();
			}
		});
    paneModeAffichage.rbAligHorDroit.addActionListener(new ActionListener() {			
			public void actionPerformed(ActionEvent e) {
				// Ajout version 4.7.7
				comClign.dec_x = 0;
				comClign.dec_y = 0;
				//
				metAJourCommentaireClignotant();
			}
		});
    paneModeAffichage.rbAligHorGauche.addActionListener(new ActionListener() {			
			public void actionPerformed(ActionEvent e) {
				// Ajout version 4.7.7
				comClign.dec_x = 0;
				comClign.dec_y = 0;
				//
				metAJourCommentaireClignotant();
			}
		});
    paneModeAffichage.rbAligVertBas.addActionListener(new ActionListener() {			
			public void actionPerformed(ActionEvent e) {
				// Ajout version 4.7.7
				comClign.dec_x = 0;
				comClign.dec_y = 0;
				//
				metAJourCommentaireClignotant();
			}
		});
    paneModeAffichage.rbAligVertCentre.addActionListener(new ActionListener() {			
			public void actionPerformed(ActionEvent e) {
				// Ajout version 4.7.7
				comClign.dec_x = 0;
				comClign.dec_y = 0;
				//
				metAJourCommentaireClignotant();
			}
		});
    paneModeAffichage.rbAligVertHaut.addActionListener(new ActionListener() {			
			public void actionPerformed(ActionEvent e) {
				// Ajout version 4.7.7
				comClign.dec_x = 0;
				comClign.dec_y = 0;
				//
				metAJourCommentaireClignotant();
			}
		});
    
		// Fin ajout
 		editeurTexteCommentaire.addKeyListener(new KeyListener() {
			public void keyTyped(KeyEvent e) {
				// TODO Auto-generated method stub
			}
			
			public void keyReleased(KeyEvent e) {
				// Seul keyReleased tient compte du dernier caractre press
				metAJourCommentaireClignotant();
			}
			public void keyPressed(KeyEvent e) {
				// TODO Auto-generated method stub
				
			}
		});
    
    paneModeAffichage.colorButton.associeA(comClign);
    
 		btnInsertionAffichageDynamique = new MtgButton(frame, getString("btnInsertionValeurDynamique"));
 		btnInsertionAffichageDynamique.addActionListener(new ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnInsertionAffichageDynamiqueActionPerformed();
      }
 		});
 		
    JPanel pane = new JPanel();
    GridBagLayout gridBag = new GridBagLayout();    
    pane.setLayout(gridBag);
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.top = 4;
    constraints.insets.bottom = 4;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    setConstraints(constraints, 0, 0, 2, 1, 80, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(bar1, constraints);
    /* Supprim version 5.0
    setConstraints(constraints, 0, 1, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(bar2, constraints);
    setConstraints(constraints, 0, 2, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(bar3, constraints);
    */
    
    setConstraints(constraints, 0, 1, 2, 1, 80, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label1, constraints);
    constraints.insets.top = 2;
    setConstraints(constraints, 0, 2, 2, 1, 0, 50, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    pane.add(scrollEditeurTexte, constraints);
    constraints.insets.top = 4;
    setConstraints(constraints, 0, 3, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.CENTER);
    pane.add(btnInsertionAffichageDynamique, constraints);
    setConstraints(constraints, 0, 4, 1, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(paneTaillePolice, constraints);
    setConstraints(constraints, 1, 4, 2, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(paneModeAffichage, constraints);
    JPanel paneRightHigh = new JPanel();
    paneRightHigh.setLayout(new BoxLayout(paneRightHigh, BoxLayout.Y_AXIS));
    paneRightHigh.add(btnOK);
		paneRightHigh.add(Box.createRigidArea(new Dimension(0, 5))); 		
    paneRightHigh.add(btnCancel);
    setConstraints(constraints, 2, 0, 1, 4, 20, 50, GridBagConstraints.NONE, GridBagConstraints.NORTHEAST);
    pane.add(paneRightHigh, constraints);
    setContentPane(pane);
    getRootPane().setDefaultButton(btnOK);    

    // On change l'ordre par dfaut des composants
    Vector<Component> order = new Vector<Component>(3);
    order.add(editeurTexteCommentaire);
    order.add(btnOK);
    order.add(btnCancel);
    mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
    setFocusTraversalPolicy(mtgFocusTraversalPolicy);
    editeurTexteCommentaire.requestFocus();
    pack();
	}

  private void btnCancelActionPerformed() {
  	if (modification) {
  		commentaire.chaineCommentaire = texteInitial;
  		commentaire.determineDependances();
			commentaire.positionne(false, frame.cadreActif().dimf);
  	}
  	dispose();
  }

  private void btnOKActionPerformed() {  
  	if (editeurTexteCommentaire.getText().length() == 0) {
  		dialogueAvertissement(getString("dlgCommentaire3"));
  		editeurTexteCommentaire.requestFocus();
  	}
  	else {
  		commentaire.chaineCommentaire = editeurTexteCommentaire.getText();
  		commentaire.indiceTaillePolice = paneTaillePolice.indiceTaille();
  		commentaire.encadrement = paneModeAffichage.styleEncadrement();
  		commentaire.effacementFond = paneModeAffichage.effacementFond();
  		commentaire.couleurFondAffichage = paneModeAffichage.couleurFond();
  		// Ajouts version 4.5
  		commentaire.alignementHorizontal = paneModeAffichage.alignementHorizontal();
  		commentaire.alignementVertical = paneModeAffichage.alignementVertical();
  		//
			// Ajout version 4.7.7
  		commentaire.dec_x = comClign.dec_x;
  		commentaire.dec_y = comClign.dec_y;
			//

  		frame.pref_IndiceTaillePoliceAffichages = commentaire.indiceTaillePolice;
  		frame.cadreActif().couleurFondAffichage = commentaire.couleurFondAffichage;
  		commentaire.determineDependances();
  		setRetourOK(true);
  		dispose();
  	}
  }
  
  private void btnInsertionAffichageDynamiqueActionPerformed() {
  	InsertionDynamiqueDansCommentaireDlg dlg = new InsertionDynamiqueDansCommentaireDlg(frame, indiceMax, editeurTexteCommentaire);
  	dlg.setLocationRelativeTo(this);
  	dlg.setVisible(true);
  }
  
  private void metAJourCommentaireClignotant() {
		comClign.chaineCommentaire = editeurTexteCommentaire.getText();
		comClign.indiceTaillePolice = paneTaillePolice.indiceTaille();
		comClign.encadrement = paneModeAffichage.styleEncadrement();
		comClign.effacementFond = paneModeAffichage.effacementFond();
		// Ajout version 4.7.1.1
		comClign.alignementHorizontal = paneModeAffichage.alignementHorizontal();
		comClign.alignementVertical = paneModeAffichage.alignementVertical();
		// Fin ajout
		comClign.determineDependances();
		comClign.positionne(false, frame.cadreActif().dimf);
		frame.cadreActif().redessineSansCalcul();  	
  }
  
  /**
   * Version 3.2.9 : on redfinit setVisible pour que le dialogue se positionne automatiquement
   * en bas et  droite de la fentre principale.
   */
  @Override
  public void setVisible(boolean b) {
  	MtgDesktopPane mtgdp = frame.mtgDesktopPane;
  	Dimension s = mtgdp.getSize();
  	Dimension d = getSize();
  	Point p = mtgdp.getLocation();
  	setLocation(p .x + s.width - d.width, p.y + s.height + frame.barHor.getHeight() - d.height);
  	super.setVisible(b);
  }
  
  @Override
  public void metAJourAffichageClignotant() {
  	metAJourCommentaireClignotant();
  }
  
	public boolean estCentreSurFenetre() {
		return false;
	}
  
}
