/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import objets.CDerivee;
import objets.CFonc;
import objets.CListeObjets;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

@SuppressWarnings("serial")
public class DeriveeDlg extends MtgDlg {
	
	private boolean modification;
	private CDerivee deriveeAModifier;
	private MtgLabel label1, label2, label3;
	private MtgTextField editeurNom , info;
  private MtgButton btnOK, btnCancel;  
  private ListeFonctionsReellesSansDerivees listeFonctions;
  
  public DeriveeDlg(MtgFrame frameInit, CDerivee deriveeAModifierInit, boolean modificationInit) {
  	super(getString("deriveeDlg1"), true, frameInit);
  	modification = modificationInit;
  	deriveeAModifier = deriveeAModifierInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK);      	
  }
  private void initComponents() {
    label1 = new MtgLabel(frame, getString("deriveeDlg2"), JLabel.RIGHT);
    label1.setRequestFocusEnabled(false);
    label2 = new MtgLabel(frame, getString("deriveeDlg3"), JLabel.RIGHT);
    label2.setRequestFocusEnabled(false);
    label3 = new MtgLabel(frame, getString("informationFonction"), JLabel.RIGHT);
    label3.setRequestFocusEnabled(false);
    editeurNom = new MtgTextField(frame, 8);
    info = new MtgTextField(frame);
    info.setEditable(false);
    info.setColumns(40);
    
    CListeObjets liste = frame.cadreActif().listePr;
    int indmax;
    if (modification) {
    	indmax = liste.indexOf(deriveeAModifier) - 1;
    	listeFonctions = new ListeFonctionsReellesSansDerivees(frame, liste, indmax, deriveeAModifier.fonctionAssociee);
    }
    else {
    	indmax = liste.size() - 1;
    	listeFonctions = new ListeFonctionsReellesSansDerivees(frame, liste, indmax, null);
    }
    listeFonctions.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent evt) {
      	listeFonctionsSelectionChanged();
      }
    });
    if (modification)  {
    	editeurNom.setEditable(false);
    	editeurNom.setText(deriveeAModifier.nom());
    	listeFonctions.setSel(deriveeAModifier.fonctionAssociee);
    }
    
    JScrollPane scrollPaneListe = new JScrollPane(listeFonctions, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    
    
    btnOK = new MtgButton(frame, getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed();
      }
    });    
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());
    
    JPanel panePrincipal = new JPanel();
    JPanel paneRight = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 40, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label1, constraints);
    
    setConstraints(constraints, 1, 0, 1, 1, 40, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(editeurNom, constraints);    
    
    setConstraints(constraints, 0, 1, 1, 1, 0, 50, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label2, constraints);

    setConstraints(constraints, 1, 1, 1, 1, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    panePrincipal.add(scrollPaneListe, constraints);
    
    setConstraints(constraints, 0, 2, 2, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label3, constraints);

    setConstraints(constraints, 0, 3, 2, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(info, constraints);
    
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
    setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.NONE, GridBagConstraints.NORTH);
    panePrincipal.add(paneRight, constraints);
    
    setContentPane(panePrincipal);
    pack(); 
    listeFonctionsSelectionChanged();
  }
  
  // Modifi version 4.6
  private void btnOKActionPerformed() {                                      
    MtgInternalFrame cadre = frame.cadreActif();
    CListeObjets listePr = cadre.listePr;
    String chNom = editeurNom.getText();
    if (!modification) {
	    // Modifi version 4.6
	    if (!listePr.validationNomCalcul(frame, chNom)) {
	      editeurNom.requestFocus();
	      return;
	    }
    }
    CFonc fonc = listeFonctions.fonctionSelectionnee();
    if (modification) {
    	deriveeAModifier.fonctionAssociee = fonc;
    }
    else {
    	CDerivee der = new CDerivee(listePr, null, false, chNom, fonc);
	    frame.ajouteElement(der);
	    der.positionne(frame.cadreActif().infRand, frame.cadreActif().dimf);
	  	cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil48"));
    }
  	setRetourOK(true);
  	dispose();
  }
  
  public void listeFonctionsSelectionChanged() {
  	int i = listeFonctions.getSelectedIndex();
  	if (i != -1) {
  		info.setText(listeFonctions.fonctionSelectionnee().infoFormule());
  		pack();
  	}
  }
	
  public void btnCancelActionPerformed() {
  	dispose();
  }

}
