/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CEditeurFormule;
import objets.CListeObjets;
import types.FonteNom;
import types.NatureCalcul;
import types.NatureCarSpe;

@SuppressWarnings("serial")
public class EditeurFormuleDlg extends MtgDlg {
	private boolean modification; // True si on modifie un objet dj cr
	private CEditeurFormule editeurFormule;
	private int indiceMax;
	private ListeCalculsOuFonctionsParFormule listeCalculs;
  private MtgButton btnCancel;
  private MtgButton btnOK;
  private TextFieldAvecCarSpeciaux editeurEnTete;
  private PaneTaillePolice paneTaillePolice;
  private PaneModeAffichage paneModeAffichage;
  private JList<String> listeTailles;
  private MtgFocusTraversalPolicy mtgFocusTraversalPolicy;
  private MtgCheckBox cbVariables1Car; // true si variables  1 caractre et signes de multiplication implicite
  // Ajout version 4.7.2
  private MtgCheckBox cbAffichageFormule;
  private JTextField editeurPreLatex;
  private MtgLabel label5;
  // Ajout version 4.7.2 o on donne la possibilit  l'utilisateur de choisir de ne pas afficher la formule laTeX au premier affichage
  private MtgCheckBox cbAffLaTeXInitial;
  private MtgCheckBox cbAffTempsReel;

	public EditeurFormuleDlg(MtgFrame frameInit, CEditeurFormule editeurFormuleInit, boolean modificationInit)	{
		super(getString("editeurFormuleDlg1"), true, frameInit);
		editeurFormule = editeurFormuleInit;
		modification = modificationInit;
    CListeObjets liste = frame.cadreActif().listePr;
    // Dans le cas ou on cre un nouvel objet, celui-ci n'a pas encore t ajout
    //  la liste des objets de l'applet.
    if (modification) indiceMax = liste.indexOf(editeurFormuleInit) - 1;
    else indiceMax = liste.size() - 1;
    listeCalculs = new ListeCalculsOuFonctionsParFormule(frame, liste, indiceMax, false, NatureCalcul.NCalculOuFonctionParFormule);
    JScrollPane scrollPaneListe = new JScrollPane(listeCalculs, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    btnOK = new MtgButton(frame, getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        btnOKActionPerformed(evt);
      }
    });    
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        btnCancelActionPerformed(evt);
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());
    
 		// Modification version 3.5 : Si l'afficgae de valeur est un objet final de construction
 		// on ne donne pas la possibilit de changer la valeur qu'on affiche mais seulement
 		// la faon dont elle est affiche
 		if (editeurFormule.estElementFinal) {
 			listeCalculs.setEnabled(false);
 		}
 		MtgLabel label1 = new MtgLabel(frame, getString("editeurFormuleDlg2"));
 		MtgLabel label2 = new MtgLabel(frame, getString("affichageValeurDlg3"));
 		MtgLabel label3 = new MtgLabel(frame, getString("editeurFormuleDlg3"));
 		MtgLabel label4 = new MtgLabel(frame, getString("editeurFormuleDlg4"));
 		cbVariables1Car = new MtgCheckBox(frame, getString("editeurFormuleDlg5"));
 		cbAffichageFormule = new MtgCheckBox(frame, getString("editeurFormuleDlg6"));
 		label5 = new MtgLabel(frame, getString("editeurFormuleDlg7"));
 		// Ajout version 4.7.2.1 pour donner  l'utilisateur la possibilit de ne pas afficher le LaTeX au chargement
 		cbAffLaTeXInitial = new MtgCheckBox(frame, getString("editeurFormuleDlg8"));
 		cbAffTempsReel = new MtgCheckBox(frame, getString("editeurFormuleDlg9"));
 		cbAffichageFormule.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				boolean b = cbAffichageFormule.isSelected();
				editeurPreLatex.setVisible(b);
				label5.setVisible(cbAffichageFormule.isSelected());
				cbAffLaTeXInitial.setVisible(b);
				cbAffTempsReel.setVisible(b);
				pack();
			}
		});
 		editeurEnTete = new TextFieldAvecCarSpeciaux(this, frame, NatureCarSpe.Tout);
 		editeurEnTete.setColumns(18);
 		editeurPreLatex = new JTextField();
 		editeurPreLatex.setColumns(18);

		listeTailles = new JList<String>();
		listeTailles.setFont(FonteNom.getFonteEdition(MtgFrame.sizeFontList, FonteNom.Normal,
				(double)frame.pref_addSystemFont/MtgFrame.sizeFontList + MtgFrame.coefMultForSystemFonts));		
		Vector<String> vec = new Vector<String>();
		// Modifie version 4.7.9
		// for (int i = 2; i <= 20; i++) {
		for (int i = 2; i <= 50; i++) {
			vec.add(Integer.toString(i));
		}
		listeTailles.setListData(vec);
		listeTailles.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		listeTailles.setLayoutOrientation(JList.VERTICAL);
		listeTailles.setVisibleRowCount(5);
		listeTailles.setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
    JScrollPane scrollpaneTaille = new JScrollPane(listeTailles, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		listeTailles.setSelectedIndex(editeurFormule.nbcol-2);
		listeTailles.ensureIndexIsVisible(listeTailles.getSelectedIndex());
 		
		cbVariables1Car.setSelected(editeurFormule.variables1Car);
		cbAffichageFormule.setSelected(editeurFormule.affichageFormule); // Ajout version 4.7.2
		editeurEnTete.setText(editeurFormuleInit.enTete);
		editeurPreLatex.setText(editeurFormule.preCodeLaTex);
		cbAffLaTeXInitial.setSelected(editeurFormule.affLaTeXPremierAffichage);
		cbAffTempsReel.setSelected(editeurFormule.affichageTempsReel);
 		if (modification) {
 			listeCalculs.setSel(editeurFormule.calculAssocie);
 			prepareChamps();
 		}
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;

    JPanel paneRightHigh = new JPanel();
 		paneRightHigh.setLayout(new BoxLayout(paneRightHigh, BoxLayout.Y_AXIS));
 		paneRightHigh.add(btnOK);
		paneRightHigh.add(Box.createRigidArea(new Dimension(0, 5))); 		 		
 		paneRightHigh.add(btnCancel);
 		JPanel paneHigh = new JPanel();
 		paneHigh.setLayout(new GridBagLayout());
    setConstraints(constraints, 0, 0, 1, 1, 20, 80, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneHigh.add(label1, constraints);
    setConstraints(constraints, 1, 0, 1, 1, 20, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneHigh.add(scrollPaneListe, constraints);
    setConstraints(constraints, 2, 0, 1, 1, 20, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneHigh.add(label3, constraints);
    setConstraints(constraints, 3, 0, 1, 1, 20, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneHigh.add(scrollpaneTaille, constraints);
    setConstraints(constraints, 4, 0, 1, 1, 20, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.EAST); 
 		paneHigh.add(paneRightHigh, constraints);
 		 		
    
    setConstraints(constraints, 0, 1, 1, 1, 20, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneHigh.add(label2, constraints);
    setConstraints(constraints, 1, 1, 1, 1, 80, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneHigh.add(editeurEnTete, constraints);
    setConstraints(constraints, 1, 2, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneHigh.add(label4, constraints);
    setConstraints(constraints, 0, 3, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneHigh.add(cbVariables1Car, constraints);
    // Ajout version 4.7.2
    setConstraints(constraints, 0, 4, 2, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneHigh.add(cbAffichageFormule, constraints);
    setConstraints(constraints, 0, 5, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.EAST);
    paneHigh.add(label5, constraints);
    setConstraints(constraints, 1, 5, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneHigh.add(editeurPreLatex, constraints);
    // Ajout version 4.7.2
    setConstraints(constraints, 0, 6, 2, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneHigh.add(cbAffLaTeXInitial, constraints);
    setConstraints(constraints, 0, 7, 2, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneHigh.add(cbAffTempsReel, constraints);
    //
    paneTaillePolice = new PaneTaillePolice(editeurFormule.indiceTaillePolice);
    paneModeAffichage = new PaneModeAffichage(frame, editeurFormule, false); // Modifi version 4.5.2
    JPanel paneLow = new JPanel();
    paneLow.setLayout(new FlowLayout());
    paneLow.add(paneTaillePolice);
    paneLow.add(paneModeAffichage);
    
    JPanel pane = new JPanel();
    pane.setLayout(new BoxLayout(pane, BoxLayout.Y_AXIS));
    pane.add(paneHigh, "North");
    pane.add(paneLow, "South");
    setContentPane(pane);
    getRootPane().setDefaultButton(btnOK);

    // On change l'ordre par dfaut des composants
    Vector<Component> order = new Vector<Component>(4);
    order.add(listeCalculs);
    order.add(editeurEnTete);
    order.add(btnOK);
    order.add(btnCancel);
    mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
    setFocusTraversalPolicy(mtgFocusTraversalPolicy);
    prepareChamps(); // Ajout version 4.7.2.2
    pack();
    editeurEnTete.requestFocus();
	}

  private void btnOKActionPerformed(java.awt.event.ActionEvent evt) {
  	MtgInternalFrame cadre = frame.cadreActif();
  	if (listeCalculs.isEnabled()) editeurFormule.calculAssocie = listeCalculs.calculSelectionne();
		editeurFormule.enTete = editeurEnTete.getText();
		editeurFormule.indiceTaillePolice = paneTaillePolice.indiceTaille();
		editeurFormule.encadrement = paneModeAffichage.styleEncadrement();
		editeurFormule.effacementFond = paneModeAffichage.effacementFond();
		editeurFormule.couleurFondAffichage = paneModeAffichage.couleurFond();
		editeurFormule.nbcol = listeTailles.getSelectedIndex() + 2;
		editeurFormule.variables1Car = cbVariables1Car.isSelected();
		// Ajout version 4.7.2
		editeurFormule.affichageFormule = cbAffichageFormule.isSelected();
		editeurFormule.preCodeLaTex = editeurPreLatex.getText();
		editeurFormule.affLaTeXPremierAffichage = cbAffLaTeXInitial.isSelected(); // Ajout version 4.7.2.1
		editeurFormule.affichageTempsReel = cbAffTempsReel.isSelected(); // Ajout version 4.7.2.1
		//
		if (editeurFormule.nbcol != editeurFormule.mtgFormulaEditor.getColumns()) {
			editeurFormule.mtgFormulaEditor.setColumns(editeurFormule.nbcol);
		}
		editeurFormule.mtgFormulaEditor.setPositionOnInternaleFrame();
		editeurFormule.mtgFormulaEditor.setForeground(editeurFormule.couleur);
		// editeurFormule.mtgFormulaEditor.setBackground(editeurFormule.couleurFondAffichage);		
		frame.pref_IndiceTaillePoliceAffichages = editeurFormule.indiceTaillePolice;
		cadre.couleurFondAffichage = editeurFormule.couleurFondAffichage;
		editeurFormule.creeFormule(); // Ajout version 4.7.2
		editeurFormule.mtgFormulaEditor.updateSizes(); // Ajout version 4.7.9
		cadre.validate(); // Ajout version 4.9.2 car en cas de changement de taille de police, l'affichage de formule
		// n'tait plus au bon endroit.
		setRetourOK(true);
		dispose();
  }
	
  private void btnCancelActionPerformed(ActionEvent evt) {                                          
    dispose();
  }
  
  private void prepareChamps() {
		boolean b = cbAffichageFormule.isSelected();
		editeurPreLatex.setVisible(b);
		label5.setVisible(b);
		cbAffLaTeXInitial.setVisible(b);
		cbAffTempsReel.setVisible(b);
  }

}