/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.Vector;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JInternalFrame;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

@SuppressWarnings("serial")
public class FenetresDlg extends MtgDlg {

	private MtgLabel label1;
	private JList<String> listeFenetres;
	private Vector<MtgInternalFrame> cadres = new Vector<MtgInternalFrame>();
	private JScrollPane scrollPaneListe;
	private MtgButton btnOK, btnCancel;
	
	public FenetresDlg(MtgFrame frameInit) {
    super (getString("fenetreDlg1"), true, frameInit);
    initComponents();
    getRootPane().setDefaultButton(btnOK);    		
	}
	
  private void initComponents() {
  	label1 = new MtgLabel(frame, getString("fenetreDlg2"));
    btnOK = new MtgButton(frame, getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed();
      }
    });    
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());
    listeFenetres = new JList<String>();
    listeFenetres.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    Vector<String> titres = new Vector<String>();
		JInternalFrame[] tableCadres = frame.mtgDesktopPane.getAllFrames();
    for (int i = tableCadres.length - 1; i >= 0; i--) {
    	titres.add(tableCadres[i].getTitle());
    	cadres.add((MtgInternalFrame) tableCadres[i]);
    }
    listeFenetres.setListData(titres);
    scrollPaneListe = new JScrollPane(listeFenetres, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    JPanel panePrincipal = new JPanel();
    JPanel paneRight = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;

    setConstraints(constraints, 0, 0, 1, 1, 80, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label1, constraints);
    setConstraints(constraints, 0, 1, 1, 1, 0, 40, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(scrollPaneListe, constraints);

    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
    setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.NONE, GridBagConstraints.NORTH);
    panePrincipal.add(paneRight, constraints);
    
    setContentPane(panePrincipal);
    pack();     
  }
  
  private void  btnOKActionPerformed() {
  	frame.activeCadre(cadres.get(listeFenetres.getSelectedIndex()));
  	setRetourOK(true);
  	dispose();
  }
  
  private void  btnCancelActionPerformed() {
  	dispose();
  }
}
