/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.text.NumberFormat;
import javax.swing.JLabel;
import javax.swing.JPanel;
import objets.CLieuDiscretDeBase;
import mathgraph32.MtgFrame;

@SuppressWarnings("serial")
public class LieuDiscretDlg extends MtgDlg {
	private CLieuDiscretDeBase lieuAModifier;
  private MtgButton btnCancel;
  private MtgButton btnOK;
  private MtgLabel label1;
  private MtgLabel label2;
  private MtgFormattedTextField editeurNombrePoints;

  public LieuDiscretDlg(MtgFrame frameInit, CLieuDiscretDeBase lieuAModifierInit) {
    super (getString("lieuDiscretDlg1"), true, frameInit);
    lieuAModifier = lieuAModifierInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK);
    editeurNombrePoints.setSelectionStart(0);
    editeurNombrePoints.setSelectionEnd(editeurNombrePoints.getText().length());
    editeurNombrePoints.requestFocus();
    getRootPane().setDefaultButton(btnOK);    
  }
  private void initComponents() {
    btnOK = new MtgButton(frame, getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed(evt);
      }
    });    
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed(evt);
      }
    });
    btnOK.setPreferredSize(btnCancel.getMaximumSize());  
    label1 = new MtgLabel(frame, getString("lieuDiscretDlg2"), JLabel.RIGHT);
    label1.setRequestFocusEnabled(false);
    label2 = new MtgLabel(frame, getString("lieuDiscretDlg3"), JLabel.RIGHT);
    label2.setRequestFocusEnabled(false);
    NumberFormat format;
    format = NumberFormat.getInstance();
    format.setParseIntegerOnly(true);
    format.setGroupingUsed(false);
    editeurNombrePoints = new MtgFormattedTextField(frame, format);
    editeurNombrePoints.setColumns(4);
    
  	editeurNombrePoints.setText(Integer.toString(lieuAModifier.nombreDePoints));
    	
    JPanel panePrincipal = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 0;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 40, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label1, constraints);	
    setConstraints(constraints, 1, 0, 1, 1, 30, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(editeurNombrePoints, constraints);
    setConstraints(constraints, 2, 0, 1, 1, 30, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(btnOK, constraints);
    constraints.insets.bottom = 10;
    setConstraints(constraints, 0, 1, 2, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label2, constraints);
    setConstraints(constraints, 2, 1, 1, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(btnCancel, constraints);
    setContentPane(panePrincipal);
    pack();    
  }
  
  private void btnOKActionPerformed(java.awt.event.ActionEvent evt) {
  	int nombrePoints;
  	try {
  		nombrePoints = Integer.parseInt(editeurNombrePoints.getText());
  	}
		catch (NumberFormatException e) {
			dialogueAvertissement(getString("ch71"));
			editeurNombrePoints.requestFocus();
			return;
		}
    if ((nombrePoints < 4 ) || nombrePoints > 5000) {
    	dialogueAvertissement(getString("ch71"));
    	editeurNombrePoints.requestFocus();
    	return;
    }
    int anciennombrePoints = lieuAModifier.nombreDePoints;
    lieuAModifier.nombreDePoints = nombrePoints;
    // Si on on a chang le nombre de points du lieu il faut
    // revoir toutes les surfaces ayant ce lieu pour bord et recrer leurs tabelaux de coorodnnes
    if (anciennombrePoints != nombrePoints) lieuAModifier.changeCaracteristiques();
    setRetourOK(true);
    dispose();
  }
  
  private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {                                          
    dispose();
  }  
 }
