/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.util.Collections;
import java.util.Vector;

import javax.swing.JList;
import javax.swing.ListSelectionModel;

import mathgraph32.MtgFrame;
import types.FonteNom;
import types.Int;
import types.OperateurFonction;
import types.OperateurFonction2Var;
import types.OperateurFonction3Var;
import types.OperateurFonction4Var;
import types.OperateurFonction5Var;

@SuppressWarnings("serial")
public class ListeFonctionsPredefinies extends JList<String> {
	private boolean fonctionsReelles;
	private Vector<String> nomsFonctions;
	private Vector<Int> nombreVariables;
	private Vector<String> syntaxe;
	/**
	 * Constructeur
	 */
	public ListeFonctionsPredefinies(MtgFrame frame, boolean fonctionsReellesInit) {
		super();
		setFont(FonteNom.getFonteEdition(MtgFrame.sizeFontList, FonteNom.Normal,
				(double)frame.pref_addSystemFont/MtgFrame.sizeFontList + MtgFrame.coefMultForSystemFonts));		
		fonctionsReelles = fonctionsReellesInit;
		nomsFonctions = new Vector<String>();
		nombreVariables = new Vector<Int>();
		syntaxe = new Vector<String>();
    setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		// Ajout des noms de fonctions prdfinies de une variable
		if (fonctionsReelles) {
	    for (int i = 0; i < OperateurFonction.nombreFonctionsReelles1Var; i++) {
				ajoute(OperateurFonction.nomsFonctions[i], new Int(1), OperateurFonction.syntaxeNomsFonctions[i]);
			}
		}
		else {
			for (int i = 0; i < OperateurFonction.nombreFonctionsComplexes1Var; i++) {
				ajoute(OperateurFonction.nomsFonctionsComplexes[i], new Int(1), OperateurFonction.syntaxeNomsFonctionsComplexes[i]);
			}
		}
		// Ajout des noms de fonctions prdfinies de deux variables
		if (fonctionsReelles) {
			for (int i = 0; i < OperateurFonction2Var.nombreFonctionsReelles2Var; i++) {
				ajoute(OperateurFonction2Var.nomsFonctions2Var[i], new Int(2), OperateurFonction2Var.syntaxeNomsFonctions2Var[i]);
			}
		}
		else {
			for (int i = 0; i < OperateurFonction2Var.nombreFonctionsComplexes2Var; i++) {
				ajoute(OperateurFonction2Var.nomsFonctionsComplexes2Var[i], new Int(2), OperateurFonction2Var.syntaxeNomsFonctionsComplexes2Var[i]);
			}
		}
		// Ajout des noms de fonctions prdfinies de trois variables
		if (fonctionsReelles) {
			for (int i = 0; i < OperateurFonction3Var.nombreFonctionsReelles3Var; i++) {
				ajoute(OperateurFonction3Var.nomsFonctions3Var[i], new Int(3), OperateurFonction3Var.syntaxeNomsFonctions3Var[i]);
			}
		}
		else {
			for (int i = 0; i < OperateurFonction3Var.nombreFonctionsComplexes3Var; i++) {
				ajoute(OperateurFonction3Var.nomsFonctionsComplexes3Var[i], new Int(3), OperateurFonction3Var.syntaxeNomsFonctionsComplexes3Var[i]);
			}
		}
		// Ajout des noms de fonctions prdfinies de trois variables
		if (fonctionsReelles) {
			for (int i = 0; i < OperateurFonction4Var.nombreFonctionsReelles4Var; i++) {
				ajoute(OperateurFonction4Var.nomsFonctions4Var[i], new Int(4), OperateurFonction4Var.syntaxeNomsFonctions4Var[i]);
			}
		}
		else {
			for (int i = 0; i < OperateurFonction4Var.nombreFonctionsComplexes4Var; i++) {
				ajoute(OperateurFonction4Var.nomsFonctionsComplexes4Var[i], new Int(4), OperateurFonction4Var.syntaxeNomsFonctionsComplexes4Var[i]);
			}
		}
		
		// Ajout des noms de fonctions prdfinies de cinq variables
		if (fonctionsReelles) {
			for (int i = 0; i < OperateurFonction5Var.nombreFonctionsReelles5Var; i++) {
				ajoute(OperateurFonction5Var.nomsFonctions5Var[i], new Int(5), OperateurFonction5Var.syntaxeNomsFonctions5Var[i]);
			}
		}
		else {
			for (int i = 0; i < OperateurFonction5Var.nombreFonctionsComplexes5Var; i++) {
				ajoute(OperateurFonction5Var.nomsFonctionsComplexes5Var[i], new Int(5), OperateurFonction5Var.syntaxeNomsFonctionsComplexes5Var[i]);
			}
		}
		
		setListData(nomsFonctions);
    setVisibleRowCount(12);
    setSelectedIndex(0);
	}
	
	public final String nomFonction() {
		return nomsFonctions.get(getSelectedIndex());
	}
	
	public final int nombreVariables() {
		return nombreVariables.get(getSelectedIndex()).getValue();
	}
	
	public final String syntaxe() {
		return syntaxe.get(getSelectedIndex());
	}
	
	private final void ajoute(String nom, Int nbVar, String synt) {
		nomsFonctions.add(nom);
		Collections.sort(nomsFonctions);
		int ind = nomsFonctions.indexOf(nom);
		nombreVariables.add(ind, nbVar);
		syntaxe.add(ind, synt);
	}
}
