/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.BorderLayout;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import types.FonteNom;
import types.UniteAngle;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

@SuppressWarnings("serial")
public class OptionsPreferencesDlg extends MtgDlg {
	private JTabbedPane tabbedPane;
	private PanePreferencesChoixUniteAngle panePreferencesChoixUniteAngle;	
	private PanePreferencesAnimation panePreferencesAnimation;
	private PanePreferencesRepertoireFichiers panePreferencesRepertoireFichiers;
	private PanePreferencesRepertoireConstructions panePreferencesRepertoireConstructions;
	private PanePreferencesLangue panePreferencesLangue;
	public PanePreferencesNiveau panePreferencesNiveau;
	private PanePreferencesTranslationFigure panePreferencesTranslationFigure;
	private PanePreferencesPersonnalisationNiveau panePreferencesPersonnalisationNiveau;
	private PanePreferencesAntialiasing panePreferencesAntialiasing;
	private panePreferencesPressePapier panePreferencesPressePapier;
	private PanePreferencePoliceSysteme panePreferencePoliceSysteme;
	
	private MtgButton btnOK;
	private MtgButton btnCancel;
	private int niveauPrec;
	private boolean niveauPersonnalisePrec;
	// Ajout version 4.7.5.2
	private int decPoliceSystemePrec;
	
  public OptionsPreferencesDlg(MtgFrame frameInit) {
    super(getString("dialoguePreferences"),true, frameInit);
    // On mmorise le niveau actuel
    niveauPrec = frame.pref_niveau;
    // Ajout version 4.7.5.2
    decPoliceSystemePrec = frame.pref_addSystemFont;
  	niveauPersonnalisePrec = frame.pref_NiveauPersonnalise;
    tabbedPane = new JTabbedPane();

    panePreferencesChoixUniteAngle = new PanePreferencesChoixUniteAngle(frame);
    panePreferencesAnimation = new PanePreferencesAnimation(frame);
    panePreferencesRepertoireFichiers = new PanePreferencesRepertoireFichiers(frame);
    panePreferencesRepertoireConstructions = new PanePreferencesRepertoireConstructions(frame);
    panePreferencesLangue = new PanePreferencesLangue(frame);
    panePreferencesNiveau = new PanePreferencesNiveau(frame);
    panePreferencesTranslationFigure = new PanePreferencesTranslationFigure(frame);
    panePreferencesPersonnalisationNiveau = new PanePreferencesPersonnalisationNiveau(frame, this);
    panePreferencesAntialiasing = new PanePreferencesAntialiasing(frame);
    panePreferencesPressePapier = new panePreferencesPressePapier(frame);
    panePreferencePoliceSysteme = new PanePreferencePoliceSysteme(frame);
		tabbedPane.setFont(FonteNom.getFonteNom(MtgFrame.sizeFontLabel,
				FonteNom.Normal, (double)frame.pref_addSystemFont/MtgFrame.sizeFontLabel + MtgFrame.coefMultForSystemFonts));		

    tabbedPane.addTab(panePreferencesChoixUniteAngle.titreOnglet(), null, panePreferencesChoixUniteAngle, null);
    tabbedPane.addTab(panePreferencesAnimation.titreOnglet(), null, panePreferencesAnimation, null);
    tabbedPane.addTab(panePreferencesRepertoireConstructions.titreOnglet(), null, panePreferencesRepertoireConstructions, null); 
    tabbedPane.addTab(panePreferencesLangue.titreOnglet(), null, panePreferencesLangue, null);
    tabbedPane.addTab(panePreferencesNiveau.titreOnglet(), null, panePreferencesNiveau, null);
    tabbedPane.addTab(panePreferencesTranslationFigure.titreOnglet(), null, panePreferencesTranslationFigure, null);
    tabbedPane.addTab(panePreferencesPersonnalisationNiveau.titreOnglet(), null, panePreferencesPersonnalisationNiveau, null);
    tabbedPane.addTab(panePreferencesAntialiasing.titreOnglet(), null, panePreferencesAntialiasing, null);
    tabbedPane.addTab(panePreferencesPressePapier.titreOnglet(), null, panePreferencesPressePapier, null);
    tabbedPane.addTab(panePreferencePoliceSysteme.titreOnglet(), null,  panePreferencePoliceSysteme, null);
   
	  btnOK = new MtgButton(frame, getString("btnOK"));
	  btnOK.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnOKActionPerformed();
	    }
	  });    
	  btnCancel = new MtgButton(frame, getString("btnCancel"));
	  btnCancel.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnCancelActionPerformed();
	    }
	  });
	  btnOK.setPreferredSize(btnCancel.getPreferredSize());
    JPanel paneSouth = new JPanel();
    paneSouth.add(btnOK);
    paneSouth.add(btnCancel);
    getContentPane().setLayout(new BorderLayout());
    getContentPane().add("North", tabbedPane);
    getContentPane().add("South", paneSouth);
    getRootPane().setDefaultButton(btnOK);    
    pack();
  }
  
  private void btnCancelActionPerformed() {
  	dispose();
  }
  
  private void btnOKActionPerformed() {
  	// On regarde si le nombre de points pour animation est valide
		MtgInternalFrame cadre = frame.cadreActif();
  	if (!panePreferencesAnimation.donneesValides()) {
  		tabbedPane.setSelectedIndex(2);
  		return;
  	}
  	if (!panePreferencesRepertoireFichiers.donneesValides()) {
  		dialogueAvertissement(getString("ch49"));
  		tabbedPane.setSelectedIndex(3);
  		return;
  	}
  	if (!panePreferencesRepertoireConstructions.donneesValides()) {
  		dialogueAvertissement(getString("ch49"));
  		tabbedPane.setSelectedIndex(4);
  		return;  		
  	}
  	if (!panePreferencesPressePapier.donneesValides()) {
  		tabbedPane.setSelectedIndex(10);
  		return;  		
  	}
  	panePreferencesLangue.sauveDonnees();
  	panePreferencesChoixUniteAngle.sauveDonnees();
  	panePreferencesAnimation.sauveDonnees();
  	panePreferencesRepertoireFichiers.sauveDonnees();
  	panePreferencesRepertoireConstructions.sauveDonnees();
  	panePreferencesNiveau.sauveDonnees();
  	panePreferencesTranslationFigure.sauveDonnees();
  	panePreferencesPersonnalisationNiveau.sauveDonnees();
  	panePreferencesAntialiasing.sauveDonnees();
  	panePreferencesPressePapier.sauveDonnees();
  	panePreferencePoliceSysteme.sauveDonnees();
  	// Ajout version 4.7.5.2
  	if (decPoliceSystemePrec != frame.pref_addSystemFont)
  		frame.adaptePoliceSysteme();
  	// Si on n'est pas en mode de fonctionnement presonnalis et qu'on a choisi le mode collge ou cole on impose le degr comme unit
  	if (!frame.pref_NiveauPersonnalise && (frame.pref_niveau >= MtgFrame.niveauMoyen) && (frame.pref_angleUnit == UniteAngle.Radian)) {
  		dialogueAvertissement(getString("ch166"));
  		frame.pref_angleUnit = UniteAngle.Degre;
  	}
  	// Si le niveau a chang ou si on a chang le mode de fonctionnement personnalis ou non on adapte les menus
  	// Modifi version 4.6.3
  	if ((niveauPrec != frame.pref_niveau) || (niveauPersonnalisePrec != frame.pref_NiveauPersonnalise)) {
  		// if (nbColPrec != frame.nbColBarreOutils) frame.prepareBarreOutilsGauche(nbColPrec); Inutile supprim version 4.6.3
			frame.setDocPreferencesNiveau();
			frame.setDescripteursNiveau();
  		frame.creeMenu();
			frame.creeBoutons();
			if (cadre.document != null) 
				frame.metAJourIconesEtMenus(cadre.document);
				else frame.metAJourIconesEtMenus(frame.docPreferencesNiveau[frame.pref_niveau]);
  		// if (cadre != null) frame.metAJourIconesEtMenus(cadre.document);
  		frame.properties.setProperty("level",Integer.toString(frame.pref_niveau));
  	}
  	// Modifi version 4.7.5.2
  	// Pour un changement de taille des polices systme
		if (decPoliceSystemePrec != frame.pref_addSystemFont) dialogueAvertissement(getString("redemarrageNecessaire"));
  	// Si le nombre de colonnes de la barre d'outils a chang ou si on a demand une taille diffrente d'icnes on la reconstruit
  	/*
  	if (nbColPrec != frame.pref_NbCol) {
  		frame.prepareBarreOutils(nbColPrec);
  		if (frame.cadreActif() != null) frame.metAJourBoutonsBarreOutils(frame.cadreActif().document);
  		else frame.metAJourBoutonsBarreOutils(new CMathGrafDoc(frame));
  	}
  	*/
  	frame.properties.setProperty("langage", frame.pref_langage);
  	String ch1;
		if (frame.pref_angleUnit == UniteAngle.Radian) ch1 = "radian"; else ch1 = "degree";
		frame.properties.setProperty("angleUnit", ch1);
		frame.properties.setProperty("moveNbPoints", Integer.toString(frame.pref_moveNbPoints));
		frame.properties.setProperty("moveFreq", Integer.toString(frame.pref_moveFreq));
		frame.properties.setProperty("moveCyclic", Boolean.toString(frame.pref_moveCyclic));
		frame.properties.setProperty("fileDirectory", frame.pref_fileDirectory);
		frame.properties.setProperty("constDirectory", frame.pref_constDirectory);
		frame.properties.setProperty("visualHelp", Boolean.toString(frame.pref_AideVisuelle));
		frame.properties.setProperty("FreeDisplaysTranslation", Boolean.toString(frame.pref_decalageAffichagesLibres));
		frame.properties.setProperty("personalizedLevel", Boolean.toString(frame.pref_NiveauPersonnalise));
		frame.properties.setProperty("measuresDisplay1", Boolean.toString(frame.pref_affichageMesuresNiveau[1]));
		frame.properties.setProperty("measuresDisplay2", Boolean.toString(frame.pref_affichageMesuresNiveau[2]));
		frame.properties.setProperty("measuresDisplay3", Boolean.toString(frame.pref_affichageMesuresNiveau[3]));
		frame.properties.setProperty("measuresDisplay4", Boolean.toString(frame.pref_affichageMesuresNiveau[4]));
		frame.properties.setProperty("antialiasing", Boolean.toString(frame.pref_antialiasing));
		
  	setRetourOK(true);
  	dispose();
  }  
	
}
