/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package mathgraph32;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JToggleButton;
import javax.swing.Timer;
import actions.MtgAction;
import objets.CMathGrafDoc;
/**
 * Classe ajoute  la version 5.0.
 * Reprsente les boutons droulables de la nouvelle barre d'outils de gauche.
 * @author Yves
 * Modifi version 5.0
 */
@SuppressWarnings("serial")
public class MtgIconButton extends JToggleButton implements MouseListener {
	public MtgFrame frame;
  public MtgHorizontalMenu menu; // Le popup menu associ
  public MtgAction[] tabActions;
  public MtgAction action;
  private boolean dblClick = false;

  ImageIcon imi = null;
  public MtgIconButton(MtgFrame frameInit, MtgAction[] tabActionsInit) {
    super();
    tabActions = tabActionsInit;
    this.frame = frameInit;
    action = premiereActionActivable();
    imi = getImageIcon(action.imicon);
    setAction(action);
    
    setBorder(BorderFactory.createLineBorder(Color.white,2));
    
    setText("");
  	setToolTipText(action.tip);
  	addMouseListener(this);
  }
  
  // Necssaire de redfinir getIcon() sinon les icnes se redimensionnent une fois qu'on cliqu sur l'icne
  @Override
  public Icon getIcon() {
  	return imi;
  }
  
  public void setAction(MtgAction newAction) {
  	// setIcon(getImageIcon(newAction.imicon));
  	action = newAction;
  	imi = getImageIcon(action.imicon);
  	setSelected(true);
  	setToolTipText(action.tip);
  	action.setEnabled(true);
  	repaint();
  }
  public boolean contientItemActivable() {
		MtgInternalFrame cadre = frame.cadreActif();
		CMathGrafDoc doc = cadre != null ? cadre.document : null;
		for (int i = 0; i < tabActions.length; i++) {
			MtgAction action = tabActions[i];
			if ((doc  == null) || (doc.itemPermis(action.idMenu()) && frame.docPreferencesNiveau[frame.pref_niveau].itemPermis(action.idMenu()) &&
					((action.outilAssocie == null) || action.outilAssocie.activationValide()))) return true;
		}
		return false;
  }
  
  // Pour Linux, ncessaire de redfinir paint car sinon on ne voit pas bien quand l'icne est slectionne.
	public void paint(Graphics arg0) {
		// TODO Auto-generated method stub
		super.paint(arg0);
		Graphics2D pDC= (Graphics2D) arg0;
		if (isSelected()) {
			pDC.setColor(Color.blue);
	    Composite originalComposite = pDC.getComposite();
	    pDC.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.2f));
	    pDC.fillRect(2, 2, getWidth()-2, getHeight()-2);
	    pDC.setComposite(originalComposite);
	    // Ne pas utiliser setBorder dans Paint. tansforme le PC en grille-pain
	    // Si l'icne est active, on l'entoure d'un cadre bleu
	  }
	}
	
	private MtgAction premiereActionActivable() {
		MtgInternalFrame cadre = frame.cadreActif();
		CMathGrafDoc doc = cadre != null ? cadre.document : null;
		for (int i = 0; i < tabActions.length; i++) {
			MtgAction action = tabActions[i];
			if ((doc  == null) || (doc.itemPermis(action.idMenu()) && frame.docPreferencesNiveau[frame.pref_niveau].itemPermis(action.idMenu()) && 
					((action.outilAssocie == null) || action.outilAssocie.activationValide()))) return action;
		};
		return tabActions[0];
	}
	public boolean fermeMenu() {
		dblClick = false;
		if ((menu != null) && menu.isVisible()) {
			menu.setVisible(false);
			return true;
		}
		else return false;
	}
	public void activeAction (MtgAction actionCherchee) {
		for (int i = 0; i < tabActions.length; i++) {
			MtgAction action = tabActions[i];
			if (action == actionCherchee) setAction(action);
		}
	}
	public void mouseClicked(MouseEvent arg0) {
	}

	/**
	 * Quand une icne a dj droul son menu et qu'on dplace la souris au-dessus d'une autre on droule le menu correspondant
	 */
	public void mouseEntered(MouseEvent arg0) {
		int ind = frame.indiceMenuHorizontal();
		if ((ind != -1) && (frame.tabIcones[ind] != this)) {
			frame.fermeMenusHorizontaux();
	    menu = new MtgHorizontalMenu(frame, tabActions, this);
			menu.setLocation(getLocationOnScreen().x+getWidth(), getLocationOnScreen().y);
			menu.setVisible(true);			
		}
	}
	
	public void mouseExited(MouseEvent arg0) {
	}

	/*
	public void mousePressed(MouseEvent arg0) {
		frame.fermeMenusHorizontauxSauf(this);
		if ((arg0.getClickCount() == 2) && ((menu == null) || !menu.isVisible())) {
	    menu = new MtgHorizontaMenu(frame, tabActions, this);
			menu.setLocation(getLocationOnScreen().x+getWidth(), getLocationOnScreen().y);
			menu.setVisible(true);
		}
		else {
			if ((menu != null) && menu.isVisible()) {
				menu.setVisible(false);
				menu.removeAll();
				menu = null;		 
			}
			action.actionPerformed(null);
		}
	}
	*/
	public void mousePressed(MouseEvent e) {
		 frame.fermeMenusHorizontauxSauf(this);
	   if (e.getClickCount() == 2) dblClick = true;
	   else {
	  	 final MtgIconButton owner = this;
	  	 // Je n'utilise pas ce qui suit.
	  	 // Car cela oblige  attendre le temps maximum considr comme un double clic pour ragir
	  	 // Si le double-click infieure  300 ms n'est pas considr comme un double click, tant pis
	  	 // Il faudra cliquer sur la flche
	  	 // Integer timerInterval = (Integer) Toolkit.getDefaultToolkit().getDesktopProperty("awt.multiClickInterval");
       // Timer timer = new Timer(timerInterval.intValue(), new ActionListener() {
	  	 Timer timer = new Timer(300, new ActionListener() {
         public void actionPerformed(ActionEvent evt) {
	         if (dblClick) {
		       		if ((menu == null) || !menu.isVisible()) {
		      	    menu = new MtgHorizontalMenu(frame, tabActions, owner);
		      			menu.setLocation(getLocationOnScreen().x+getWidth(), getLocationOnScreen().y);
		      			menu.setVisible(true);
		      		}
		       		else {
		       			dblClick = false;
		    				menu.setVisible(false);
		    				menu.removeAll();
		    				menu = null;
		    				action.actionPerformed(null);
		       		}
		       }
	         else {
	     			if ((menu != null) && menu.isVisible()) {
	    				menu.setVisible(false);
	    				menu.removeAll();
	    				menu = null;		 
	    			}
	    			action.actionPerformed(null);
		       }
         }    
       	});
       timer.setRepeats(false);
       timer.start();
	   }
	}
	
	public void mouseReleased(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

  private final ImageIcon getImageIcon(ImageIcon imi) {
  	if (imi == null) return null;
  	return new MtgImageIcon(frame.coefMultVertIcons(), imi);
  }

}