/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package mathgraph32;

import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import javax.swing.JTextField;
import javax.swing.SpringLayout;
import dialogs.MtgDlg;
import objets.CBipoint;
import objets.CDroite;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CPointAncetre;
import objets.CPointLieBipoint;
import types.FonteNom;
import types.NatureObjet;

// Ajout version 4.9.9 pour proposer automatiquement de nommer un point ds sa cration.
@SuppressWarnings("serial")
public class MtgNameEditor extends JTextField implements KeyListener, FocusListener {
	private MtgInternalFrame cadre;
	private CElementGraphique pointOuDroiteAssocie;
	public MtgNameEditor(MtgInternalFrame cadreInit) {
		super(2); // 2 colonnes
		// Version 5.0 : On tient compte des crans haute rsolution
		double k = MtgFrame.coefMultForSystemFonts;
		setFont(FonteNom.getFonteNom((int)(k*MtgFrame.sizeFontEditor), FonteNom.Normal, -1));
		cadre = cadreInit;
		pointOuDroiteAssocie = null;
		addKeyListener(this);
		addFocusListener(this);

	}
	public final void associeA(CElementGraphique pt) {
		cache();
		pointOuDroiteAssocie = pt;
	}
	public final void cache() {
		setVisible(false);
		// cadre.remove(this);
		// cadre.validate();
	}
	
	public final void setPositionOnInternaleFrame() {
		if (pointOuDroiteAssocie == null) return;
		CPointAncetre pointAssocie = null;
		CDroite droiteAssociee = null;
		pointAssocie = pointOuDroiteAssocie.estDeNature(NatureObjet.NToutPoint) ? (CPointAncetre) pointOuDroiteAssocie
				: null;
		droiteAssociee = pointOuDroiteAssocie.estDeNature(NatureObjet.NToutPoint) ? null : (CDroite) pointOuDroiteAssocie;
		boolean horsEcran = (pointAssocie != null) ? !pointAssocie.dansFenetre : droiteAssociee.horsFenetre;
		if (horsEcran) {
			cadre.remove(this);
			return;
		}
 		cadre.remove(this);
 		this.setText("");
    cadre.add(this, new Integer(399));
    int width = getPreferredSize().width;
    int height = getPreferredSize().height;
    int decalagevert = height+3;
    int decalagehor = (int) (width/2);
    int x = (pointAssocie != null) ? (int) (pointAssocie.x-decalagehor) : (int) (droiteAssociee.x_nom-droiteAssociee.dec_x-decalagehor);
    int xmax = cadre.dimf.longueurAbscisses;
    int ymax = cadre.dimf.longueurOrdonnees;
    if (x + width > xmax) x = x - decalagehor;
    else if (x < 0) x = x + decalagehor;    
    int y = (pointAssocie != null) ? (int) (pointAssocie.y + 10) : (int) (droiteAssociee.y_nom-droiteAssociee.dec_y);
    if (y > ymax - decalagevert) y -= (pointAssocie != null) ? 2*decalagevert : decalagevert;
    
		cadre.springl.putConstraint(SpringLayout.WEST, this, x, SpringLayout.WEST, cadre);
		cadre.springl.putConstraint(SpringLayout.NORTH, this, y, SpringLayout.NORTH, cadre);
		cadre.validate();
		setVisible(true);
		requestFocus();
	}


	public void focusGained(FocusEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void focusLost(FocusEvent e) {
	}

	public void keyTyped(KeyEvent e) {
	}

	public void keyPressed(KeyEvent e) {
		CListeObjets liste = cadre.listePr;
    String ch = getText();
		if ((e.getKeyCode() == KeyEvent.VK_ENTER)) { // Appui sur la touche entre quand l'diteur a le focus
      boolean nomValide = liste.validationNomPoint(ch);
      if (nomValide) {
        if (liste.existeMemeNomNonIntermediaire(pointOuDroiteAssocie,ch)) {
          MtgDlg.dialogueAvertissementCentreEcran(MtgFrame.getString("ch20"));
          requestFocus();
          return;
        }
        else {
	        pointOuDroiteAssocie.donneNom(ch);
        	// Ajout version 4.9.9.3 pour que lors d'une annuler refaire le nom soit bien prsent.
	        cadre.gestionnaire.enregistreFigureEnCours(MtgFrame.getString("chaineOutil67"));
	        pointOuDroiteAssocie.indiceTailleNom = cadre.frame.pref_indiceTaillePoliceNom;
	        pointOuDroiteAssocie.positionneNom();
	        this.setVisible(false);
	        cadre.repaintFigureSansCalcul();
	        if (pointOuDroiteAssocie.getClass()==CPointLieBipoint.class) {
	        	CPointLieBipoint ptl = (CPointLieBipoint) pointOuDroiteAssocie;
	        	CBipoint bp = ptl.ptBipoint;
	        	CPointLieBipoint p = bp.pointReelDansListe((ptl.indiceDansBipoint == 1) ? 2 : 1);
	        	// p peut tre nul par exemple avec l'outil de cration d'un rectangle direct
	        	if ((p != null) && p.existe && p.rendNom().isEmpty() && p.dansFenetre) {
	        		associeA(p);
	        		setPositionOnInternaleFrame();
	        	}  	
	        }
        }
      }
      else {
        MtgDlg.dialogueAvertissementCentreEcran(MtgFrame.getString("ch19"));
        return;
      }
		}
		else {
			// Si appui sur la touche chappement on fait disparaitre l'diteur
			if (e.getKeyCode() == KeyEvent.VK_ESCAPE) 
				setVisible(false);
		}
	}

	public void keyReleased(KeyEvent e) {
	}

}