/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package mathgraph32;

import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.JToggleButton;
import types.MotifPoint;


@SuppressWarnings("serial")
public class MtgStylePointButton extends JToggleButton implements MouseListener, ActionListener {
	private MtgFrame frame;
	private int stylePointAssocie;
  private static Insets insets = new Insets(1,1,1,1);
  
	/**
	 * Constructeur
	 */
	public MtgStylePointButton(MtgFrame frameInit, int StylePointAssocieInit) {
		super(new MtgImageIcon(0.25*MtgFrame.coefMultForDlgBoxIcons, MtgFrame.class.getResource("images/" + nomFichierAssocie(StylePointAssocieInit) + ".png")));
		setMargin(insets);
		frame = frameInit;
		stylePointAssocie = StylePointAssocieInit;
    addMouseListener(this);
    addActionListener(this);
    setToolTipText(MtgFrame.getString("tipChoixStylePoint"));
	}
	
	public static String nomFichierAssocie(int stylePoint) {
		String ch;
		switch (stylePoint) {
		case MotifPoint.Carre :
			ch = "StylePointCarre";
			break;
		case MotifPoint.Croix :
			ch = "StylePointCroix";
			break;
		case MotifPoint.Rond :
			ch = "StylePointRond";
			break;
		case MotifPoint.PetitRond :
			ch = "StylePointPetitRond";
			break;
		case MotifPoint.Mult :
			ch = "StylePointMult";
			break;
		case MotifPoint.Losange :
			ch = "StylePointLosange";
			break;
		// Ajout version 5.0
		case MotifPoint.GrandRond :
			ch = "StylePointGrandRond";
			break;
		case MotifPoint.GrandMult :
			ch = "StylePointGrandMult";
			break;
			// Ajout version 5.1
		case MotifPoint.Pixel :
			ch = "StylePointPixel";
			break;

		default : ch = "";
		}
		return ch;
	}

	public void mouseClicked(MouseEvent e) {
	}

	public void mouseEntered(MouseEvent e) {
    if (getCursor() != MtgFrame.defaultCursor) frame.setCursor(MtgFrame.defaultCursor);
	}

	public void mouseExited(MouseEvent e) {
	}

	public void mousePressed(MouseEvent e) {
	}

	public void mouseReleased(MouseEvent e) {
	}

	public void actionPerformed(ActionEvent e) {
		frame.stylePointActif = stylePointAssocie;
	}

}
