/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


package objets;

import java.io.IOException;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import types.Complexe;
import types.ErreurCalculException;

public class CAppelFonctionNVar extends CCalculBase {
	protected int nbVar; // Le nombre de variables
	protected CFoncNVar fonctionAssociee;
	protected CCalculBase[] operande;

  /** Creates new CAppelFonction */
  public CAppelFonctionNVar() {
  }
  
  /** Constructeur */
  public CAppelFonctionNVar(CListeObjets liste, int nbVarInit, CFoncNVar fonctionInit, CCalculBase[] operandeInit)
  {
    nbVar = nbVarInit;
  	listeProprietaire = liste;
    fonctionAssociee = fonctionInit;
    operande = operandeInit;
  }
  
  @Override
  protected long nature() {
  	return natAppelFonctionNVar;
  }
  
  public CCalculBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
  	CCalculBase[] operandeClone = new CCalculBase[nbVar];
  	for (int i = 0; i < nbVar;i++)
	    operandeClone[i] = operande[i].getClone(listeSource, listeCible); 
    //  revoir car la fonction a dj t cre
    int ind1 = listeSource.indexOf(fonctionAssociee);    
    CCalculBase ptcalc = new CAppelFonctionNVar(listeCible, nbVar, (CFoncNVar)(listeCible.get(ind1, CFoncNVar.class)),
      operandeClone);
    return ptcalc;
  }
  
  @Override
  public boolean existe(boolean infoRandom) {
    boolean res = fonctionAssociee.existe;
    if (!res) return false;
    for (int i = 0; i < nbVar; i++) res = res || operande[i].existe(infoRandom);
    return res;
  }
  
  @Override
  public double resultat(boolean infoRandom) throws ErreurCalculException {
  	double[] d = new double[nbVar];
  	for (int i = 0; i < nbVar; i++) d[i] = operande[i].resultat(infoRandom);
    return fonctionAssociee.rendValeurFonction(infoRandom, d);
  }
  
  
  @Override
  public double resultatFonction(boolean infoRandom, double valeurParametre) throws ErreurCalculException {
  	double[] d = new double[nbVar];
  	for (int i = 0; i < nbVar; i++) d[i] = operande[i].resultatFonction(infoRandom, valeurParametre);
    return fonctionAssociee.rendValeurFonction(infoRandom, d);
  }

  // A revoir version 3.0
  @Override
  public double resultatFonction(boolean infoRandom, double[] valeurParametre) throws ErreurCalculException {
  	double[] d = new double[nbVar];
  	for (int i = 0; i < nbVar; i++) d[i] = operande[i].resultatFonction(infoRandom, valeurParametre);
    return fonctionAssociee.rendValeurFonction(infoRandom, d);
  }
  
  // Ajout version 4.9.7 pour qu'un calcul ou fonction complexe puisse utiliser une fonction relle.
  // Transfr depuis CAppelFonctionComplexeNVar
  @Override
  void  resultatFonctionComplexe(boolean infoRandom, Complexe valeurParametre, Complexe zRes) 
    throws ErreurCalculException {
  	Complexe[] z1 = new Complexe[nbVar];
  	for (int i =0; i < nbVar; i++) z1[i] = new Complexe();
  	for (int i = 0; i < nbVar; i++)  operande[i].resultatFonctionComplexe(infoRandom, valeurParametre, z1[i]);
  	((CFoncComplexeNVar) fonctionAssociee).rendValeurFonctionComplexe(infoRandom, z1, zRes);
  }
 
  // Ajout version 4.9.7 pour qu'un calcul ou fonction complexe puisse utiliser une fonction relle.
  @Override
  void  resultatFonctionComplexe(boolean infoRandom, Complexe[] valeurParametre, Complexe zRes)
    throws ErreurCalculException {
  	Complexe[] z1 = new Complexe[nbVar];
  	for (int i =0; i < nbVar; i++) z1[i] = new Complexe();
  	for (int i = 0; i < nbVar; i++) operande[i].resultatFonctionComplexe(infoRandom, valeurParametre, z1[i]);
  	((CFoncComplexeNVar) fonctionAssociee).rendValeurFonctionComplexe(infoRandom, z1, zRes);
  }

  
  @Override
  public void initialisePourDependance() {
  	super.initialisePourDependance();
  	for (int i = 0; i < nbVar; i++) operande[i].initialisePourDependance();
  }

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
  	boolean res = super.dependDe(p); // modifi version 4.8
  	for (int i = 0; i < nbVar; i++) res = res || operande[i].dependDe(p);
  	elementTestePourDependDe = p;
    return memoriseDependance(fonctionAssociee.dependDe(p) || res);
  }

  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTeste;
  	boolean res = super.dependDePourReclassement(p);
  	for (int i = 0; i < nbVar; i++) res = res || operande[i].dependDePourReclassement(p);
    return memoriseDependancePouRec(fonctionAssociee.dependDePourReclassement(p) || res);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
  	boolean res = false;
  	for (int i = 0; i < nbVar; i++) res = res || operande[i].dependDePourBoucle(p);
    return fonctionAssociee.dependDePourBoucle(p) || res;
  }
  
  @Override
  public boolean dependDeVariable(int ind) {
  	boolean res = false;
  	// Modifi version 4.1
  	// for (int i = 0; i < nbVar; i++) res = res || operande[i].dependDeVariable(ind);
  	for (int i = 0; i < nbVar; i++) {
  		res = res || (fonctionAssociee.calcul.dependDeVariable(i) && operande[i].dependDeVariable(ind));
  	}
    return res;
  }
  
  public CCalculBase getCopie() {
  	CCalculBase[] operandeCopie = new CCalculBase[nbVar];
  	for (int i = 0; i < nbVar; i++) operandeCopie[i] = operande[i].getCopie();
    return new CAppelFonctionNVar(listeProprietaire, nbVar, fonctionAssociee, operandeCopie);
  }
  
  @Override
	public String chaineCalcul(String[] varFor) {
  	String ch = fonctionAssociee.nom() + "(";
  	for (int i = 0; i < nbVar - 1; i++) ch = ch + operande[i].chaineCalculSansPar(varFor)  + ",";
  	return ch + operande[nbVar - 1].chaineCalculSansPar(varFor) + ")";
	}
  
  @Override
	public String chaineLatex(String[] varFor) {
  	String ch = fonctionAssociee.nom() + "(";
  	for (int i = 0; i < nbVar - 1; i++) ch = ch + operande[i].chaineLatexSansPar(varFor)  + ",";
  	return ch + operande[nbVar - 1].chaineLatexSansPar(varFor) + ")";
	}
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	boolean res = listeOb.contains(fonctionAssociee) || fonctionAssociee.estDefiniParObjetDans(listeOb);
		for (int i = 0; i < nbVar; i++) res = res && operande[i].estDefiniParObjetDans(listeOb);
		return res;
	}	 
  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    nbVar = inps.readInt();
    int ind1 = inps.readInt();
    fonctionAssociee = (CFoncNVar) (list.get(ind1,CFoncNVar.class));
    operande = new CCalculBase[nbVar];
    for (int i = 0; i < nbVar; i++) operande[i] = (CCalculBase) (inps.readObject(list));
  }
  
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeInt(nbVar);
    int ind1 = list.indexOf(fonctionAssociee);
    oups.writeInt(ind1);
    for (int i = 0; i < nbVar; i++) oups.writeObject(operande[i]);
  }

  @Override
	public boolean estConstant() {
		boolean res = fonctionAssociee.estConstant();
		for (int i = 0; i < nbVar; i++) res = res && operande[i].estConstant();
		return res;
	}
  
  @Override
  public boolean deriveePossible(int indiceVariable) {
  	boolean b = true;
  	for (int i = 0; (i < operande.length) && b; i++) 
  		if (operande[i].dependDeVariable(indiceVariable)) 
  			b = fonctionAssociee.deriveePossible(i) && operande[i].deriveePossible(indiceVariable);
  	return b;
  }  
  
  // Ajout version 4.6
  @Override
  public CCalculBase calculAvecValeursRemplacees() {
  	int nbvar = fonctionAssociee.nombreVariables();
  	CCalculBase[] tabcal = new CCalculBase[fonctionAssociee.nombreVariables()];
  	for (int i = 0; i < nbvar; i++) {
  		tabcal[i] = operande[i].calculAvecValeursRemplacees();
  	}
  	return new CAppelFonctionNVar(listeProprietaire, nbvar, fonctionAssociee, tabcal);
  }

  @Override
	public CCalculBase membreGauche() {
  	return fonctionAssociee.membreGauche();
	}
	
	// Ajout version 4.6
	public CCalculBase membreDroit() {
  	return fonctionAssociee.membreDroit();
	}

}
