/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.geom.*;
import java.awt.Color;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

import mathgraph32.MtgFrame;

public class CDemiDroiteImage extends CDemiDroite {
	private CDemiDroite antecedent;
	private CTransformation transformation;
  private Point2D.Double point1 = new Point2D.Double();

  /** Creates new CDemiDroiteImage */
  public CDemiDroiteImage() {
  }
  // Constructeur pour clonage
  public CDemiDroiteImage(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean masqueInit, StyleTrait styleInit ,
      CDemiDroite antecedentInit, CTransformation transformationInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit);
    antecedent = antecedentInit;
    transformation = transformationInit;
  }  

  /** Fonction devant tre redfinie pour tous les objets images par une transformation
   *  Sera utilise lorsqu'on modifie une image par une transformation pour que cette image
   *  soit associe  une autre transformation lrsque d'autres objets taient des images par
   *  cette mme transformation
   */
  public void associeATransformation(CTransformation trans) {
  	transformation = trans;
  }
  
  @Override
  public String infoPourHistorique() {
  	return nom() + " : " + getString("chinfo36") + " " + antecedent.nom() + "\n" + getString("chinfo189") + " " +
        transformation.designationTransf();
  }
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
  	return nom() + " : " + getString("chinfo36") + " " + antecedent.nom() + " " + getString("chinfo189") + " " +
        transformation.complementInfo();
	}

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(antecedent);
    transformation.AjouteElementsVisuelsPourHist(liste);
  } 
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(antecedent);
    int ind2 = listeSource.indexOf(transformation);
    int ind3 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CDemiDroiteImage(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, masque, style, (CDemiDroite)(listeCible.get(ind1, CDemiDroite.class)),
        (CTransformation)(listeCible.get(ind2, CTransformation.class)));
    return ptelb;
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || antecedent.dependDe(p) || transformation.dependDe(p));
  }

  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || antecedent.dependDePourReclassement(p) || transformation.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || antecedent.dependDePourBoucle(p) || transformation.dependDePourBoucle(p));
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
		existe = transformation.existe && antecedent.existe;
		if (!existe) return;
		transformation.transformePoint(antecedent.point_x, antecedent.point_y, point1);
    point_x = point1.x;
    point_y = point1.y;
		transformation.transformeVecteur(antecedent.vecteur, vecteur);
		super.positionne(infoRandom, dimfen);
  }  
  
  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne.
  **/
  public boolean confonduAvec(CElementBase p) {
    CDemiDroiteImage pt;
    if (p.getClass() == this.getClass()) {
      pt = (CDemiDroiteImage) p;
      return (antecedent == pt.antecedent)
			&& (transformation.confonduAvec(pt.transformation));
    }
    else return false;
  }
  
  @Override
  public CElementBase antecedentDirect() {
    return transformation;
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(antecedent) || antecedent.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(transformation) || transformation.estDefiniParObjetDans(listeOb));
	}  	        

  @Override
  public boolean modifiableParMenu() {
  	// Modifi version 3.5
  	// return transformation.imageModifiableParMenu();
  	return !estElementFinal && transformation.imageModifiableParMenu();
  }
    
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    int ind2 = inps.readCInt();
    antecedent = (CDemiDroite) (list.get(ind1,CDemiDroite.class));
    transformation = (CTransformation) (list.get(ind2,CTransformation.class));
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    antecedent = (CDemiDroite) (list.get(ind1,CDemiDroite.class));
    transformation = (CTransformation) (list.get(ind2,CTransformation.class));
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(antecedent);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(transformation);
    oups.writeInt(ind2);
  }
 
  @Override
  public boolean contientParDefinition(CPointAncetre po) {
  	if (po.estPointImage()) {
  		CPointImage ptim = (CPointImage) po;
  		return (transformation == ptim.transformation) && (ptim.antecedent.appartientParDefinition(antecedent));
  	}
  	else return false;
  }
  
  @Override
  public boolean estParalleleParDefinition(CDroiteAncetre dte){
  	return (dte == this) || 
  			(((transformation.natureTransformation() & (CTransformation.translation | CTransformation.homothetie | CTransformation.symetrieCentrale)) != 0)
  			&& (antecedent.estParalleleParDefinition(dte) || dte.estParalleleParDefinition(antecedent)));
  }
  
}
