/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


package objets;

import java.io.IOException;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.Kernel;
import types.Complexe;
import types.ErreurCalculException;
import types.OperateurFonction2Var;


public class CFonction2Var extends CCalculBase {
	
  // Attibut rajout pour optimisation
  Complexe z1loc = new Complexe();
  Complexe z2loc = new Complexe();
	
  int operateurFonction;
  CCalculBase operande1, operande2;

  /** Creates new CFonction */
  public CFonction2Var() {
  }

  /** Constructeur pour clonage */
  public CFonction2Var(CListeObjets liste, int operateurFonctionInit, CCalculBase operande1Init, CCalculBase operande2Init) {
    super(liste);
    operateurFonction = operateurFonctionInit;
    operande1 = operande1Init;
    operande2 = operande2Init;
  }
  
  @Override
  public int nombreVariables() {
  	return 2;
  }
  
  @Override
  protected long nature() {
  	return natFonction2Var;
  }  

  public CCalculBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    CCalculBase cloneOperande1 = operande1.getClone(listeSource, listeCible); 
    CCalculBase cloneOperande2 = operande2.getClone(listeSource, listeCible); 
    CCalculBase ptcalc = new CFonction2Var(listeCible, operateurFonction, cloneOperande1, cloneOperande2);
    return ptcalc;
  }

  // Ajout version 4.8
  @Override
  public void initialisePourDependance() {
  	super.initialisePourDependance();
  	operande1.initialisePourDependance();
  	operande2.initialisePourDependance();
  }

  // Modifi version 4.8
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
  	// elementTestePourDependDe = p; // Modifi version 4.8
    return memoriseDependance(super.dependDe(p) || operande1.dependDe(p) || operande2.dependDe(p));
  }
  
  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePouRec(super.dependDePourReclassement(p) || operande1.dependDePourReclassement(p) || operande2.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return operande1.dependDePourBoucle(p) || operande2.dependDePourBoucle(p);
  }
  
  @Override
  public boolean existe(boolean infoRandom) {
    return operande1.existe(infoRandom) && operande2.existe(infoRandom);
  }
  
  public double resultatBase(double x, double y) throws ErreurCalculException {
    switch (operateurFonction)
    {
      case OperateurFonction2Var.maxi:
      	if (x > y) return x; else return y;
      case OperateurFonction2Var.mini:
      	if (x < y) return x; else return y;
      case OperateurFonction2Var.gcd:
      	if ((x != Math.floor(x)) || (y != Math.floor(y)) || ((x == 0) && (y == 0))
      			|| (x < 0) || (y < 0))
      		throw new ErreurCalculException();
      	else return Kernel.pgcd(x, y);
      case OperateurFonction2Var.lcm:
      	if ((x != Math.floor(x)) || (y != Math.floor(y)) || (x < 0) || (y < 0))
      		throw new ErreurCalculException();
      	else return Kernel.ppcm(x, y);
      case OperateurFonction2Var.ncr:
      	if ((x != Math.floor(x)) || (y != Math.floor(y)) || (x < 0) || (y < 0) || (y > x))
      		throw new ErreurCalculException();
      	else return Kernel.ncr(x, y);
      case OperateurFonction2Var.npr:
      	if ((x != Math.floor(x)) || (y != Math.floor(y)) || (x < 0) || (y < 0) || (y > x))
      		throw new ErreurCalculException();
      	else return Kernel.npr(x, y);
      case OperateurFonction2Var.mod:
      	if ((x != Math.floor(x)) || (y != Math.floor(y)) || (x < 0) || (y <= 0))
      		throw new ErreurCalculException();
      	else return Math.floor(x - Math.floor(x/y)*y + 0.5);
      	
      default : throw new ErreurCalculException();
    }
  }
  
  @Override
  public double resultat(boolean infoRandom) throws ErreurCalculException {
    double x1 = operande1.resultat(infoRandom);
    double y1 = operande2.resultat(infoRandom);
    return resultatBase(x1, y1);
  }

  @Override
  public double resultatFonction(boolean infoRandom, double valeurParametre) throws ErreurCalculException {
    double x1 = operande1.resultatFonction(infoRandom, valeurParametre);
    double y1 = operande2.resultatFonction(infoRandom, valeurParametre);
    return resultatBase(x1, y1);
  }
  
  @Override
  public double resultatFonction(boolean infoRandom, double[] valeurParametre) throws ErreurCalculException {
    double x1 = operande1.resultatFonction(infoRandom, valeurParametre);
    double y1 = operande2.resultatFonction(infoRandom, valeurParametre);
    return resultatBase(x1, y1);
  }
  
  void resultatComplexeBase(boolean infoRandom, Complexe z1, Complexe z2, Complexe zRes)  throws ErreurCalculException {
  	zRes.y = 0;
    if ((z1.y != 0) || (z2.y != 0)) throw new ErreurCalculException();
    switch (operateurFonction)
    {
      case OperateurFonction2Var.maxiC:
      	if (z1.x > z2.x) zRes.x = z1.x; else zRes.x = z2.x;
      	break;
      case OperateurFonction2Var.miniC:
      	if (z1.x < z2.x) zRes.x = z1.x; else zRes.x = z2.x;
      	break;
      case OperateurFonction2Var.gcdC:
      	// Corrig version 4.7.9
      	/*
      	if ((z1.x != Math.floor(z1.x)) || (z2.x != Math.floor(z2.x)) || ((z1.x == 0) && (z2.x == 0))
      			|| (z1.x < 0) || (z1.x < 0))
      		throw new ErreurCalculException();
      	else zRes.x = Kernel.pgcd(z1.x, z2.x);
      	break;
      	*/
      	if ((z1.x != Math.floor(z1.x)) || (z2.x != Math.floor(z2.x)) || ((z1.x <= 0) && (z2.x <= 0)))
      		throw new ErreurCalculException();
      	else zRes.x = Kernel.pgcd(z1.x, z2.x);
      	break;
      case OperateurFonction2Var.lcmC:
      	if ((z1.x != Math.floor(z1.x)) || (z2.x != Math.floor(z2.x)) || (z1.x < 0) || (z2.x < 0))
      		throw new ErreurCalculException();
      	else zRes.x = Kernel.ppcm(z1.x, z2.x);
      	break;
      case OperateurFonction2Var.ncrC:
      	if ((z1.x != Math.floor(z1.x)) || (z2.x != Math.floor(z2.x)) || (z1.x < 0) || (z2.x < 0) || (z2.x > z1.x))
      		throw new ErreurCalculException();
      	else zRes.x = Kernel.ncr(z1.x, z2.x);
      	break;
      case OperateurFonction2Var.nprC:
      	if ((z1.x != Math.floor(z1.x)) || (z2.x != Math.floor(z2.x)) || (z1.x < 0) || (z2.x < 0) || (z2.x > z1.x))
      		throw new ErreurCalculException();
      	else zRes.x = Kernel.npr(z1.x, z2.x);
      	break;
      case OperateurFonction2Var.modC:
      	if ((z1.x != Math.floor(z1.x)) || (z2.x != Math.floor(z2.x)) || (z1.x < 0) || (z2.x <= 0))
      		throw new ErreurCalculException();
      	else zRes.x =  Math.floor(z1.x - Math.floor(z1.x/z2.x)*z2.x + 0.5);
      	break;
      default : throw new ErreurCalculException();
    }
  }
  
  public void resultatComplexe(boolean infoRandom, Complexe zRes)  throws ErreurCalculException {
    operande1.resultatComplexe(infoRandom, z1loc);
    operande2.resultatComplexe(infoRandom, z2loc);
    resultatComplexeBase(infoRandom, z1loc, z2loc, zRes);
  }
  
  void  resultatFonctionComplexe(boolean infoRandom, Complexe valeurParametre, Complexe zRes)
    throws ErreurCalculException 
  {
    operande1.resultatFonctionComplexe(infoRandom, valeurParametre, z1loc);
    operande2.resultatFonctionComplexe(infoRandom, valeurParametre, z2loc);
    resultatComplexeBase(infoRandom, z1loc, z2loc, zRes);
  }
  
  void  resultatFonctionComplexe(boolean infoRandom, Complexe valeurParametre[], Complexe zRes)
    throws ErreurCalculException 
  {
    operande1.resultatFonctionComplexe(infoRandom, valeurParametre, z1loc);
    operande2.resultatFonctionComplexe(infoRandom, valeurParametre, z2loc);
    resultatComplexeBase(infoRandom, z1loc, z2loc, zRes);
  }
  
  
  public boolean  dependDeVariable(int ind) {
    return operande1.dependDeVariable(ind) || operande2.dependDeVariable(ind);
  }
  
  public CCalculBase getCopie() {
   	return new CFonction2Var(listeProprietaire, operateurFonction, operande1.getCopie(), operande2.getCopie());
  }
  
  @Override
	public String chaineCalcul(String[] varFor) {
  	String cha1 = operande1.chaineCalculSansPar(varFor);
  	String cha2 = operande2.chaineCalculSansPar(varFor);
  	String ch = parouv + cha1 + virg + cha2 + parfer;

  	switch (operateurFonction)
  	{
  		case OperateurFonction2Var.maxi :
  		case OperateurFonction2Var.maxiC :
  			return getString("maxi") + ch;
  		case OperateurFonction2Var.mini :
  		case OperateurFonction2Var.miniC :
  			return getString("mini") + ch;
  		case OperateurFonction2Var.gcd :
  		case OperateurFonction2Var.gcdC :
  			return getString("gcd") + ch;
  		case OperateurFonction2Var.lcm :
  		case OperateurFonction2Var.lcmC :
  			return getString("lcm") + ch;
  		case OperateurFonction2Var.ncr :
  		case OperateurFonction2Var.ncrC :
  			return getString("ncr") + ch;
  		case OperateurFonction2Var.npr :
  		case OperateurFonction2Var.nprC :
  			return getString("npr") + ch;
  		case OperateurFonction2Var.mod :
  		case OperateurFonction2Var.modC :
  			return getString("mod") + ch;
  		default : return "";
  	}
  }

  @Override
	public String chaineLatex(String[] varFor) {
  	String cha1 = operande1.chaineLatexSansPar(varFor);
  	String cha2 = operande2.chaineLatexSansPar(varFor);
  	String ch = parouvLatex + cha1 + virg + cha2 + parferLatex;

  	switch (operateurFonction)
  	{
  		case OperateurFonction2Var.maxi :
  		case OperateurFonction2Var.maxiC :
  			return getString("maxi") + ch;
  		case OperateurFonction2Var.mini :
  		case OperateurFonction2Var.miniC :
  			return getString("mini") + ch;
  		case OperateurFonction2Var.gcd :
  		case OperateurFonction2Var.gcdC :
  			return getString("gcd") + ch;
  		case OperateurFonction2Var.lcm :
  		case OperateurFonction2Var.lcmC :
  			return getString("lcm") + ch;
  		case OperateurFonction2Var.ncr :
  		case OperateurFonction2Var.ncrC :
  			return getString("ncr") + ch;
  		case OperateurFonction2Var.npr :
  		case OperateurFonction2Var.nprC :
  			return getString("npr") + ch;
  		case OperateurFonction2Var.mod :
  		case OperateurFonction2Var.modC :
  			return getString("mod") + ch;
  		default : return "";
  	}
  }

  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return operande1.estDefiniParObjetDans(listeOb) && operande2.estDefiniParObjetDans(listeOb);
	}	   
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    operateurFonction = inps.readByte();
    operande1 = (CCalculBase) inps.readObject(list);
    operande2 = (CCalculBase) inps.readObject(list);
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeByte(operateurFonction);
    oups.writeObject(operande1);
    oups.writeObject(operande2);
  }
  
  @Override
	public boolean estConstant() {
		return (operande1.estConstant() && operande2.estConstant());
	}
  
  @Override
  public boolean deriveePossible(int indiceVariable) {
  	return operande1.deriveePossible(indiceVariable) && operande2.deriveePossible(indiceVariable);
  }

  // Ajout version 4.6
  @Override
  public CCalculBase calculAvecValeursRemplacees() {
  	return new CFonction2Var(listeProprietaire, operateurFonction,
  			operande1.calculAvecValeursRemplacees(), operande2.calculAvecValeursRemplacees());
  }
  
}
