/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.Color;

import mathgraph32.MtgFrame;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

/**
 *
 * @author Yves
 */
public class CLieuObjetParPtLie extends CLieuObjetAncetre {
  // Numro de version de l'objet. 1 par dfaut.
  private CPointLie pointLieGenerateur; // Le point li dont les positions successives gnreront la trace de l'objet
  
  /** Creates a new instance of CLieuObjetParPtLie */
  public CLieuObjetParPtLie() {
  }
  
  // Constructeur pour clonage
  public CLieuObjetParPtLie(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit,
    CElementGraphique elementAssocieInit, CValeurAssocieeAVariable nombreTracesInit,
    CPointLie pointLieGenerateurInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit,
      masqueInit, elementAssocieInit,  nombreTracesInit);
    pointLieGenerateur = pointLieGenerateurInit;
    // Rajout version 3.4.2
    // etablitListeElementsAncetres(); Inutile et provoquait un bug sous version 4.4 en appelant getClone
  }
  
  /**
   * A partir de la version 3.4.2, changement de version (version 1 -> 2) car on enregistre dans le flux la CSousListeObjetstres
   * contenant les pointeurs sur les lments anctres
   */
  @Override
  public int numeroVersion() {
  	return 2;
  }  
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(elementAssocie);
    int ind2 = listeSource.indexOf(pointLieGenerateur);
    int ind3 = listeSource.indexOf(impProto);
    CValeurAssocieeAVariable nombreTracesClone = (CValeurAssocieeAVariable) nombreTraces.getClone(listeSource, listeCible);
    
    CElementBase ptelb = new CLieuObjetParPtLie(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, masque, (CElementGraphique)(listeCible.get(ind1, CElementGraphique.class)), nombreTracesClone,
      	(CPointLie)(listeCible.get(ind2, CPointLie.class)));
    if (listeCible.getClass() != CPrototype.class) {
	    ((CLieuObjetParPtLie)ptelb).listeElementsAncetres.setImage(listeElementsAncetres, listeProprietaire,
	      ptelb.listeProprietaire);
	    // Modification version 2.0
    // Comme le nombre d'objets est dynamique on ne peut pas savoir au chargement combien il y en a.
    // On laisse la liste vide et cela sera mis  jour lorsque positionne() sera appel
    // if (listeSource.getClass() != CPrototype.class) ((CLieuObjetParPtLie)ptelb).listeCopiesObjet.ajouteClonesDe(((CLieuObjetParPtLie)ptelb).elementAssocie,
	  //    listeCopiesObjet.size());
	    
    }
    return ptelb;
  }
  
  @Override
  public String infoPourHistorique() {
		return nom() + " : " + getString("natLieuObjets") + " " + getString("ch31") + " " + elementAssocie.nom() + "\n "
				+ getString("chinfo207") + " " + pointLieGenerateur.nom();
  }
	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
		return nom() + " : " + getString("natLieuObjets") + " " + getString("ch31") + " " + elementAssocie.nom() + " "
					+ getString("chinfo207") + " " + pointLieGenerateur.nom();
	}

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(pointLieGenerateur);
    liste.add(elementAssocie);
  } 
  
    
  /**
   * Fonction n'existant pas en C++ et charge de mettre  jour l'objet quand un lment dont il dpend a t modifi.
   * Devra tre appel par les lieux de points et lieux d'objets pour remettre  jour les liste qu'ils utilisent
   * de faon interne
   * Pour les lieux d'objets il faut aussi recrer la liste d'objets clones servant pour la trace
   */
  @Override
  public void metAJour() {
  	super.metAJour();
  	// Bug corrig version 4.0 : il faut mettre  jour la liste d'lments anctres car metAJour est appel quand
  	// on modifie un obejt dont dpend cet objet
  	// if (nVersion == 1) etablitListeElementsAncetres();
  	etablitListeElementsAncetres();
  }
  
  public boolean dependDePourCapture(CElementBase p) {
    return (pointLieGenerateur != p) && (((p == this)
    || elementAssocie.dependDe(p)) || nombreTraces.dependDe(p));
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    int nbValeurs;
    double abscisseMini,abscisseMaxi,abscisse,abscisseInitiale,pas,abscisseFinale;
    super.positionne(infoRandom, dimfen);
    existe = existe && pointLieGenerateur.existe && (nombreObjetsPourLieuObjet() <= nombreMaxiPourLieuObjet) ;
    if (!existe) return;
    double dnbt = Math.round(nombreTraces.rendValeur());
    existe = (dnbt>=2) && (dnbt<=nombreMaxiPourLieuObjet);
    if (!existe) return;
    int nbt = (int) dnbt;
    
    // Changement important de la version 2.5, le nombre de traces tant devenu dynamique
    if (nbt != listeCopiesObjet.size()) ajusteListeCopieObjets();   
    abscisseMini = pointLieGenerateur.abscisseMinimale();
    abscisseMaxi = pointLieGenerateur.abscisseMaximale();
    if (abscisseMini == abscisseMaxi) {
      existe = false;
      return;
    } else {
      if (abscisseMini > abscisseMaxi) {
        double w = abscisseMini;
        abscisseMini = abscisseMaxi;
        abscisseMaxi = w;
      }
    }
    // Pour tenir compte des points lis lis  des lieux ferms
    if (pointLieGenerateur.lieALigneFermee()) {
    	nbValeurs = nbt;
    	pas = (abscisseMaxi-abscisseMini)/nbValeurs;
    	abscisseFinale = abscisseMaxi - pas;
    }
    else {
    	nbValeurs = nbt - 1;
    	pas = (abscisseMaxi-abscisseMini)/nbValeurs;
    	abscisseFinale = abscisseMaxi;
    }
    abscisseInitiale = pointLieGenerateur.abscisse;
    abscisse = abscisseMini;
    
    existe = false;
    boolean elementAssocieMasque = elementAssocie.masque;
    elementAssocie.montre();
    // Modifi version 3.9.1 pour viter des erreurs d'arrondi
    int nb = nbt - 1;
    for (int i = 0; i < nbt; i++) {
      // Modifi version 3.9.1 pour viter des erreurs d'arrondi
    	abscisse = (i == 0) ? abscisseMini : ((i == nb) ? abscisseFinale : abscisse + pas);
      pointLieGenerateur.donneAbscisse(abscisse);
      listeElementsAncetres.positionne(infoRandom, dimfen);
      existe = existe || elementAssocie.existe;
      CElementGraphique ptelg = (CElementGraphique)listeCopiesObjet.get(i);
      ptelg.setClone(elementAssocie);
    }
    elementAssocie.masque = elementAssocieMasque;
    pointLieGenerateur.donneAbscisse(abscisseInitiale);
    listeElementsAncetres.positionne(infoRandom, dimfen);
  }
  
  @Override
  public void positionneTikz(boolean infoRandom, DimensionFenetre dimfen) {
    int nbValeurs;
    double abscisseMini,abscisseMaxi,abscisse,abscisseInitiale,pas,abscisseFinale;
    super.positionneTikz(infoRandom, dimfen);
    existe = existe && pointLieGenerateur.existe && (nombreObjetsPourLieuObjet() <= nombreMaxiPourLieuObjet) ;
    if (!existe) return;
    double dnbt = Math.round(nombreTraces.rendValeur());
    existe = (dnbt>=2) && (dnbt<=nombreMaxiPourLieuObjet);
    if (!existe) return;
    int nbt = (int) dnbt;
    
    // Changement important de la version 2.5, le nombre de traces tant devenu dynamique
    if (nbt != listeCopiesObjet.size()) ajusteListeCopieObjets();   
    abscisseMini = pointLieGenerateur.abscisseMinimale();
    abscisseMaxi = pointLieGenerateur.abscisseMaximale();
    if (abscisseMini == abscisseMaxi) {
      existe = false;
      return;
    } else {
      if (abscisseMini > abscisseMaxi) {
        double w = abscisseMini;
        abscisseMini = abscisseMaxi;
        abscisseMaxi = w;
      }
    }
    // Pour tenir compte des points lis lis  des lieux ferms
    if (pointLieGenerateur.lieALigneFermee()) {
    	nbValeurs = nbt;
    	pas = (abscisseMaxi-abscisseMini)/nbValeurs;
    	abscisseFinale = abscisseMaxi - pas;
    }
    else {
    	nbValeurs = nbt - 1;
    	pas = (abscisseMaxi-abscisseMini)/nbValeurs;
    	abscisseFinale = abscisseMaxi;
    }
    abscisseInitiale = pointLieGenerateur.abscisse;
    abscisse = abscisseMini;
    
    existe = false;
    boolean elementAssocieMasque = elementAssocie.masque;
    elementAssocie.montre();
    // Modifi version 3.9.1 pour viter des erreurs d'arrondi
    int nb = nbt - 1;
    for (int i = 0; i < nbt; i++) {
      // Modifi version 3.9.1 pour viter des erreurs d'arrondi
    	abscisse = (i == 0) ? abscisseMini : ((i == nb) ? abscisseFinale : abscisse + pas);
      pointLieGenerateur.donneAbscisse(abscisse);
      listeElementsAncetres.positionneTikz(infoRandom, dimfen);
      existe = existe || elementAssocie.existe;
      CElementGraphique ptelg = (CElementGraphique)listeCopiesObjet.get(i);
      ptelg.setCloneTikz(elementAssocie);
    }
    elementAssocie.masque = elementAssocieMasque;
    pointLieGenerateur.donneAbscisse(abscisseInitiale);
    listeElementsAncetres.positionne(infoRandom, dimfen);
  }

  
  @Override
  public boolean confonduAvec(CElementBase p) {
    CLieuObjetParPtLie pt;
    if (p.getClass() == this.getClass()) {
      pt = (CLieuObjetParPtLie) p;
  		return (pointLieGenerateur == pt.pointLieGenerateur)
			&& (elementAssocie == pt.elementAssocie)
			&& nombreTraces.confonduAvec(pt.nombreTraces);    
		}
    else return false;
  }  
  
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		super.estDefiniParObjetDans(listeOb)
		&&
		(listeOb.contains(pointLieGenerateur) || pointLieGenerateur.estDefiniParObjetDans(listeOb))
		&& nombreTraces.estDefiniParObjetDans(listeOb);
	}     

  public void etablitListeElementsAncetres() {
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    if (listeProprietaire.getClass() != CPrototype.class) {
	    listeElementsAncetres.retireTout();
	    int indiceElementAssocie = listeProprietaire.indexOf(elementAssocie);
	    int indicePointLieGenerateur = listeProprietaire.indexOf(pointLieGenerateur);
    	listeElementsAncetres.add(pointLieGenerateur);
      for (int i = indicePointLieGenerateur + 1; i < indiceElementAssocie; i++) {
        CElementBase ptelb = (CElementBase)(listeProprietaire.get(i));
        if ((elementAssocie.dependDe(ptelb)) && (ptelb.dependDe(pointLieGenerateur)))
          listeElementsAncetres.add(ptelb);
      }
    listeElementsAncetres.add(elementAssocie);
    }	
  }
  
	/**
	 * Fonction renvoyant true si l'objet dpend des positions gnres pas les dplacements d'un point li
	 * A redfinir pour les lieux de points et lieux d'objets et macros utilisant les dplacements d'un point li
	 * @param pointlie
	 * @return
	 */
  @Override
	public boolean estGenereParPointLie(CPointLie pointlie) {
		return pointLieGenerateur == pointlie;
	}
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    if (nVersion == 1) {
    	super.chargeDepuisFlux(inps, list);
    	nVersion = numeroVersion(); // On remet tout  zro pour la version java
    }
    else super.chargeDepuisFluxNew(inps, list);
    int ind1 = inps.readCInt();
    pointLieGenerateur = (CPointLie) (list.get(ind1,CPointLie.class));
    // Il faut prparer la liste des lments dont dpend l'objet  tracer
    // Version 1.9.5 : modifi pour optimisation
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    if (list.getClass() != CPrototype.class) {
      int indiceElementAssocie = list.indexOf(elementAssocie);
      for (int i = 0; i <= indiceElementAssocie; i++) {
        CElementBase ptelb = (CElementBase)(list.get(i));
        if ((elementAssocie.dependDe(ptelb)) && (ptelb.dependDe(pointLieGenerateur)))
          listeElementsAncetres.add(ptelb);
      }
    }
    nVersion = numeroVersion(); // On remet tout  zro pour la version java
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    pointLieGenerateur = (CPointLie) (list.get(ind1,CPointLie.class));
    // Il faut prparer la liste des lments dont dpend le point  tracer
    // Version 1.9.5 : modifi pour optimisation
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    // Modifi version 3.4.2
    /*
    if (list.getClass() != CPrototype.class) {
      int indiceElementAssocie = list.indexOf(elementAssocie);
      for (int i = 0; i <= indiceElementAssocie; i++) {
        CElementBase ptelb = (CElementBase)(list.get(i));
        if ((elementAssocie.dependDe(ptelb)) && (ptelb.dependDe(pointLieGenerateur)))
          listeElementsAncetres.add(ptelb);
      }
    }
    */
    if (nVersion == 1) etablitListeElementsAncetres();
    else listeElementsAncetres.read(inps, list);
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(pointLieGenerateur);
    oups.writeInt(ind1);
    listeElementsAncetres.write(oups, listeProprietaire);
  }
  
}