/*
 * MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package objets;

// Ajout version 4.8
import java.awt.Color;
import java.io.IOException;
import java.util.ListIterator;
import java.util.Vector;
import dialogs.MacroConstructionIterativeDlg;
import dialogs.MtgDlg;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import outils.Outil;
import types.NatureObjet;
import types.StyleRemplissage;
import types.StyleTrait;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class CMacroConstructionIterative extends CMacroAvecListe {
	public CPrototype proto; // La construction qui sera excute par la macro
	public int nbIter; // Le nombre d'itrations choisi
	public int nbSourcesPourIter; // Le nombre d'objets sources communs  chaque itration
	public boolean imposerCouleur; // Si on souhaite donner la mme couleur  tous les objets finaux graphiques crs
	public Color couleurImp; // La couleur des objets crs dans le cas o imposerCouleur est true
	public int choixLigne; // Choix de joindre les points crs par rien, une ligne brise ou un polygone
	// Modifi version 5.0
	// public int styleLigne; // Dans le cas d'une ligne, style de trait de la ligne
	public StyleTrait styleLigne;
	public Color couleurLigne; //  Dans le cas d'une ligne, style de trait de la ligne
	public boolean polygoneRempli; // Dans le cas o un polygone est demand, cre la surface qu'il dlimite
	private boolean dejaExecute = false;

	/**
	 * Constructeur par dfaut
	 */
	public CMacroConstructionIterative() {
		dejaExecute = false;
	}
	
	// Constructeur pour clonage
	public CMacroConstructionIterative(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
	  Color couleurInit, double x_nomInit, double y_nomInit, double dec_xInit, double dec_yInit, boolean masqueInit,
	  CPointAncetre pointLieInit, int indiceTaillePoliceInit, int encadrementInit,
	  boolean effacementFondInit, Color couleurFondInit, String intituleInit, String commentaireMacroInit,
	  CSousListeObjets listeAssocieeInit, int alignementHorizontalInit, int alignementVerticalInit,
	  CPrototype protoInit, int nbIterInit, int nbSourcesPourIterInit, boolean imposerCouleurInit, Color couleurImpInit,
	  int choixLigneInit, StyleTrait styleLigneInit, Color couleurLigneInit, boolean polygoneRempliInit) {
	  super(liste, impProtoInit, estElementFinalInit, couleurInit, x_nomInit, y_nomInit,
	    dec_xInit, dec_yInit, masqueInit, pointLieInit, indiceTaillePoliceInit, encadrementInit,
	    effacementFondInit, couleurFondInit, intituleInit, commentaireMacroInit, listeAssocieeInit,
	    alignementHorizontalInit, alignementVerticalInit);
	  proto = protoInit;
	  nbIter = nbIterInit;
	  nbSourcesPourIter = nbSourcesPourIterInit;
	  imposerCouleur = imposerCouleurInit;
	  couleurImp = couleurImpInit;
	  choixLigne = choixLigneInit;
	  styleLigne = styleLigneInit;
	  couleurLigne = couleurLigneInit;
	  polygoneRempli = polygoneRempliInit;
		dejaExecute = false;
	}
	
	@Override
	public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
	  int ind1 = listeSource.indexOf(pointLie);
	  int ind2 = listeSource.indexOf(impProto);
	  CSousListeObjets listeAssocieeClone = new CSousListeObjets();
	  listeAssocieeClone.setImage(listeAssociee, listeSource, listeCible);
	  CElementBase ptelb = new CMacroConstructionIterative(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
	    estElementFinal, couleur, x_nom, y_nom, dec_x, dec_y, masque,
	    (CPointAncetre)listeCible.get(ind1, CPointAncetre.class), indiceTaillePolice, encadrement,
	    effacementFond, couleurFondAffichage, intitule, commentaireMacro, listeAssocieeClone,
	    alignementHorizontal, alignementVertical, proto, nbIter, nbSourcesPourIter, imposerCouleur,
	    couleurImp, choixLigne, styleLigne, couleurLigne, polygoneRempli);
	  return ptelb;
	}
	
  /**
   * Ajout version 4.8 : Renvoie true si la macro est une CMacroConstructionIterative ou une CMacroConstructionRecursive
   * ou une CMacroLSuiteMacros ne contenant que des macros du type prcdent.
   * Redfini pour CMacroConstructionIteartive, CMacroConnstructionRecursive et CMacroSuiteMacros
   * @return
   */
  public boolean utilisablePourDemarrage() {
  	return true;
  }

  /**
   * Si la figure comporte une longuer unit et si la construction associe utilise la longueur unit, dependDe doit rendre true
   */
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
  	if (p == listeProprietaire.pointeurLongueurUnite) return  memoriseDependance(super.dependDe(p) || proto.utiliseLongueurUnite()) ;
  	else return memoriseDependance(super.dependDe(p));
  }

	@Override
	public boolean dependDeProto(CPrototype proto) {
		return this.proto == proto;
	}
	
  @Override
  public String infoPourHistorique() {
    String ch = nom()+ " : " + getString("macroConstructionIterativeDlg1") + "\n" +
    	getString("ch31") + " " + proto.nom;
    ch += "\n" + getString("macroApparitionDlg2") + " " + intitule;
  	if (pointLie != null) ch += "\n" + getString("chinfo200") + " " + pointLie.nom();
  	if (effacementFond) ch += "\n" + getString("paneAffichage1");
  	return ch;
  }
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
    return nom()+ " : " + getString("macroConstructionIterativeDlg1") + " " + getString("ch31") + " " + proto.nom;
	}

	@Override
	public boolean executionPossible() {
		// On recherche si la construction a dja t implmente avec les mmes objets sources
		// Sinon on permet l'excution mme si la macro a dj t excute
		CListeObjets liste = listeProprietaire;
		ListIterator<CElementBase> li = liste.listIterator();
		boolean trouve = false;
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.getNature() == NatureObjet.NImplementationProto) {
				CImplementationProto imp = (CImplementationProto) elb;
				if ((imp.nomProto == proto.nom) && (imp.listeSources.size() == listeAssociee.size())) {
					boolean res = true;
					for (int i = 0; i < imp.listeSources.size(); i++) {
						if (imp.listeSources.get(i) != listeAssociee.get(i)) {
							res = false;
							break;
						}
					}
					if (res) {
						trouve = true;
						break;
					}
				}
			}
		}
		if (trouve) return !dejaExecute;
		else return true;
	}
	
  @Override
  public void actionAvantExecution(MtgInternalFrame cadre) { // par dfaut, met executionEnCours  true;
  	super.actionAvantExecution(cadre);  // Met executionEnCours  true
  	// On implmente la construction de faon itrative
  	// La liste associe contient les pointeurs sur les objets sources
  	// On affecte ces lments aux objets sources du prototype
  	for (int i = 0; i < proto.nbObjSources; i++) ((CElementGenerique)proto.get(i)).elementAssocie = listeAssociee.get(i);
		CListeObjets liste = cadre.listePr;
  	Vector<CNoeudPointeurSurPoint> listePoints = new Vector<CNoeudPointeurSurPoint> (); // Utilis si on veut que les points crs soient relis.
    if (choixLigne != Outil.sansLigne) {
    	for (int i = nbSourcesPourIter; i < proto.nbObjSources; i++) {
    		CElementGenerique gen = (CElementGenerique) proto.get(i);
    		if (gen.estDeNature(NatureObjet.NToutPoint)) {
    			listePoints.add(new CNoeudPointeurSurPoint(liste, (CPointAncetre)gen.elementAssocie));
    			break;
    		}
    	}
    }
		// CLongueur ancienneLongueurUnite = liste.pointeurLongueurUnite;
		CImplementationProto impProto = new CImplementationProto(liste, proto);
		// On implmente la construction une premire fois.
		int indiceImp = liste.size();
		int indiceImpInitial = indiceImp;
		impProto.implementePourIter(liste, proto, cadre); // Modifi version 4.8
		// liste.reconstruitChainesCalcul(indiceImpInitial + 1, indiceImpInitial + proto.size() - proto.nbObjSources);
		if (imposerCouleur) {
	    for (int i= indiceImp + 1; i < liste.size(); i++) {
	    	CElementBase elb = liste.get(i);
	    	if (elb.estElementFinal && (elb.estDeNature(NatureObjet.NToutObjet))) {
	    		CElementGraphique elg = (CElementGraphique) elb;
	    		elg.donneCouleur(couleurImp);
	    	}
	    }
		}
		for (int k = 1; k < nbIter; k++) {
			// On implmente avec les objets finaux sauf ceux qui sont communs aux implmentations
			for (int i = nbSourcesPourIter; i < proto.nbObjSources; i++) {
				CElementBase elb = liste.getFinal(indiceImp, i - nbSourcesPourIter);
				CElementGenerique gen = (CElementGenerique)proto.get(i);
				gen.elementAssocie = elb;
			}
			indiceImp = liste.size();
			impProto = new CImplementationProto(liste, proto);
			impProto.implementePourIter(liste, proto, cadre); // Modifi version 4.8
	    if (choixLigne != Outil.sansLigne) {
	    	for (int i = indiceImp + 1; i < liste.size(); i++) {
	    		CElementBase elb = liste.get(i);
	    		if (elb.estElementFinal && elb.estDeNature(NatureObjet.NToutPoint)){
	    			listePoints.add(new CNoeudPointeurSurPoint(liste, (CPointAncetre)elb));
	    			break;
	    		}
	    	}
	    }
			if (imposerCouleur) {
		    // On regarde tous les objets finaux et s'ils sont graphiques, on leur donne le style actif
		    for (int i= indiceImp + 1; i < liste.size(); i++) {
		    	CElementBase elb = liste.get(i);
		    	if (elb.estElementFinal && (elb.estDeNature(NatureObjet.NToutObjet))) {
		    		CElementGraphique elg = (CElementGraphique) elb;
		    		elg.donneCouleur(couleurImp);
		    	}
		    }
			}
		}
		liste.traiteImplementationsProtoSuccessivesIter(indiceImpInitial, proto, cadre, proto.size() - proto.nbObjSources);
    // Si demand dans la bote de dialogue, on relie le premier des points crs finaux points crs par un polygone ou une ligne brise
    if (choixLigne != Outil.sansLigne) {
    	if (choixLigne == Outil.avecPolygone) {
				listePoints.add(new CNoeudPointeurSurPoint(liste, listePoints.firstElement().pointeurSurPoint));
				CPolygone poly = new CPolygone(liste, listePoints, couleurLigne, styleLigne);
				liste.add(poly);
				poly.positionne(false, cadre.dimf);
				if (polygoneRempli) {
					CSurfacePolygone s = new CSurfacePolygone(liste, null, false, couleurLigne, false, StyleRemplissage.QuadrillagePleinTransp, poly);
					liste.add(s);
					s.positionne(false, cadre.dimf);
				}
	    }
	    else {
				CLigneBrisee ligne = new CLigneBrisee(liste, listePoints, couleurLigne, styleLigne);
				liste.add(ligne);
				ligne.positionne(false, cadre.dimf);
	    }
    }
    cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil161"));
		cadre.repaintFigureSansCalcul();
		dejaExecute = true;
  }

  @Override
  public void actionDansPaint(MtgInternalFrame cadre) {
  	termineAction(cadre);
  }

  @Override
  public boolean modifiableParMenu() {
  	return !estElementFinal;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new MacroConstructionIterativeDlg(frame, this, true);
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int indProto = inps.readInt();
    proto = list.documentProprietaire.tablePrototypes.get(indProto);
    nbIter = inps.readInt();
    nbSourcesPourIter = inps.readInt();
    imposerCouleur = inps.readBoolean();
    if (imposerCouleur) {
  		int r = inps.readUnsignedByte();
  		int g = inps.readUnsignedByte();
  		int b = inps.readUnsignedByte();
  		couleurImp = new Color(r, g, b, 255);
    }
    choixLigne = inps.readInt();
    if (choixLigne != 0) {
  		int r = inps.readUnsignedByte();
  		int g = inps.readUnsignedByte();
  		int b = inps.readUnsignedByte();
  		couleurLigne = new Color(r, g, b, 255);
  		// Modification version 5.0
    	// styleLigne = inps.readInt();
  		styleLigne = new StyleTrait();
  		// Correction version 5.0.4
  		if (listeProprietaire.numeroVersion < 15) styleLigne.readOld(inps, false);
  		else styleLigne.read(inps);
    	if (choixLigne == Outil.avecPolygone)
    		polygoneRempli = inps.readBoolean();
    }
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    // On enregistre l'indice du prototype utilis dans la table des prototype du document
    oups.writeInt(list.documentProprietaire.tablePrototypes.indexOf(proto));
    oups.writeInt(nbIter);
    oups.writeInt(nbSourcesPourIter);
    oups.writeBoolean(imposerCouleur);
    if (imposerCouleur) {
  		oups.writeByte(couleurImp.getRed());
  		oups.writeByte(couleurImp.getGreen());
  		oups.writeByte(couleurImp.getBlue());
    }
    oups.writeInt(choixLigne);
    if (choixLigne != 0) {
  		oups.writeByte(couleurLigne.getRed());
  		oups.writeByte(couleurLigne.getGreen());
  		oups.writeByte(couleurLigne.getBlue());
  		// Modifi version 5.0
    	// oups.writeInt(styleLigne);
  		styleLigne.write(oups);
    	if (choixLigne == Outil.avecPolygone)
    		oups.writeBoolean(polygoneRempli);
    }
  }  

}