/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.*;
import java.io.*;


import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;


public class CMarqueAngleOriente extends CMarqueAngleGeometrique {
	double xdeb,ydeb;  // les coordones du point dbut de l'arc
	VecteurDirecteur v0; // Vecteur de longueur rayon colinaire  OB
	VecteurDirecteur vecteurDirection; // vecteur dfinissant la direction
														// inverse de l'arc
	public int motifFleche;

  /** Creates new CMarqueAngleOriente */
  public CMarqueAngleOriente() {
  	v0 = new VecteurDirecteur();
    vecteurDirection = new VecteurDirecteur();
  }
  
  // Constructeur pour clonage
  public CMarqueAngleOriente(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, StyleTrait styleInit, int styleMarqueInit, double rayonInit, int motifFlecheInit,
    	CPointAncetre AInit, CPointAncetre OInit, CPointAncetre BInit) {
    super (liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit,styleMarqueInit, rayonInit,
    		 AInit, OInit, BInit);
    motifFleche = motifFlecheInit;
  	v0 = new VecteurDirecteur();
    vecteurDirection = new VecteurDirecteur();
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(A);
    int ind2 = listeSource.indexOf(O);
    int ind3 = listeSource.indexOf(B);
    int ind4 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CMarqueAngleOriente(listeCible, (CImplementationProto)(listeCible.get(ind4, CImplementationProto.class)),
      estElementFinal, couleur, masque, style, styleMarque, rayon, motifFleche, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
    (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)),(CPointAncetre)(listeCible.get(ind3, CPointAncetre.class)));
    return ptelb;
  }

  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    ((CMarqueAngleOriente)ptel).vecteurDirection.setCopy(vecteurDirection);
    xdeb = ((CMarqueAngleOriente)ptel).xdeb;
    ydeb = ((CMarqueAngleOriente)ptel).ydeb;
  }
  
	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
  	return nom() + " : " + getString("chinfo67") + " " + getString("ch31") + " " + A.nom() + O.nom() + B.nom();
	} 

  /**
   * Appel pour les marques d'angle de sens direct et indirect
   * @param infoRandom
   * @param dimfen
   */
  public void positionneAncetre(boolean infoRandom, DimensionFenetre dimfen) {
  	super.positionne(infoRandom, dimfen);
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	double petitAngle; // Reprsente l'angle de correction pour la direction de la flche
  	// due  la longueur de la flche
    
    super.positionne(infoRandom, dimfen);
    if (!existe) return;
    // Optimisation pour la version 1.9.5
    double longueurFleche =StyleFleche.longeurFlechePourArc(motifFleche, listeProprietaire);
    v.vecteurColineaire(rayon,v0);
    double angle = u.mesureAngleVecteurs(v);
    if (Kernel.zeroAngle(Math.abs(angle) - Math.PI/2)) petitAngle = 0;
    // Corrig version 5.3.3
   //  else petitAngle = longueurFleche/rayon;
    else petitAngle = Math.atan(longueurFleche/rayon/2);

    if (Kernel.zero(angle))
    {
      existe = false;
      return;
    }
    else
    {
      if (Kernel.zeroAngle(Math.abs(angle)-Math.PI)) v0.tourne(-Math.PI/2,vecteurDirection);
      else
      {
        if (angle <= 0) v0.tourne(Math.PI/2+petitAngle, vecteurDirection);
        else v0.tourne(-Math.PI/2-petitAngle, vecteurDirection);
      }
    }
    xdeb = O.x + v0.x;
    ydeb = O.y + v0.y;
  }

  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
    if (A.horsEcran || O.horsEcran || B.horsEcran) return;
    super.trace(pDC, couleurFond, transparence);
    StyleTrait styleTrait;
    if (masque) styleTrait = StyleTrait.traitPointille;
    else styleTrait = style;
    pDC.setStroke(Pinceau.CreePinceau(styleTrait, listeProprietaire));
    pDC.setColor(couleur);
    StyleFleche.traceFlechePourArc(pDC,xdeb,ydeb,vecteurDirection,motifFleche, listeProprietaire);
  }
 
  @Override
  public boolean estOriente() {
  	return true;
  }
  
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    motifFleche = inps.readByte();
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    motifFleche = inps.readByte();
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeByte(motifFleche);
  }
  
  public void donneStyleFleche(int styleFleche) {
  	motifFleche = styleFleche;
  }
  
  /**
   * Ajout version 4.8.1
   * Ajoute le code correspondant  la flche de l'arc orient
   * @param pDC
   * @param xa
   * @param ya
   * @param u
   * @param motifFleche
   * @param liste
   */
  public final String tikzFleche(DimensionFenetre dimf, double xa, double ya, VecteurDirecteur u, double coefmult, boolean bu) {
    double[] abscisses = new double[3];
    double[] ordonnees = new double[3];
    VecteurDirecteur u1 = new VecteurDirecteur();
    VecteurDirecteur u2 = new VecteurDirecteur();
    double longueurFleche = StyleFleche.longeurFlechePourArc(motifFleche, listeProprietaire);

    u.vecteurColineaire(longueurFleche,u1);
    // On fait tourner le vecteur u1 de -pi/6 pour avoir u2
    u2.x = u1.x*Kernel.cos30 - u1.y*Kernel.sin30;
    u2.y = u1.x*Kernel.sin30 + u1.y*Kernel.cos30;
    abscisses[1] = xa;
    ordonnees[1] = ya;
    abscisses[0] = xa + u2.x;
    ordonnees[0] = (int)Math.round(ya + u2.y);
    // On fait tourner le vecteur u1 de pi/6 pour avoir u2
    u2.x = u1.x*Kernel.cos30 + u1.y*Kernel.sin30;
    u2.y = -u1.x*Kernel.sin30 + u1.y*Kernel.cos30;
    abscisses[2] = xa + u2.x;
    ordonnees[2] = ya + u2.y;
    switch (motifFleche)
    {
      case StyleFleche.FlecheLonguePleine:
      case StyleFleche.FlecheCourtePleine:
        // Il faut tracer le polygone car fillPolygon ne trace que l'intrieur'
      	return "\\filldraw " + tikzdrawFillStyle(dimf, coefmult) + "(" + Kernel.chaineNombre(listeProprietaire.tikzXcoord(abscisses[0], dimf, bu), 2) +
      			"," + Kernel.chaineNombre(listeProprietaire.tikzYcoord(ordonnees[0], dimf, bu), 3) + ")--(" +
      			Kernel.chaineNombre(listeProprietaire.tikzXcoord(abscisses[1], dimf, bu), 3) + "," +
      			Kernel.chaineNombre(listeProprietaire.tikzYcoord(ordonnees[1], dimf, bu), 3) + ")--(" +
      			Kernel.chaineNombre(listeProprietaire.tikzXcoord(abscisses[2], dimf, bu), 3) + "," +
      			Kernel.chaineNombre(listeProprietaire.tikzYcoord(ordonnees[2], dimf, bu), 3) + ")--cycle;";
      case StyleFleche.FlecheLongueSimple:
      case StyleFleche.FlecheCourteSimple:
      	return "\\draw " + tikzLineStyle(dimf, coefmult) + "(" + Kernel.chaineNombre(listeProprietaire.tikzXcoord(abscisses[0], dimf, bu), 2) +
      			"," + Kernel.chaineNombre(listeProprietaire.tikzYcoord(ordonnees[0], dimf, bu), 3) + ")--(" +
      			Kernel.chaineNombre(listeProprietaire.tikzXcoord(abscisses[1], dimf, bu), 3) + "," +
      			Kernel.chaineNombre(listeProprietaire.tikzYcoord(ordonnees[1], dimf, bu), 3) + ")--(" +
      			Kernel.chaineNombre(listeProprietaire.tikzXcoord(abscisses[2], dimf, bu), 3) + "," +
      			Kernel.chaineNombre(listeProprietaire.tikzYcoord(ordonnees[2], dimf, bu), 3) + ");";
      default : return "";
    }
  }  

  @Override
  public String tikz(DimensionFenetre dimf, boolean nomaff, double coefmult, boolean bu) {
  	return super.tikz(dimf, nomaff, coefmult, bu) + System.lineSeparator() +
  			tikzFleche(dimf, xdeb, ydeb, vecteurDirection, coefmult, bu);
  }
  
}
