/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;


import glob.*;

import java.awt.*;


import types.*;

public class CMarqueAngleOrienteDirecte extends CMarqueAngleOriente {
	double angle; // Mesure principale de l'angle (OA,OB)

  /** Creates new CMarqueAngleOrienteDirecte */
  public  CMarqueAngleOrienteDirecte() {
  }
  
  // Constructeur pour clonage  
  public CMarqueAngleOrienteDirecte(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean masqueInit, StyleTrait styleInit, int styleMarqueInit, double rayonInit, int motifFlecheInit,
      	CPointAncetre AInit, CPointAncetre OInit, CPointAncetre BInit) {
    super (liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit, styleMarqueInit,
    		rayonInit, motifFlecheInit, AInit, OInit, BInit);
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(A);
    int ind2 = listeSource.indexOf(O);
    int ind3 = listeSource.indexOf(B);
    int ind4 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CMarqueAngleOrienteDirecte(listeCible, (CImplementationProto)(listeCible.get(ind4, CImplementationProto.class)),
      estElementFinal, couleur, masque, style, styleMarque, rayon, motifFleche, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)),(CPointAncetre)(listeCible.get(ind3, CPointAncetre.class)));
    return ptelb;
  }
  
	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
  	return nom() + " : " + getString("chinfo68") + " " + getString("ch31") + " " + A.nom() + O.nom() + B.nom();
	} 
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	double petitAngle; // Reprsente l'angle de correction pour la direction de la flche
  	// due  la longueur de la flche
    super.positionneAncetre(infoRandom, dimfen);
    if (!existe) return;
    // Optimisation pour la version 1.9.5
    double longueurFleche =StyleFleche.longeurFlechePourArc(motifFleche, listeProprietaire);
    v.vecteurColineaire(rayon,v0);
    angle = u.mesureAngleVecteurs(v);
    if (Kernel.zeroAngle(angle - Math.PI/2)) petitAngle = 0;
    // Corrig version 5.3.3
   //  else petitAngle = longueurFleche/rayon;
    else petitAngle = Math.atan(longueurFleche/rayon/2);
    
    if (Kernel.zero(angle))
    {
      existe = false;
      return;
    }
    else
    {
      v0.tourne(-Math.PI/2-petitAngle,vecteurDirection);
    }
    xdeb = O.x + v0.x;
    ydeb = O.y + v0.y;
  }

  @Override
  public double distancePoint(double xp, double yp, boolean masquage) {
  	  	
    if (!(existe) || (masquage && masque)) return -1;
    else {
    	boolean appsecteur;    	
    	VecteurDirecteur w = new VecteurDirecteur(centre_x, centre_y, xp, yp);
  		double ang = w.angleRad();
  		if (Kernel.zeroAngle(Math.abs(ang2rad - ang1rad) - Math.PI))
  		{
  			if (ang1rad < ang2rad) appsecteur = (ang>=ang1rad) && (ang<=ang2rad);
  			else appsecteur = (ang >= ang1rad) || (ang <= ang2rad);
  		}
  		else
  		{
  			if (ang1rad <= ang2rad) appsecteur = (ang>=ang1rad) && (ang<ang2rad);
  			else appsecteur = (ang>=ang1rad) || (ang<=ang2rad);
  		}
  		// Ici on ne peut pas appeler super.distancePoint car on appellerait la mthode
  		// de CMarqueAngleOriente
  		if (appsecteur) return Math.abs(Kernel.distancePointPoint(centre_x,centre_y,xp,yp) - rayon);
  		else return -1;
    	
    }
  }  
  
  @Override
  protected void remplitSecteur(Graphics2D pDC, double ang, float transparence) {
    Composite originalComposite = pDC.getComposite();
    pDC.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, transparence));
    Kernel.secteurDirectPlein(pDC, A.x, A.y, O.x, O.y, rayon, ang);	  	
    pDC.setComposite(originalComposite);    		
  }
  
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
  	if (angle >= 0) super.trace(pDC, couleurFond, transparence);
  	else {
	    if (A.horsEcran || O.horsEcran || B.horsEcran) return;
	    StyleTrait styleTrait;
	    if (masque) styleTrait = StyleTrait.traitPointille;
	    else styleTrait = style;
	    pDC.setStroke(Pinceau.CreePinceau(styleTrait, null));
	    pDC.setColor(couleur);
	  	double difang = ang2rad - ang1rad;
	    Kernel.secteurDirect(pDC,A.x,A.y,O.x,O.y,rayon,difang);
	    if ((styleMarque & StyleMarqueAngle.marqueRemplie) != 0) remplitSecteur(pDC, difang, transparence);
      if ((styleMarque & StyleMarqueAngle.marqueAvecTrait) != 0) 
      	traceMarqueAngle(pDC, A.x, A.y, O.x, O.y, Math.PI*2 + angle);
	    StyleFleche.traceFlechePourArc(pDC,xdeb,ydeb,vecteurDirection,motifFleche, listeProprietaire);
	  }
  }
  
  /**
   * Ajout version 4.8.1
   */
  @Override
  public String tikzPath(DimensionFenetre dimf, boolean bu) {
    VecteurDirecteur u = new VecteurDirecteur(centre_x, centre_y, A.x, A.y);
    double ang1 = u.angleRad();
    VecteurDirecteur u1 = new VecteurDirecteur();
    u.vecteurColineaire(rayon, u1);
		double x1 = listeProprietaire.tikzXcoord(centre_x + u1.x, dimf, bu);
		double y1 = listeProprietaire.tikzYcoord(centre_y + u1.y, dimf, bu);
		double ray = listeProprietaire.tikzLongueur(rayon, dimf, bu);
    VecteurDirecteur v = new VecteurDirecteur(centre_x, centre_y, B.x, B.y);
    double mesureAngle = u.mesureAngleVecteurs(v);
    double ang2 = ang1 + mesureAngle;
    if (ang2 < 0) ang2 = ang2 + 2*Math.PI;
    if (ang2 >= 2*Math.PI) ang2 = ang2 - 2*Math.PI;
    if (ang2 < ang1) ang2 = ang2 + 2*Math.PI;
    /* Modifi version 5.1.1
    ang1 = ang1*Kernel.ConvRadDeg;
    ang2 = ang2*Kernel.ConvRadDeg;
    */
    ang1 = Math.toDegrees(ang1);
    ang2 = Math.toDegrees(ang2);
    
		return "(" + Kernel.chaineNombre(x1, 3) +  "," + Kernel.chaineNombre(y1, 3) +") arc " +
		"(" + Kernel.chaineNombre(ang1, 3) + " : " + Kernel.chaineNombre(ang2, 3) + " : " + Kernel.chaineNombre(ray, 3) + ")";
  }

}
