/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;


import java.awt.Color;
import java.awt.Graphics2D;
import java.io.IOException;

import mathgraph32.MtgFrame;

import types.DimensionFenetre;
import types.NatureObjet;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;


public class CObjetDuplique extends CElementGraphique {
	CElementGraphique elementDuplique;

  /** Creates new CObjetDuplique */
  public CObjetDuplique() {
  }
  
  // Constructeur pour clonage
  public CObjetDuplique(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit, boolean masqueInit, String nomInit, int indiceTailleNomInit,
    CElementGraphique elementDupliqueInit) {
      super (liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
        dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit);
      elementDuplique = elementDupliqueInit;
  }
 
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(elementDuplique);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CObjetDuplique(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y,
      masque, nom, indiceTailleNom, (CElementGraphique)(listeCible.get(ind1, CElementGraphique.class)));
    return ptelb;
  }
  
  @Override
  public boolean confonduAvec(CElementBase p) {
    CObjetDuplique pt;
    if (p.getClass() == this.getClass()) {
      pt = (CObjetDuplique) p;
      return (elementDuplique == pt.elementDuplique);
    }
    else return false;
  }  

  @Override
  public String infoPourHistorique() {
    return nom() + " : " + getString("chinfo143") + " " + getString("ch31") + " " + elementDuplique.nom();
  }
  // Ajout version 4.9.9
  @Override
  public String rendNom() {
  	return elementDuplique.nom;
  }
  @Override
  public String nom() {
  	return getString("chinfo233") + " " + elementDuplique.nom();
  }
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
    return nom();
	}
  
  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(elementDuplique);
  }  
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || elementDuplique.dependDe(p));
  }

  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || elementDuplique.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || elementDuplique.dependDePourBoucle(p));
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	existe = elementDuplique.existe;
  }

  @Override
  public void afficheNom(Graphics2D pDC) {
    if (!masque) elementDuplique.afficheNom(pDC);
  }
  
  @Override
  public void affiche(Graphics2D pDC, boolean masquage, Color couleurFond, float transparence) {
    if (!masque) {
      boolean masquageElementCache = elementDuplique.masque;
      elementDuplique.montre();
      elementDuplique.affiche(pDC, masquage, couleurFond, transparence);
      if (elementDuplique.estDeNatureCalcul(NatureObjet.NObjetNommable)) elementDuplique.afficheNom(pDC);
      elementDuplique.masque = masquageElementCache;
    }
  }
  
  // Pour satisfaire le compilateur
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {}

  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (elementDuplique== ancienPoint) elementDuplique = nouveauPoint;
  }
  
  @Override
  public long getNature() {
    return NatureObjet.NObjetDuplique;
  }
  
  @Override
  public String chaineDesignation() {
    return getString("designationObjetDuplique");
  }
  
  public double distancePointPourSurface(double xp, double yp, boolean masquage) {
  	 if (!(existe) || (masquage && masque)) return -1;
  		else return	elementDuplique.distancePointPourSurface(xp,yp,masquage);
  }

  public double distancePoint(double xp, double yp, boolean masquage) {
  	 if (!(existe) || (masquage && masque)) return -1;
 		boolean masquageElementCache = elementDuplique.masque;
		if (masque) elementDuplique.cache();
		else elementDuplique.montre();
		double Resul = elementDuplique.distancePoint(xp,yp,masquage);
		elementDuplique.masque = masquageElementCache;
		return Resul;
	}  
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return listeOb.contains(elementDuplique) || elementDuplique.estDefiniParObjetDans(listeOb);
	}           
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    elementDuplique = (CElementGraphique) (list.get(ind1,CElementGraphique.class));
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    elementDuplique = (CElementGraphique) (list.get(ind1,CElementGraphique.class));
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(elementDuplique);
    oups.writeInt(ind1);
  }
  
  /**
   * Ajout version 4.8.1
   */
  @Override
  public String tikz(DimensionFenetre dimf, boolean nomaff, double coefmult, boolean bu) {
  	return elementDuplique.tikz(dimf, nomaff, coefmult, bu);
  }
}
