/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.Color;
import java.awt.geom.*;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;


/** Classe anctre de tous les points lis
 * Modification pour la version 2.5 : la classe n'est plus abstract car il
 * faut pouvoir l'instancier dans CListeObjets.get(int indice, Class cla)
 * */
public class CPointLie extends CPointAncetrePointsMobiles {
	public double abscisse;

  /** Creates new CPointLie */
  public CPointLie() {
  }
  
  // Constructeur pour clonage
  protected CPointLie(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
    boolean masqueInit, String nomInit, int indiceTailleNomInit,
    int motifInit, boolean pointLibreInit, double abscisseInit, boolean marquePourTraceInit) {
      super (liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit, dec_xInit, dec_yInit, 
        masqueInit, nomInit, indiceTailleNomInit, motifInit, pointLibreInit, marquePourTraceInit);
      abscisse = abscisseInit;
  }
  
  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    abscisse = ((CPointLie)ptel).abscisse;
  }
  

  public void donneAbscisse(double ab) {
    abscisse = ab;
  }
  
  // A redfinir pour les descendants.
  /** Renvoie un pointeur sur l'objet auquel est li le point */
  public CElementBase lieA() {
    return null;
  } 
  
  @Override
  public boolean confonduAvec(CElementBase p) {
    CPointLie pt;
    if (p.getClass() == this.getClass()) {
      pt = (CPointLie) p;
      return glob.Kernel.zero(abscisse - pt.abscisse);
    }
    else return false;
  }  
  

  public boolean testDeplacement(DimensionFenetre dimfen, double xtest, double ytest,
        Point2D.Double pointr, Doub abscr) {
    return false;
  }
  
  @Override
  public long getNature() {
    return NatureObjet.NPointLie;
  }
    
	// Modification pour la version 1.9.5 : 
	// Pour les lieux de points, macros de traces auto et les lieux d'objet
	// il faut savoir pour que le nombre de valeurs prises soit correct si l'objet auquel
	// le point est li est ferm ou non.
	// Renverra false par dfaut et true pour les cercles et polygones pour lesquels la fonction
	// sera donc redfinie
	public boolean lieALigneFermee() {
    return false;
  }

  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    abscisse = inps.readCDouble();
  }

  @Override
  public void read( MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    abscisse = inps.readDouble();
  }
  
  @Override
  public void write( MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeDouble(abscisse);
  }  
}
