/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;


import glob.Kernel;

import java.awt.Color;
import java.awt.geom.Point2D.Double;


import types.DimensionFenetre;
import types.Doub;
import types.InfoLieu;

public class CPointLieLieuParVar extends CPointLieLieuAncetre {

  /** Constructeur par dfaut */
  public CPointLieLieuParVar() {
  }
	
  /** Constructeur pour clonage */
  public CPointLieLieuParVar(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
      boolean masqueInit, String nomInit, int indiceTailleNomInit,
      int motifInit, boolean pointLibreInit, CLieuParVariable lieuAssocieInit, double abscisseInit, boolean MarquePourTraceInit) {
    super (liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit, dec_xInit, dec_yInit, 
        masqueInit, nomInit, indiceTailleNomInit, motifInit, pointLibreInit, abscisseInit, lieuAssocieInit, MarquePourTraceInit);
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(lieuAssocie);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CPointLieLieuParVar(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom, 
      indiceTailleNom, motif, pointLibre, (CLieuParVariable)listeCible.get(ind1, CLieuParVariable.class),
      abscisse, marquePourTrace);
    return ptelb;
  }  
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
		existe = lieuAssocie.existe;
		if (!existe) return;
		CVariableBornee var = (CVariableBornee) ((CLieuParVariable) lieuAssocie).variableGeneratrice;
		CPointAncetre ptGenerateurTrace = (CPointAncetre) ((CLieuParVariable) lieuAssocie).pointATracer;
		// On mmorise la valeur actuelle de la variable qui gnre le lieu auquel le point est li
		double ancienneAbscisse = var.valeurActuelle;
		var.donneValeur(abscisse);
		((CLieuParVariable) lieuAssocie).listeElementsAncetres.positionne(infoRandom, dimfen);
		// Correction dfaut version 4.9.5.3
		existe = ptGenerateurTrace.existe;
		if (!existe) return;
		//
		super.placeEn(ptGenerateurTrace.x, ptGenerateurTrace.y);
		var.donneValeur(ancienneAbscisse);
		((CLieuParVariable) lieuAssocie).listeElementsAncetres.positionne(infoRandom, dimfen);
		super.positionne(infoRandom, dimfen); // Ajout version 4.6.4
  }
  
  @Override
  public double abscisseMaximale() {
  	return ((CLieuParVariable) lieuAssocie).variableGeneratrice.valeurMini;
  }

  @Override
  public double abscisseMinimale() {
  	return ((CLieuParVariable) lieuAssocie).variableGeneratrice.valeurMaxi;
  }
  // Modifi version 4.9.5.4
  /*
	@Override
	public boolean testDeplacement(DimensionFenetre dimfen, double xtest,
			double ytest, Double pointr, Doub abscr) {
		CLieuParVariable lieu = (CLieuParVariable) lieuAssocie;
		CVariableBornee var = lieu.variableGeneratrice;
		InfoLieu info = lieu.infoLieu;
    double abscisseMini = var.valeurMini;
    double abscisseMaxi = var.valeurMaxi;
    // On ordonne l'abscisse minimale et maximale
    if (abscisseMini > abscisseMaxi) {
      double w = abscisseMini;
      abscisseMini = abscisseMaxi;
      abscisseMaxi = w;
    }
    // Modification pour la version 1.9.5
    double pas = (abscisseMaxi - abscisseMini)/(info.nombreDePoints-1);
    long dis = Long.MAX_VALUE;
    int indDistMini = 0;
    for (int i = 0; i < info.nombreDePoints; i++) {
    	long d = (long) Kernel.distancePointPoint(xtest, ytest, lieu.abscissesCoordonneesPoints[i], lieu.ordonneesCoordonneesPoints[i]);
    	if (d < dis) {
    		dis = d;
    		indDistMini = i;
    	}  
    }
    abscr.setValue(abscisseMini + indDistMini*pas);
    // Il est inutile ici de donner des valeurs  pointr qui n'est pas utilis par l'outil correspondant
    return true;
	}  
  
}
*/
  // Modifi version 4.9.5.4
	@Override
	public boolean testDeplacement(DimensionFenetre dimfen, double xtest,
			double ytest, Double pointr, Doub abscr) {
		CLieuParVariable lieu = (CLieuParVariable) lieuAssocie;
		CVariableBornee var = lieu.variableGeneratrice;
		InfoLieu info = lieu.infoLieu;
    double abscisseMini = var.valeurMini;
    double abscisseMaxi = var.valeurMaxi;
    // On ordonne l'abscisse minimale et maximale
    if (abscisseMini > abscisseMaxi) {
      double w = abscisseMini;
      abscisseMini = abscisseMaxi;
      abscisseMaxi = w;
    }
    // Modification pour la version 1.9.5
    double pas = (abscisseMaxi - abscisseMini)/(info.nombreDePoints-1);
    long dis = Long.MAX_VALUE;
    int indDistMini = 0;
    for (int j = 0; j < lieu.nombreLignes; j++) {
      int nombrePoints = lieu.infoLignes[j].nombrePoints;
      // Optimis version 3.9 On trace maintenant les points isols du lieu
      int indPremPoint = lieu.infoLignes[j].indicePremierPoint;
      for (int k = 0; k < nombrePoints; k++) {
      	int k1 = indPremPoint + k;
      	int x = lieu.abscissesCoordonneesPoints[k1];
      	int y = lieu.ordonneesCoordonneesPoints[k1];
      	if (dimfen.dansFenetre(x, y)) {
		    	long d = (long) Kernel.distancePointPoint(xtest, ytest, x, y);
		    	if (d < dis) {
		    		dis = d;
		    		indDistMini = k1;
		    	}
      	}
      }
    }
    abscr.setValue(abscisseMini + indDistMini*pas);
    // Il est inutile ici de donner des valeurs  pointr qui n'est pas utilis par l'outil correspondant
    return true;
	}  
  
}
