/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.geom.*;
import java.awt.Color;


import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

import mathgraph32.MtgFrame;


public class CPointLieLigne extends CPointLie {
  private double[] tableauLongueurs;
  private double longueurTotale;
  CLignePolygonaleAncetre ligneLiee;
  private Point2D.Double pointres = new Point2D.Double();
  // Objets servant dans testDeplacement()
  private CPointBase point1;
  private CPointBase point2;
  private CSegment seg;
  
  /** Creates new CPointLieLigne */
  public CPointLieLigne() {
  }
  
  // tableauLongueurs ne doit pas tre cr dans le constructeur quand ion implmente une construction car celui-ci est appel par getClone
  // et quand on  cr une construction on a un null pointerException
  /** Constructeur pour clonage
   * */
  public CPointLieLigne(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit, boolean masqueInit, String nomInit, int indiceTailleNomInit,
    int motifInit, boolean pointLibreInit, double abscisseInit, CLignePolygonaleAncetre ligneLieeInit, boolean marquePourTraceInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
      dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit, motifInit, pointLibreInit, abscisseInit, marquePourTraceInit);
    point1 = new CPointBase();
    point2 = new CPointBase();
    seg = new CSegment(liste,point1, point2);
    ligneLiee = ligneLieeInit;
    if (listeProprietaire.getClass() != CPrototype.class) tableauLongueurs = new double[ligneLiee.collectionPoints.size() - 1];
  }
    
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(ligneLiee);
    int ind2 = listeSource.indexOf(impProto);
    CElementBase ptelb = new CPointLieLigne(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom,
      indiceTailleNom, motif, pointLibre, abscisse, (CLignePolygonaleAncetre)(listeCible.get(ind1, CLignePolygonaleAncetre.class)),
      marquePourTrace);
    return ptelb;
  }
  
  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    // Il faut copier les valeurs du tableau de longueurs et la longueur totale
    longueurTotale = ((CPointLieLigne) ptel).longueurTotale;
    // for (int i = 0; i <  - 1; i++) { // Erreur corrige version 4.7.6
    for (int i = 0; i < tableauLongueurs.length - 1; i++)
      tableauLongueurs[i] = ((CPointLieLigne) ptel).tableauLongueurs[i];
  }
 
  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(ligneLiee);
  }
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
  	return nom() + " : " + getString("chinfo2") + " " + ligneLiee.nom();
	}
  
  @Override
  public boolean confonduAvec(CElementBase p) {
  	boolean res = super.confonduAvec(p);
   	if (!res) return false;
  	CPointLieLigne pt = (CPointLieLigne) p;
  	return (ligneLiee == pt.ligneLiee);
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || ligneLiee.dependDe(p));
  }
  
  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || ligneLiee.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return super.dependDePourBoucle(p) || ligneLiee.dependDePourBoucle(p);
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    int i;
    
    existe = ligneLiee.existe;
    if (!existe) return;
    calculeLongueurs();
    if (longueurTotale == 0) {
      existe = false;
      return;
    }
    // On cherche maintenant sur quel segment est situ le point li
    // Java : a t remani par rapport au C++
    double longueurPartielle = 0;
    double longueurJusqueDebSegment = 0;
    int nbSommets = ligneLiee.collectionPoints.size();
    for (i = 0; i < nbSommets - 1; i++) {
      longueurJusqueDebSegment = longueurPartielle;
      longueurPartielle += tableauLongueurs[i];
      if (longueurPartielle/longueurTotale >= abscisse) break;
    }
    // Juste par prcaution
    if (i == nbSommets-1) i = nbSommets - 2;
    CNoeudPointeurSurPoint n1 = (CNoeudPointeurSurPoint)ligneLiee.collectionPoints.get(i);
    CPointAncetre ptp1 = (CPointAncetre) n1.pointeurSurPoint;
    CNoeudPointeurSurPoint n2 = (CNoeudPointeurSurPoint)ligneLiee.collectionPoints.get(i + 1);
    CPointAncetre ptp2 = (CPointAncetre) n2.pointeurSurPoint;
    double abscisseSurSegment;
    if (tableauLongueurs[i] == 0) {
      abscisseSurSegment = 0;
    } else {
      abscisseSurSegment = (longueurTotale*abscisse-longueurJusqueDebSegment)/tableauLongueurs[i];
    }
    double xn = ptp1.x + (abscisseSurSegment)*(ptp2.x - ptp1.x);
    double yn = ptp1.y + (abscisseSurSegment)*(ptp2.y - ptp1.y);
    super.placeEn(xn,yn);
    super.positionne(infoRandom, dimfen);
  }
  
  public double abscisseMaximale() {
    return ligneLiee.abscisseMaximale();
  }
  
  public double abscisseMinimale() {
    return ligneLiee.abscisseMinimale();
  }
  
  public CElementBase lieA() {
    return ligneLiee;
  }
  
  public void calculeLongueurs() {
    
    longueurTotale = 0;
    for (int i = 0; i < ligneLiee.collectionPoints.size() - 1; i++) {
      CNoeudPointeurSurPoint n1 = (CNoeudPointeurSurPoint) ligneLiee.collectionPoints.get(i);
      CNoeudPointeurSurPoint n2 = (CNoeudPointeurSurPoint) ligneLiee.collectionPoints.get(i + 1);
      
      CPointAncetre ptp1 = (CPointAncetre) n1.pointeurSurPoint;
      CPointAncetre ptp2 = (CPointAncetre) n2.pointeurSurPoint;
      VecteurDirecteur u = new VecteurDirecteur(ptp1,ptp2);
      double longueurSegment = u.norme();
      tableauLongueurs[i] = longueurSegment;
      longueurTotale += longueurSegment;
    }
  }
  
  public boolean testDeplacement(DimensionFenetre dimfen, double xtest, double ytest,
    Point2D.Double pointr, Doub abscr) {
    CPointAncetre ptPointProche = null;
    double distMin = Double.MAX_VALUE;
    
    // On calcule les longueurs des cts et la longueur totale
    // de la ligne polygonale clique
    calculeLongueurs();
    
    int indiceDistMin = -1;
    boolean valable = false;
    for (int i = 0; i < ligneLiee.collectionPoints.size() - 1; i++) {
      CNoeudPointeurSurPoint n1 = (CNoeudPointeurSurPoint) ligneLiee.collectionPoints.get(i);
      CNoeudPointeurSurPoint n2 = (CNoeudPointeurSurPoint) ligneLiee.collectionPoints.get(i + 1);
      
      CPointAncetre ptp1 = (CPointAncetre) n1.pointeurSurPoint;
      CPointAncetre ptp2 = (CPointAncetre) n2.pointeurSurPoint;
      point1.placeEn(ptp1);
      point2.placeEn(ptp2);
      seg.positionne();
      Kernel.projetteOrtho(xtest,ytest,ptp1.x,ptp1.y,seg.vecteur,pointres);
      if (seg.appartientA(pointres.x,pointres.y)) {
        valable = true;
        VecteurDirecteur u = new VecteurDirecteur(pointres.x, pointres.y, xtest, ytest);
        double dist = u.norme();
        if (dist < distMin) {
          ptPointProche = ptp1;
          distMin = dist;
          indiceDistMin = i;
          pointr.x = pointres.x;
          pointr.y = pointres.y;
        }
      }
      else {
      	double d1 = Kernel.distancePointPoint(ptp1.x, ptp1.y, xtest, ytest);
        if (d1 < distMin) {
        	valable = true;
          ptPointProche = ptp1;
          distMin = d1;
          indiceDistMin = i;
          pointr.x = ptp1.x;
          pointr.y = ptp1.y;
        }
      }
    }
    if (valable) {
      // Il faut calculer l'abscisse du point
      double longueurDebutLigne = 0;
      for (int j = 0; j < indiceDistMin; j ++) {
        longueurDebutLigne += tableauLongueurs[j];
      }
      VecteurDirecteur v = new VecteurDirecteur(ptPointProche.x, ptPointProche.y,
        pointr.x, pointr.y);
      longueurDebutLigne += v.norme();
      abscr.setValue(longueurDebutLigne/longueurTotale);
    }
    return valable;
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return listeOb.contains(ligneLiee) || ligneLiee.estDefiniParObjetDans(listeOb);
	}    
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    point1 = new CPointBase();
    point2 = new CPointBase();
    seg = new CSegment(list, point1, point2);
    int ind1 = inps.readCInt();
    ligneLiee = (CLignePolygonaleAncetre) (list.get(ind1,CLignePolygonaleAncetre.class));
    // Pas de rservation en mmoire pour le chargement des prototypes
    if (list.getClass() != CPrototype.class)
      tableauLongueurs = new double[ligneLiee.collectionPoints.size() - 1];
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    point1 = new CPointBase();
    point2 = new CPointBase();
    seg = new CSegment(list, point1, point2);
    int ind1 = inps.readInt();
    ligneLiee = (CLignePolygonaleAncetre) (list.get(ind1,CLignePolygonaleAncetre.class));
    // Pas de rservation en mmoire pour le chargement des prototypes
    if (list.getClass() != CPrototype.class)
      tableauLongueurs = new double[ligneLiee.collectionPoints.size() - 1];
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(ligneLiee);
    oups.writeInt(ind1);
  }
  
}
