/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.geom.Point2D;
import java.io.IOException;

import types.DimensionFenetre;
import types.VecteurDirecteur;

import dialogs.AngleEtRapportSimilitudeDlg;
import dialogs.MtgDlg;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

import mathgraph32.MtgFrame;


public class CSimilitude extends CRotation {
	public CValeurAssocieeAVariable rapport;
  private Point2D.Double point1 = new Point2D.Double();
  // Java : modification par rapport au C++ : on stocke la valeur numrique du rapport
  // pour ne pas avoir  la recalculer plus tard
  double valeurRapport;

  /** Creates new CSimilitude */
  public CSimilitude() {
  }
  
  @Override
	public int natureTransformation() {
		return similitude;
	}  
  
  // Constructeur pour clonage
  public CSimilitude(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      CPointAncetre centreInit, CValeurAngle angleInit, CValeurAssocieeAVariable rapportInit) {
    super(liste, impProtoInit, estElementFinalInit, centreInit, angleInit);
    rapport = rapportInit;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(centre);
    int ind2 = listeSource.indexOf(impProto);        
    CValeurAngle angleClone = (CValeurAngle) angle.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable rapportClone = (CValeurAssocieeAVariable) rapport.getClone(listeSource, listeCible);
    CElementBase ptelb = new CSimilitude(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)), angleClone, rapportClone);
    return ptelb;
  }

  @Override
  void AjouteElementsVisuelsPourHist(CListeObjets liste) {
    liste.add(centre);
  }
  
	// Ajout version 4.9.9.
	/**
	 * Fonction renvoyant complment d'info pour liste totale d'objets
	 * @return
	 */
  @Override
	String complementInfo() {
    return getString("chinfo26") + " " + getString("chinfo202") + " " + centre.nom() + getString("chinfo205") + angle.chaineInfo() + " " + getString("chinfo28") + " " + rapport.chaineInfo();
	}

  // Ajout version 4.8
  @Override
  public void initialisePourDependance() {
  	super.initialisePourDependance();
  	rapport.initialisePourDependance();
  }

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
  	return memoriseDependance(super.dependDe(p) || rapport.dependDe(p));
  }

  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || rapport.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
  	return super.dependDePourBoucle(p) || rapport.dependDePourBoucle(p);
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    super.positionne(infoRandom, dimfen);
    rapport.positionne(infoRandom, dimfen);
    existe = existe && rapport.existe();
    if (existe) valeurRapport = rapport.rendValeur();
  }

  public boolean confonduAvec(CElementBase p) {
    CSimilitude pt;
    if (p.getClass() == this.getClass()) {
      pt = (CSimilitude) p;
      return centre.confonduAvec(pt.centre) && angle.confonduAvec(pt.angle)
              && rapport.confonduAvec(pt.rapport);
    } else return false;
  }
  
  public double transformeLongueur(double L) {
  	return Math.abs(valeurRapport)*L;
  }

  public void transformePoint(double x, double y, Point2D.Double pointr) {
    // On cre d'abord l'image par l'objet rotation dont CSimilitude
    // hrite puis on prend l'image par une homothtie de rapport k
    super.transformePoint(x,y,point1);
    pointr.x = centre.x + valeurRapport*(point1.x - centre.x);
    pointr.y = centre.y + valeurRapport*(point1.y - centre.y);
  }

  public void transformeVecteur(VecteurDirecteur u, VecteurDirecteur ur) {
    VecteurDirecteur v = new VecteurDirecteur();
    super.transformeVecteur(u, v);
    ur.x = valeurRapport* v.x;
    ur.y = valeurRapport* v.y;
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(centre) || centre.estDefiniParObjetDans(listeOb))
		&&
		angle.estDefiniParObjetDans(listeOb)
		&&
		rapport.estDefiniParObjetDans(listeOb);  	
	}  	    
    
  /**
   * Fonction renvoyant true lorsque les images par ce type de transformation peuvent tre modifies
   * via un menu. C'est le cas pour les rotations, homothties, similitudes, inversions pour lequels
   * il faudra redfinir  cette fonction
   * @return
   */
  public boolean imageModifiableParMenu() {
  	return true;
  }
  
  /** Fonction renvoyant un dialogue seulement dans le cas o imageModifiableParMenu() renvoie true
   * Renverra le dailogue associ  la modification de la transformation.
   * Par exemple pour une rotation renverra un angleRotDlg.
   * @param image
   * @return
   */
  public MtgDlg dialogueModificationImage(MtgFrame frame) {
  	return new AngleEtRapportSimilitudeDlg(frame, this, true);
  }

  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    rapport = new CValeurAssocieeAVariable();
    rapport.chargeDepuisFlux(inps, list);
  } 
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    rapport = new CValeurAssocieeAVariable();
    rapport.read(inps, list);
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    rapport.write(oups, list);
  }     
}
