/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package objets;

import mathgraph32.MtgFrame;
import dialogs.MtgDlg;
import dialogs.TestFactDlg;
import types.DimensionFenetre;
import types.NatureCalcul;

/**
 * Ajout version 4.7.9. Objet permettant de dterminer si la formule d'un calcul ou une fonction est factorise par un autre
 * @author Yves
 *
 */
public class CTestFact extends CTestEquivalence {
	
	/**
	 * Constructeur
	 */
	public CTestFact() {
		existe = true;
		dejaPositionne = false;
	}
	// Version 5.1.3 : O rajoute un membre rempDecParFrac. S'il est true, les nombres dcimaux (au plus 6 chiffres aprs la virgule) seront remplacs
	// par des fractions irrductibles avant le test

  public CTestFact(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      String nomCalculInit, CCalcul calcul1Init, CCalcul calcul2Init, boolean remplacementValeursInit1,
      boolean remplacementValeursInit2, CValeurAssocieeAVariable testInit, boolean RempDecParFracInit) {
      super(liste, impProtoInit, estElementFinalInit, nomCalculInit, calcul1Init, calcul2Init, remplacementValeursInit1, remplacementValeursInit2, testInit, RempDecParFracInit);
  }
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(impProto);
    int ind2 = listeSource.indexOf(calcul1);
    int ind3 = listeSource.indexOf(calcul2);   
    CValeurAssocieeAVariable testClone = (CValeurAssocieeAVariable) test.getClone(listeSource, listeCible);
    return new CTestFact(listeCible, (CImplementationProto)(listeCible.get(ind1, CImplementationProto.class)),
      estElementFinal, nomCalcul, (CCalcul)(listeCible.get(ind2, CCalcul.class)),
      (CCalcul)(listeCible.get(ind3, CCalcul.class)), remplacementValeurs1, remplacementValeurs2, testClone, equivalenceDecimalFracIrr);
  }
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NTestFact;
  }
  // Modifi version 4.9.9
  @Override
  public String infoPourHistorique() {
    String ch = nom() + " : " + getString("ch200") + "\n" + calcul1.nom() + " " + getString("chinfo189") + " " + calcul2.nom() + "\n";
    ch = ch + calcul1.nom() + " : " + (remplacementValeurs1 ? getString("ch187") : getString("ch188"));
    ch = ch + " " + getString("ch189") + "\n";
    ch = ch + calcul2.nom() + " : " + (remplacementValeurs2 ? getString("ch187") : getString("ch188"));
    ch = ch + " " + getString("ch189") + "\n";
    ch = ch + getString("testEquivalenceDlg4") + " " + test.chaineInfo() + "\n";
    ch = ch + getString("chinfo97") + " " + glob.Kernel.chaineNombre(resultat, 16);
    return ch;
  }
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
    return nom() + " : " + getString("ch200") + " " + calcul1.nom() + " " + getString("chinfo189") + " " +
    		calcul2.nom();
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    // Existe toujours gal  true pour cet objet. Fini  partir vrsion 4.9.8
  	// Ajout version 4.9.8
  	test.positionne(infoRandom, dimfen);
  	if (!test.existe()) {
  		resultat = 0;
  		return;
  	}
  	if (test.rendValeur() != 1) {
  		resultat = 0;
  		return;
  	}
  	//
  	if (!dejaPositionne) {
  		if (remplacementValeurs1) calculeq1 = calcul1.calcul.calculAvecValeursRemplacees().calculNormalise(true, true, equivalenceDecimalFracIrr);
  		else calculeq1 = calcul1.calcul.calculNormalise(remplacementValeurs1, true, equivalenceDecimalFracIrr);
  		if (remplacementValeurs2) calculeq2 = calcul2.calcul.calculAvecValeursRemplacees().calculNormalise(true, true, equivalenceDecimalFracIrr);
  		else calculeq2 = calcul2.calcul.calculNormalise(remplacementValeurs2, true, equivalenceDecimalFracIrr);
  		dejaPositionne = true; // Ajout version 4.7.2
  	}
    if (!calcul1.existe || !calcul2.existe) {
    	resultat = 0;
    	return;
    }
    if (calculeq1.estFactorisePar(calculeq2)) resultat = 1; else resultat = 0;
  }
  // Ajout version 4.9.3. Utilis dans les sommes et produits indics
  @Override
  public void positionneFull(boolean infoRandom, DimensionFenetre dimfen) {
    // Existe toujours gal  true pour cet objet. Fini  partir vrsion 4.9.8
  	// Ajout version 4.9.8
  	test.positionne(infoRandom, dimfen);
  	if (!test.existe()) {
  		resultat = 0;
  		return;
  	}
  	if (test.rendValeur() != 1) {
  		resultat = 0;
  		return;
  	}
  	//
		if (remplacementValeurs1) calculeq1 = calcul1.calcul.calculAvecValeursRemplacees().calculNormalise(true, true, equivalenceDecimalFracIrr);
		else calculeq1 = calcul1.calcul.calculNormalise(remplacementValeurs1, true, equivalenceDecimalFracIrr);
		if (remplacementValeurs2) calculeq2 = calcul2.calcul.calculAvecValeursRemplacees().calculNormalise(true, true, equivalenceDecimalFracIrr);
		else calculeq2 = calcul2.calcul.calculNormalise(remplacementValeurs2, true, equivalenceDecimalFracIrr);
		// dejaPositionne = false; // Modifi version 4.9.6
		dejaPositionne = true;
    if (!calcul1.existe || !calcul2.existe) {
    	resultat = 0;
    	return;
    }
    if (calculeq1.estFactorisePar(calculeq2)) resultat = 1; else resultat = 0;
  }
  
  @Override
	public MtgDlg dialogueModification(MtgFrame frame) {
	 	 return new TestFactDlg(frame,this, true);
	}
}
