/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.*;

import objets.*;
import types.*;

import dialogs.*;
import mathgraph32.MtgFrame;


/**
 *
 * @author Yves
 */
public class OutilCerOR extends Outil {
  CPointAncetre point;
  CValeurAssocieeAVariable ray;
  CCercleOR ptCercle;
  /** Creates a new instance of OutilCerOR */
  public OutilCerOR(MtgFrame frameInit) {
    super(frameInit, true);
  }
	/**
	 * Ajout versions 5.0.
	 * Renverra true si on peut crer un point quand on clique sur un endroit vide de la figure.
	 * Devra tre redfini pour les outils pour lesquels ce n'est   pas possible comme les outils de mesure.
	 * @return
	 */
	protected boolean creationPointPossible() {
		return true;
	}

  /** Appel lorsque l'outil est slectionn */
  @Override
  public void select() {
    super.select();
    point = null;
    frame.outilPointageCre.designationDe(NatureObjet.NToutPoint);
    frame.outilPointageActif = frame.outilPointageCre;
    frame.indication(getString("cercleOR1"));
  }
  
  @Override
  public void creeObjet() {
    if (frame.ajouteElementAvecVerification(ptCercle))
      frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil5"));
    annuleClignotement();
    select();
  }

  @Override
  public boolean activationValide() {
    return (frame.cadreActif().listePr.pointeurLongueurUnite != null);
  }
  
	@Override
	public String messageActivationNonValide() {
		return getString("messagePasDeLongueurUnite");
	}	
  
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    CListeObjets liste = frame.cadreActif().listePr;
    if (point == null) {
      frame.indication(getString("cercleOR2"));
    	// Ajout version 5.0
    	if (elg == null) {
    		point = frame.ajoutePoint(pointClique.getX(), pointClique.getY(), false);
    	}
    	else point = (CPointAncetre) elg;
      // ajouteClignotementDe(elg); Supprim version 5.0
    	ajouteClignotementDe(point); // Ajout version 5.0
      resetClignotement();
      ray = new CValeurAssocieeAVariable(liste, 0);
      // Modifi version 4.6.3
      ptCercle = new CCercleOR(liste, null, false, frame.couleurActive(),
          false, frame.styleTraitActif(),  point, ray, false);
      // On cre un dialogue modal permettant d'entrer l'angle de la rotation
      RayonDlg rayonDlg = new RayonDlg(frame, ptCercle, false);
      // On redessine la figure pour liminer la dsignation ce point affiche pur le centre
      frame.cadreActif().videDesignation();
      frame.cadreActif().paneFigure.repaint();
  		rayonDlg.setLocationRelativeTo(frame);
      rayonDlg.setVisible(true);
      // On regarde si la bote de dialogue a bien t valide par OK
      if (rayonDlg.retourOK()) {
        creeObjet();
      }
      else {
        annuleClignotement();
        select(); 
      }
    }
  }      
}
