/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;
import java.util.ListIterator;

import objets.CElementBase;
import objets.CElementGraphique;
import objets.CListeObjets;
import types.NatureObjet;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class OutilConstructionChoixSourcesGraphiques extends Outil {

  public OutilConstructionChoixSourcesGraphiques(MtgFrame frameInit) {
    super(frameInit, true);
  }
    
  @Override
	public void select() {
    super.select();
    MtgInternalFrame cadre = frame.cadreActif();
    frame.listeExclusionDesignation.ajouteElementsDe(cadre.listeObjetsSourcesG);
    frame.outilPointageActif = frame.outilPointageCre;
    frame.outilPointageActif.designationDe(NatureObjet.NObjetGraphiqueSourcePourConst);
    frame.listeClignotante.ajouteElementsDe(cadre.listeObjetsSourcesG);
    excluDesignationObjetsDependantObjetsSources();
    frame.indication(getString("objetsSourcesGraphiques1"));
    resetClignotement();
	}

  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
  	frame.listeClignotante.add(elg);
  	// frame.listeExclusionDesignation.add(elg);
  	frame.cadreActif().listeObjetsSourcesG.add(elg);
  	excluDesignationObjetsDependantDe(elg);
  }
	
  @Override
  public void creeObjetApresClicDroit() {
  	frame.activeOutilCapture();
  }
  
  // Ajout version 4.2
  /**
   * Interdit la dsignation comme objets sources graphiques d'objets dpendant des objets sources
   * graphiques ou non graphiques dj dsigns
   */
  private void excluDesignationObjetsDependantObjetsSources() {
  	MtgInternalFrame cadre = frame.cadreActif();
  	CListeObjets liste = cadre.listePr;
  	ListIterator <CElementBase> li = liste.listIterator();
  	while (li.hasNext()) {
  		CElementBase elb = li.next();
  		if (elb.dependDeAuMoinsUn(cadre.listeObjetsSourcesNG) || elb.dependDeAuMoinsUn(cadre.listeObjetsSourcesG)
  				|| cadre.listeObjetsSourcesNG.dependDe(elb) || cadre.listeObjetsSourcesG.dependDe(elb)) {
  			frame.listeExclusionDesignation.add(elb);
  		}
  	}
  }
  
  /**
   * Interdit la designation d'objets dpendant de l'objet qui vient d'tre dsign
   * @param elg
   */
  private void excluDesignationObjetsDependantDe(CElementGraphique elg) {
  	MtgInternalFrame cadre = frame.cadreActif();
  	CListeObjets liste = cadre.listePr;
  	ListIterator <CElementBase> li = liste.listIterator();
  	while (li.hasNext()) {
  		CElementBase elb = li.next();
  		if (elb.dependDe(elg)) frame.listeExclusionDesignation.add(elb);
  	}
  }
  
  
}