/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Cursor;
import java.awt.Point;

import dialogs.MacroAnimationDlg;

import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CAffichageLieAPoint;
import objets.CCommentaire;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CMacroAnimationPointLie;
import objets.CPointLie;
import types.NatureObjet;
import types.StyleEncadrement;

public class OutilMacroAnimation extends Outil {
  private boolean dialogueRempli;
  private CMacroAnimationPointLie macroanim;

  public OutilMacroAnimation(MtgFrame frameInit) {
    super(frameInit, true);
  }
    
  @Override
	public Cursor curseurUtilise() {
		if (dialogueRempli) return MtgFrame.crossHairCursor;
		else return MtgFrame.cornerCursor;
	}

  @Override
	public void select() {
    super.select();
    frame.outilPointageActif = frame.outilPointageClic;
    frame.indication(getString("macro1"));
    dialogueRempli = false;
  }
	
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    int x = pointClique.x;
    int y = pointClique.y;
    MtgInternalFrame cadre = frame.cadreActif();
    CListeObjets liste = cadre.listePr;
    if (!dialogueRempli) {
	    // Avant l'affichage du dialogue on cre un message qui sera ajout  la liste des objets clignotants
	    // pour matrialiser l'emplacement de l'affichage
    	/* Modifi version 5.0
	    CCommentaire comClignotant = new CCommentaire(liste, null, false, frame.couleurActive(), x, y, 0d, 0d, false, null, 2,
	    		StyleEncadrement.Sans, false, frame.cadreActif().couleurFond, getString("ch80"),
					CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
			*/
	    CCommentaire comClignotant = new CCommentaire(liste, null, false, frame.couleurActive(), x, y, 0d, 0d, false, null, (int)(MtgFrame.coefMultForSystemFonts*13),
	    		StyleEncadrement.Sans, false, frame.cadreActif().couleurFond, getString("ch80"),
					CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
	    comClignotant.positionne(false, frame.cadreActif().dimf);
	    ajouteClignotementDe(comClignotant);
	    resetClignotement();
	    /* Modifi version 5.0
	    macroanim = new CMacroAnimationPointLie(liste, null, false, frame.couleurActive(), 
	    		x, y, 0d, 0d, false, null, 2, false, cadre.couleurFondAffichage, "", "", true, 10, 50, 50, null,
	    		CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut, true, false, false);
	    */
	    macroanim = new CMacroAnimationPointLie(liste, null, false, frame.couleurActive(), 
	    		x, y, 0d, 0d, false, null, MtgFrame.screenIsHighResolution ? 26 : 13, false, cadre.couleurFondAffichage, "", "", true, 10, 50, 50, null,
	    		CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut, true, false, false);
	    frame.indication("");
	    MacroAnimationDlg dlg = new MacroAnimationDlg(frame, macroanim, false);
			dlg.setLocationRelativeTo(frame);
	    dlg.setVisible(true);
	    if (dlg.retourOK()) {
	    	frame.indication(getString("macroAnimationAvecTrace2"));				
	    	dialogueRempli = true;
	    	frame.listeClignotante.ajouteObjetsParNature(liste,NatureObjet.NPointLie);
	    	frame.outilPointageActif = frame.outilPointageObjetClignotant;
	    	frame.outilPointageActif.designationDe(NatureObjet.NPointLie);
	    	frame.outilPointageActif.reset();
	      resetClignotement();      
	    }
	    else {
	    	annuleClignotement();
	    	frame.activeOutilCapture();
	    }
    }
    else {
    	macroanim.pointLieAssocie = (CPointLie) elg;
    	annuleClignotement();
			if (frame.ajouteElementAvecVerification(macroanim)) {
			  frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil88"));
			  frame.cadreActif().repaintFigureAvecCalcul();
			}
			frame.activeOutilCapture();	  	
    }
  }
  
  @Override
	public boolean activationValide() {
		return (frame.cadreActif().listePr.nombreObjetsParNatureVisibles(NatureObjet.NPointLie) != 0);
	}
  
}
