/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Cursor;
import java.awt.Point;
import java.util.ListIterator;

import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CAffichageLieAPoint;
import objets.CCommentaire;
import objets.CElementBase;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CMacroTraceAuto;
import objets.CPointLie;
import objets.CSousListeObjets;
import types.NatureObjet;
import types.StyleEncadrement;
import dialogs.MacroTraceAutoParPointLieDlg;

public class OutilMacroTraceAutoParPointLie extends Outil {
  private boolean dialogueRempli;
  private boolean ObjetsATracerDesignes;
  private CMacroTraceAuto macro;
  private CSousListeObjets sousListe;

  public OutilMacroTraceAutoParPointLie(MtgFrame frameInit) {
    super(frameInit, true);
  }
    
  @Override
	public Cursor curseurUtilise() {
		if (dialogueRempli) return MtgFrame.crossHairCursor;
		else return MtgFrame.cornerCursor;
	}

  @Override
	public void select() {
    super.select();
    frame.outilPointageActif = frame.outilPointageClic;
    frame.indication(getString("macro1"));
    dialogueRempli = false;
    ObjetsATracerDesignes = false;
  }
	
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    int x = pointClique.x;
    int y = pointClique.y;
    MtgInternalFrame cadre = frame.cadreActif();
    CListeObjets liste = cadre.listePr;
    if (!dialogueRempli) {
	    // Avant l'affichage du dialogue on cre un message qui sera ajout  la liste des objets clignotants
	    // pour matrialiser l'emplacement de l'affichage
    	/*
	    CCommentaire comClignotant = new CCommentaire(liste, null, false, frame.couleurActive(), x, y, 0d, 0d, false, null, 2,
	    		StyleEncadrement.Sans, false, frame.cadreActif().couleurFond, getString("ch80"),
					CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
			*/
	    CCommentaire comClignotant = new CCommentaire(liste, null, false, frame.couleurActive(), x, y, 0d, 0d, false, null, (int)(MtgFrame.coefMultForSystemFonts*13),
	    		StyleEncadrement.Sans, false, frame.cadreActif().couleurFond, getString("ch80"),
					CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
	    comClignotant.positionne(false, frame.cadreActif().dimf);
	    ajouteClignotementDe(comClignotant);
	    resetClignotement();
	    /* Modifi version 5.0
	    macro = new CMacroTraceAuto(liste, null, false, frame.couleurActive(), 
	    		x, y, 0d, 0d, false, null, 2, StyleEncadrement.Effet3D, false, cadre.couleurFondAffichage, "", "",
	    		null, null, 50, 0, CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut); 
	    */   
	    macro = new CMacroTraceAuto(liste, null, false, frame.couleurActive(), 
	    		x, y, 0d, 0d, false, null, MtgFrame.screenIsHighResolution ? 26 : 13, StyleEncadrement.Effet3D, false, cadre.couleurFondAffichage, "", "",
	    		null, null, 50, 0, CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut); 
	    frame.indication("");
	    MacroTraceAutoParPointLieDlg dlg = new MacroTraceAutoParPointLieDlg(frame, macro, false);
			dlg.setLocationRelativeTo(frame);
	    dlg.setVisible(true);
	    if (dlg.retourOK()) {
	    	dialogueRempli = true;
	    	frame.indication(getString("macroAnimationAvecTrace1"));
	    	sousListe = new CSousListeObjets();
	    	frame.outilPointageActif = frame.outilPointageCre;
	    	frame.outilPointageActif.designationDe(NatureObjet.NToutObjet);
	    	frame.outilPointageActif.reset();
	      resetClignotement();      
	    }
	    else {
	    	annuleClignotement();
	    	frame.activeOutilCapture();
	    }
    }
    else {
    	if (ObjetsATracerDesignes) {
	    	macro.pointLieAssocie = (CPointLie) elg;
	    	macro.listeAssociee = sousListe;
	    	annuleClignotement();
				if (frame.ajouteElementAvecVerification(macro)) {
				  frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil103"));
				  frame.cadreActif().repaintFigureAvecCalcul();
				}
				frame.activeOutilCapture();	  
    	}
    	else {
      	frame.listeClignotante.add(elg);
      	frame.listeExclusionDesignation.add(elg);
      	sousListe.add(elg);
    	}
    }
  }
  
  @Override
	public void creeObjetApresClicDroit() {
		if (dialogueRempli) {
			if (sousListe.size() == 0) {
				frame.dialogueAvertissement(getString("ch88"));
				frame.activeOutilCapture();
			}
			else {
		    CListeObjets liste = frame.cadreActif().listePr;
	    	frame.indication(getString("macroAnimationAvecTrace2"));				
				ObjetsATracerDesignes = true;
				// On compte le nombre d'objets clignotants
				int nb = frame.listeClignotante.size();
				// On rajoute  la liste clignotante les points lis visibles dont dpend
				// au moins un des objets clignotants
				ListIterator<CElementBase> li = liste.listIterator();
				while (li.hasNext()) {
					CElementBase elb = li.next();
					if (elb.getNature() == NatureObjet.NPointLie) {
						CPointLie ptlie = (CPointLie) elb;
						if (!ptlie.masque && frame.listeClignotante.dependDe(ptlie)) 
							frame.listeClignotante.add(elb);
					}
				}
				if (frame.listeClignotante.size() == nb) {
					frame.dialogueAvertissement(getString("ch128"));
		    	annuleClignotement();
		    	frame.activeOutilCapture();
		    	return;
				}
	    	frame.outilPointageActif = frame.outilPointageObjetClignotant;
	    	frame.outilPointageActif.reset();
	    	frame.outilPointageActif.designationDe(NatureObjet.NPointLie);	
			}
		}
	}
  
  @Override
	public boolean activationValide() {
		return (frame.cadreActif().listePr.nombreObjetsParNatureVisibles(NatureObjet.NPointLie) != 0);
	}
	
	@Override
	public String messageActivationNonValide() {
		return getString("messagePasDePointLie");
	}	  
}
