/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Cursor;
import java.awt.Point;

import objets.CBarycentre;
import objets.CElementGraphique;
import objets.CTransformation;
import types.NatureObjet;

import dialogs.MtgDlg;

import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class OutilModifierObjetGraphique extends Outil {
	public CElementGraphique objetDesigne; // public car accessible par le dialogue d'historique

	/** Creates a new instance of OutilPtLib */
	public OutilModifierObjetGraphique(MtgFrame frameInit) {
		super(frameInit, true);
	}

  @Override
	public void select() {
		super.select();
		frame.outilPointageActif = frame.outilPointageModification;
		frame.outilPointageActif.designationDe(NatureObjet.NToutObjet);
		frame.indication(getString("modificationObjetGraphique1"));
	}
	
	@Override
  public Cursor curseurUtilise() {
		return MtgFrame.modifCursor;
	}
	

	@Override
	public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
		if (elg.getNature() == NatureObjet.NBarycentre) {
			annuleClignotement();
			deselect();
			frame.outilActif = frame.outilModifierBarycentre;
			frame.outilModifierBarycentre.select((CBarycentre) elg);
		}
		else {
			objetDesigne = elg;
	    ajouteClignotementDe(elg);
			elg.ajouteAntecedents(frame, frame.listeClignotante);
	    frame.cadreActif().videDesignation();
	    resetClignotement();
			creeObjet();
		}
	}

	@Override
	public void creeObjet() {
		MtgInternalFrame cadre = frame.cadreActif();
		if (objetDesigne.antecedentDirect().getNature() == NatureObjet.NTransformation) {
			// Cas o l'objet cliqu est l'image d'une autre objet par une transformation
			CTransformation trans = (CTransformation)objetDesigne.antecedentDirect();
			trans.modificationImage(frame, objetDesigne, true); // Modifi version 5.1.3
		}
		else {
			MtgDlg dlg = objetDesigne.dialogueModification(frame);
			dlg.setLocationRelativeTo(frame);
			dlg.setVisible(true);
			if (dlg.retourOK()) {
			  frame.cadreActif().gestionnaire.enregistreFigureEnCours(objetDesigne.informationModification());
			  frame.cadreActif().listePr.metAJourObjetsDependantDe(objetDesigne);
			  frame.cadreActif().repaintFigureAvecCalcul();
			}
		}
	  cadre.videListesPourConstructions();  // On remet  jour une construction ventuellement commence			  
		annuleClignotement();
		select();
	}
}
