/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;
import mathgraph32.MtgFrame;
import objets.CCercle;
import objets.CCercleClone;
import objets.CDemiDroite;
import objets.CDemiDroiteClone;
import objets.CDroiteAncetre;
import objets.CDroiteClone;
import objets.CElementGraphique;
import objets.CPointAncetre;
import objets.CPointClone;
import objets.CSegment;
import objets.CSegmentClone;
import types.NatureObjet;

/**
 * Ajout version 4.8
 * @author Yves
 *
 */

public class OutilObjetClone extends Outil {

  private CElementGraphique element;

  /** Creates a new instance of OutilPtLie */
  public OutilObjetClone(MtgFrame frameInit) {
    super(frameInit, false);
  }
  
  @Override
  public void select() {
    super.select();
    element = null;
    frame.outilPointageCre.designationDe(NatureObjet.NToutPoint | NatureObjet.NDroite | NatureObjet.NSegment | NatureObjet.NDemiDroite | NatureObjet.NCercle);
    frame.outilPointageActif = frame.outilPointageCre;
    frame.indication(getString("objetClone1"));
  }
  
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    if (element == null) {
      element = elg;
      creeObjet();
    }
  }
  
  @Override
  public void creeObjet() {
  	CElementGraphique elg = null;
  	if (element.estDeNature(NatureObjet.NToutPoint))
  		elg = new CPointClone(frame.cadreActif().listePr, null, false, frame.couleurActive(),
      		false, 0, 3, false, "", frame.pref_indiceTaillePoliceNom, frame.stylePointActif, false, (CPointAncetre) element);
  	else 
    if (element.estDeNature(NatureObjet.NDroite))
    	elg = new CDroiteClone(frame.cadreActif().listePr, null, false, frame.couleurActive(),
        		false, frame.styleTraitActif(), (CDroiteAncetre) element);
    else 
	  if (element.estDeNature(NatureObjet.NSegment))
	  	elg = new CSegmentClone(frame.cadreActif().listePr, null, false, frame.couleurActive(),
	        		false, frame.styleTraitActif(), (CSegment) element);
    else 
	  if (element.estDeNature(NatureObjet.NDemiDroite))
	  	elg = new CDemiDroiteClone(frame.cadreActif().listePr, null, false, frame.couleurActive(),
	        		false, frame.styleTraitActif(), (CDemiDroite) element);
    else 
	  if (element.estDeNature(NatureObjet.NCercle))
	  	elg = new CCercleClone(frame.cadreActif().listePr, null, false, frame.couleurActive(),
	        		false, frame.styleTraitActif(), (CCercle) element);
  	if (frame.ajouteElementAvecVerification(elg))
      frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil162"));
    annuleClignotement();
    frame.activeOutilCapture();
  }
}
