/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Toolkit;
import objets.CListeObjets;
import objets.CMacro;
import objets.CMacroApparition;
import types.InfoProx;
import types.NatureObjet;

import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class OutilPointageExecutionMacro extends OutilPointage {
  /** Creates a new instance of OutilPointageDecalageFigure */
  public OutilPointageExecutionMacro(MtgFrame frameInit) {
    super(frameInit);
  }

  @Override
  public void mouseMoved(java.awt.event.MouseEvent mouseEvent, MtgInternalFrame cadre) {
  	if (cadre.macroEnCours != null) return;
    CListeObjets liste = cadre.listePr;
    InfoProx info = frame.infoProx;
    int nbObjetsProches = liste.procheDe(aDesigner, mouseEvent.getPoint(), info,
      frame.listeExclusionDesignation, true);
    boolean designationDejaAffichee = (cadre.commentaireDesignation().chaineCommentaire.length() != 0);
      // true si une dsignation est dj affiche prs du pointeur
    if (nbObjetsProches > 0) {
      setActiveCursor(MtgFrame.macroCursor);
      cadre.commentaireDesignation().donneChaine(getString("ch106"));
      cadre.commentaireDesignation().placeEn(0, 0);
      cadre.paneFigure.repaint();
    }
    else {
      setActiveCursor(MtgFrame.defaultCursor);
      cadre.videDesignation();
      // if ((frame.macroEnCours == null) && designationDejaAffichee) frame.repaintFigure();
      if (designationDejaAffichee) cadre.paneFigure.repaint();
    }
  }
  @Override
  public void mousePressed(java.awt.event.MouseEvent mouseEvent, MtgInternalFrame cadre) {
    CListeObjets liste = cadre.listePr;
    InfoProx info = frame.infoProx;
    int nbObjetsProches = liste.procheDe(aDesigner, mouseEvent.getPoint(), info,
        frame.listeExclusionDesignation, true);
    if (nbObjetsProches > 0) {
      CMacro macroCliquee = (CMacro) info.infoParType[NatureObjet.indiceMacro].premierVoisin;
      // Ligne suivante modifie le 17-12-08
      if (cadre.macroEnCours == null) { // Si aucune macro n'est dj en cours d'excution
                                  // on lance l'excution de la macro clique si possible

        if (macroCliquee.executionPossible()) {
        	/* Modifi version 5.1
          cadre.macroEnCours = macroCliquee;
          */
        	// Ajout version 5.1
          macroCliquee.setMacroLanceuse(null);
          macroCliquee.initialisePourLancement(cadre);
          //
        	cadre.macroEnCours = macroCliquee.macroEnCours(); // Pour en cas d'une macro d'excution d'une suite de macros ce soit la premire macrsro de la liste qui soit excute
          // if (macroCliquee.typeAnimation() == CMacro.animationParThread) { Modifi version 5.1
        	if (cadre.macroEnCours.typeAnimation() == CMacro.animationParThread) {
          	cadre.videDesignation();
          	cadre.paneFigure.repaint();
          }
          cadre.macroEnCours.initialisePourLancement(cadre);
          // cadre.macroEnCours.macroEnCours().actionAvantExecution(cadre); Modifi version 5.1
          cadre.macroEnCours.actionAvantExecution(cadre);
          // Si la macro utilise un thread, il faut le crer.
          //  revoir
          // cadre.macroEnCours.macroEnCours().initialiseInterruptions(cadre); Modifi version 5.1
          cadre.macroEnCours.initialiseInterruptions(cadre);
            //macroEnCours.execute(this, infRand, dimf);
          // if (cadre.macroEnCours.macroEnCours().typeAnimation() == CMacro.sansAnimation) { Modifi version 5.1
          if (cadre.macroEnCours.typeAnimation() == CMacro.sansAnimation) {
          	//  revoir
          	// cadre.macroEnCours.macroEnCours().actionDansPaint(cadre);
            cadre.paneFigure.repaint();
           // Ligne suivante rajoute pour la version 1.9.5
            /* Supprim version 5.1
            if (cadre.macroEnCours.macroEnCours() == cadre.macroEnCours) {
              if (!cadre.macroEnCours.executionEnCours) cadre.macroEnCours = null;
            }
            */
            //cadre.paneFigure.repaint();
          }
          cadre.videDesignation();
          setActiveCursor(MtgFrame.defaultCursor);
        }
        else Toolkit.getDefaultToolkit().beep();
      }
      else {
        // CMacro mac = cadre.macroEnCours.macroEnCours(); Modifi version 5.1
      	CMacro mac = cadre.macroEnCours;
        // Si la macro en cours d'excution s'excute par un thread et si elle n'a pas fini de s'excuter
        // on n'active pas une autre macro
        if ((mac.typeAnimation() == CMacro.animationParThread) && mac.executionEnCours) return;
        // Si la macro en cours est un CMacroSuiteMacros on la fait passer  la macro suivante
        /* Modifi version 5.1
        if (cadre.macroEnCours instanceof CMacroSuiteMacros) {
          // if (mac.dureePourAnimation() == 0) {
        	if (mac.arretParClic()) {
            mac.termineAction(cadre);
            ((CMacroSuiteMacros) cadre.macroEnCours).passageMacroSuiv(cadre);
          }
        }
        */
        /* Modifi version 5.1
        if (cadre.macroEnCours instanceof CMacroSuiteMacros) {
          // if (mac.dureePourAnimation() == 0) {
        	if (mac.arretParClic()) {
            mac.termineAction(cadre);
            ((CMacroSuiteMacros) cadre.macroEnCours).passageMacroSuiv(cadre);
          }
        }
        */
      	if (mac.arretParClic()) mac.termineAction(cadre);
        else {
			    if ((mac instanceof CMacroApparition) && mac.executionEnCours) {
			      // mac.actionDansPaint(cadre);
			      cadre.paneFigure.repaint();
			      return;
			    }
          if (macroCliquee == cadre.macroEnCours) {
            mac.termineAction(cadre);
            if (mac.typeAnimation() == CMacro.sansAnimation) {
            	mac.setMacroLanceuse(null);
              mac.actionAvantExecution(cadre);
              cadre.paneFigure.repaint();
            }
            else {
              mac.termineAction(cadre);
              cadre.macroEnCours = null;
              // System.gc(); Supprim version 5.0.0.5
            }           
          }
          else {
          	cadre.macroEnCours.termineAction(cadre);
            if (macroCliquee.executionPossible()) {
            	// Ajout version 5.1
              macroCliquee.setMacroLanceuse(null);
              macroCliquee.initialisePourLancement(null);
              //
              /* Modifi version 5.1
            	cadre.macroEnCours = macroCliquee;
            	macroCliquee.setMacroLanceuse(null);
            	*/
            	cadre.macroEnCours = macroCliquee.macroEnCours();
            	// macroCliquee.setMacroLanceuse(null);
            	cadre.macroEnCours.initialisePourLancement(cadre);
            	cadre.macroEnCours.actionAvantExecution(cadre);
              // Si la macro utilise un thread, il faut le crer.
            	cadre.macroEnCours.initialiseInterruptions(cadre);

                //macroEnCours.execute(this, infRand, dimf);
              if (mac.typeAnimation() == CMacro.sansAnimation) cadre.paneFigure.repaint();
            }
            else Toolkit.getDefaultToolkit().beep();
          }
        }
      }
    }
    else {
      // Si aucun objet n'est proche du pointeur souris, et si une macro est en cours
      // d'excution, on annule son excution
			if (cadre.macroEnCours != null) {
				/* Modifi version 5.1
			  if (cadre.macroEnCours instanceof CMacroSuiteMacros) {
			    if (cadre.macroEnCours.macroEnCours().arretParClic()) {
			    	cadre.macroEnCours.macroEnCours().termineAction(cadre);
			      ((CMacroSuiteMacros) cadre.macroEnCours).passageMacroSuiv(cadre);
			    }
			    // passageMacroSuiv redonne ventuellement un ordre paint() suivant la nature de la macro suivante
			  }
			  */
		    if (cadre.macroEnCours.arretParClic()) cadre.macroEnCours.termineAction(cadre); // Ajout version 5.1

			  // Modifi version 4.9.7.1 pour une macro d'apparition avec clic pir objet suivant incorpore dan une macro
			  // d'excution d'une suite de macros.
			  //else {
		    // Si la macro en cours est une macro d'apparition avec clic pour objet suivant
		    // on ne la dsactive que si on est au dernier objet
		    // CMacro mac = cadre.macroEnCours.macroEnCours(); Modifi version 5.1
		    CMacro mac = cadre.macroEnCours;
		    if ((mac instanceof CMacroApparition) && mac.executionEnCours) {
		      // mac.actionDansPaint(cadre);
		      cadre.paneFigure.repaint();
		    }
		    /* Supprim version 5.1
		    else {
		    	if (mac.arretParClic()) {
		    		mac.termineAction(cadre);
		    		cadre.macroEnCours = null;
		    	}
		    }
		    */
		    // System.gc(); Supprim version 5.0.0.5
		    // On peut arriver ici alors qu'on a utilis l'outil mouvoir animer un point li.
		    // A la fin de l'animation il faut reveneir  l'outil par dfaut de capture
		    if (frame.outilActif == frame.outilMouvoirAnimationPointLie) frame.activeOutilCapture();
		  //}
			}    	
    }
  }
}
