/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CPointAncetre;
import objets.CPointBase;
import objets.CPointLie;
import types.NatureObjet;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class OutilSuppressionLiaison extends Outil {
	private CPointAncetre ptlie;
	public OutilSuppressionLiaison(MtgFrame frameInit) {
		super(frameInit, true);
	}

  @Override
	public void select() {
		super.select();
		ptlie = null;
		CListeObjets liste = frame.cadreActif().listePr;
		// Modification version 4.7.1 : On n'ajoute pas les objets finaux de construction
		/*
		frame.listeClignotante.ajouteObjetsParNature(liste, NatureObjet.NPointLie | NatureObjet.NPointLiePoint | NatureObjet.NPointInterieurPolygone
				| NatureObjet.NPointInterieurCercle);
		*/
		frame.listeClignotante.ajouteObjetsParNatureNonFinaux(liste, NatureObjet.NPointLie | NatureObjet.NPointLiePoint | NatureObjet.NPointInterieurPolygone
				| NatureObjet.NPointInterieurCercle);
		
		frame.outilPointageActif = frame.outilPointageObjetClignotant;
		frame.outilPointageActif.designationDe(NatureObjet.NPointLie | NatureObjet.NPointLiePoint | NatureObjet.NPointInterieurPolygone
				| NatureObjet.NPointInterieurCercle);
		frame.indication(getString("suppressionLiaison1"));
		resetClignotement();
	}
	
	public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
		CListeObjets liste = frame.cadreActif().listePr;
    if (ptlie == null) {
      ptlie = (CPointAncetre) elg;
      if (ptlie.getNature() == NatureObjet.NPointLie) {
	      // Ajout version 3.2.2 : on refuse de supprimer une liaison quand l'objet dsigne est un objet final de construction
	      // On conseille alors de fusionner les constructions de la figure
	      if (ptlie.estElementFinal){
	      	frame.dialogueAvertissement(getString("ch172")+ "\n" + getString("ch173"));
	      	select();
	      	return;
	      }
	      if (liste.contientObjetGenereParPointLie((CPointLie)ptlie)) {
	      	frame.dialogueAvertissement(getString("ch121"));
	      	ptlie = null;
	      	annuleClignotement();
	      	return;
	      }
      }
      annuleClignotement();
      creeObjet();
    }
	}
	
	// Modifi version 4.8
  @Override
	public void creeObjet() {
		MtgInternalFrame cadre = frame.cadreActif();
		CListeObjets liste = cadre.listePr;
		CPointBase pbase = new CPointBase(liste, ptlie.impProto, ptlie.estElementFinal, ptlie.rendCouleur(),
				ptlie.nomMasque, ptlie.dec_x, ptlie.dec_y, ptlie.masque, ptlie.rendNom(), ptlie.indiceTailleNom,
				ptlie.motif, true, ptlie.x, ptlie.y, ptlie.marquePourTrace);
		liste.remplacePoint(ptlie, pbase);
		
		frame.remplaceElement(ptlie, pbase); // Positionne la liste
    // Ajout version 4.8.0 : IL faut remettre  jour les index de cahque lment de la liste (CElementBase.index)
    liste.updateIndexes();
    //
    cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil96"));
    cadre.repaintFigureAvecCalcul();
    frame.activeOutilCapture();
    cadre.videListesPourConstructions();  // On remet  jour une construction ventuellement commence
    // liste.initialiseDependances(); // Ajout version 4.8 Supprim version 4.9.2. Fait par GestionnaireEdition.sauvegardeFigureEnCours()
	}
	
  @Override
  public boolean activationValide() {
  	CListeObjets liste = frame.cadreActif().listePr;
    return (liste.nombreObjetsParNature(NatureObjet.NPointLie | NatureObjet.NPointLiePoint | NatureObjet.NPointInterieurPolygone
    		| NatureObjet.NPointInterieurCercle) != 0);
  }
  
	@Override
	public String messageActivationNonValide() {
		return getString("messagePasDePointLie");
	}	
	
	
}
